package cox.jmatt.java.MathTools.demo;

import java.awt.*;
import java.awt.event.*;
import cox.jmatt.java.MathTools.*;

/** <p>This class showcases the powerful <code>MathFileIO.processDirectory()</code> method. Every feature available from that method can be accessed from the 
    GUI here. The resulting array is amalgamated into a single newline-delimited String and printed on the console. If the 'processMathObject(result)' Checkbox 
    is checked, the results are sent to the clipboard.</p>
    <p>The top TextField selects the directory to process. Pick a file in that directory and the directory will be extracted from it. The next three lines 
    enable/disable prefix, suffix and regex matching and provide TextFields for the filtering data. In order to activate a particular type of filtering, its 
    Checkbox MUST be enabled!</p>
    <p>Output is formatted by the 'Template' line. If the Checkbox is off, a default format is used. If the 'Template' TextArea is blank, the default format is 
    used. Unless altered, this template provides the complete path to the files, including the filename itself. This allows fine-grained control both of selection 
    and of formatting.</p>
*/

public class DirectoryProcessorDemo extends Panel implements ActionListener {

  private static final char CHOOSE_FILE = 'C';
  private static final char PROCESS_DIR = 'P';

  private FileDialog fdFilePicker = null;
  private Checkbox cbxStartsWith = null, cbxEndsWith = null, cbxRegex = null, cbxTemplate = null, cbxRecursive = null, cbxClip = null;
  private TextField txfSourceFile = null, txfStartsWith = null, txfEndsWith = null, txfRegex = null,txfTemplate = null;

  public DirectoryProcessorDemo(Frame pParent) {
    super();
    createGUI(pParent);
  }

  private final void createGUI(Frame pParent) {
    if(pParent != null) fdFilePicker = new FileDialog(pParent, "Choose source file", FileDialog.LOAD);
    Button btnTemp = null;
    setLayout(new GridBagLayout());
    GridBagConstraints gbc = new GridBagConstraints(); gbc.insets = new Insets(1,1,1,1);
     gbc.gridx=1; gbc.gridy=1; gbc.gridwidth = GridBagConstraints.REMAINDER;
    add(new Label("MathFileIO Directory Processor Demo", Label.CENTER), gbc); gbc.gridy++; gbc.gridwidth=1; gbc.anchor = GridBagConstraints.WEST;
    add(new Label("Source"), gbc); gbc.gridx++;
    txfSourceFile = new TextField(30); add(txfSourceFile, gbc); gbc.gridx++;
    btnTemp = new Button("Choose"); btnTemp.setActionCommand(""+CHOOSE_FILE); add(btnTemp, gbc); btnTemp.addActionListener(this);
     gbc.gridx=2; gbc.gridy++;
    cbxRecursive = new Checkbox("Recursive"); add(cbxRecursive, gbc); gbc.gridx = 1; gbc.gridy++;
    cbxStartsWith = new Checkbox("Starts with:"); add(cbxStartsWith, gbc); gbc.gridx++;
    txfStartsWith = new TextField(30); add(txfStartsWith, gbc); gbc.gridx=1; gbc.gridy++;
    cbxEndsWith = new Checkbox("Ends with:"); add(cbxEndsWith, gbc); gbc.gridx++;
    txfEndsWith = new TextField(30); add(txfEndsWith, gbc); gbc.gridx=1; gbc.gridy++;
    cbxRegex = new Checkbox("Regex:"); add(cbxRegex, gbc); gbc.gridx++;
    txfRegex = new TextField(30); add(txfRegex, gbc); gbc.gridx=1; gbc.gridy++;
    cbxTemplate = new Checkbox("Template:", true); add(cbxTemplate, gbc); gbc.gridx++;
    txfTemplate = new TextField(30); add(txfTemplate, gbc); gbc.gridx=1; gbc.gridy++;
     gbc.gridx=1;
    btnTemp = new Button("Process!"); btnTemp.setActionCommand(""+PROCESS_DIR); add(btnTemp, gbc); btnTemp.addActionListener(this); gbc.gridx++;
    cbxClip = new Checkbox("processMathObject(result)", true); add(cbxClip, gbc);
     gbc.gridx=1; gbc.gridy++;
  }

  public void actionPerformed(ActionEvent aev) {
    String sCmd = aev.getActionCommand()+"  ";
    switch(sCmd.charAt(0)) {
      case CHOOSE_FILE:
        chooseSourceFile();
        break;
      case PROCESS_DIR:
        processDirectory();
        break;
      case 'D':
        
        break;
      default:
    }
  }

  private void chooseSourceFile() {
    if(fdFilePicker == null) {
      CapCom.println("No FileDialog present.");
      return;
    }
    fdFilePicker.setTitle("Choose source directory");
    fdFilePicker.setVisible(true);
    String sDir = fdFilePicker.getDirectory(), sFile = fdFilePicker.getFile();
    if(Question.validString(sDir)) txfSourceFile.setText(sDir);
  }

  private void processDirectory() {
    String sSource = txfSourceFile.getText().trim();
    if(!Question.validString(sSource)) {
      CapCom.println("No source file set.");
      return;
    }
    String sTemplate = cbxTemplate.getState()?txfTemplate.getText().trim():"";
    String sStart = cbxStartsWith.getState()?txfStartsWith.getText():null;
    String sEnd = cbxEndsWith.getState()?txfEndsWith.getText():null;
    String sRegex = cbxRegex.getState()?txfRegex.getText():null;
    String[] sLinez = cox.jmatt.java.MathTools.util.MathFileIO.processDirectory(sSource, sTemplate, cbxRecursive.getState(), sStart, sEnd, sRegex);
    StringBuffer sbResult = new StringBuffer();
    for(String sLine : sLinez) sbResult.append(sLine).append("\n");
    if(cbxClip.getState()) CapCom.processMathObject(sbResult.toString());
    CapCom.println(sbResult.toString());
  }

}
