package cox.jmatt.java.MathTools.demo;

import java.awt.*;
import java.awt.event.*;
import java.util.Properties;
import java.util.Hashtable;
import java.io.StringReader;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.demo.gpgdemo.*;
import cox.jmatt.java.MathTools.markup.svg.*;

/** <p>This class is a graph paper generator demo/utility. It uses SVG and the <code>svg.*</code> package to create various types of graph paper. The SVG documents 
    can then be transformed into PDF or other formats by an external utility like Apache <a href="http://batik.apache.org">Batik</a>. Each panel creates a different 
    type of graph paper and the output can be sent to the clipboard.</p>
    <p>All graph paper generated is by default for letter-sized paper (8.5&quot; x 11&quot;) with a viewport set to '850 1100'. This yields a scale of 100 units 
    per inch in both dimensions and that is how all spacing is measured. NOTE: Whenever a length or other absolute linear measure is used each unit is automatically 
    1/100 of the base unit! So, margin settings are 100 per inch. Default margins are Top 50, Bottom 1150, Left 50 and Right 800; this yields a half-inch margin 
    all the way around the paper.</p>
    <p>For advanced users all of these quantities can be changed. The units are all recognized by the SVG standard: inches, centimeters, millimeters, 'em', 'ex', 
    points, picas and pixels. The SVG 'viewBox=' attribute is calculated for 100 user units per inch: it is set to 100 * the (double) value in the height and 
    width TextFields. There is no safety checking so USE WITH CARE!!</p>
    <h3>Global Pens</h3>
    <p><code>GPGenerator</code> defines 31 built-in pens for creating graphs. Specific details are given in the <code>addPens()</code> method, which adds the 
    standard (global) pens to <i>Any</i> <code>SVGTag</code>. Pen 30 is the suppression pen; it has no stroke or fill set, so it is invisible. It can also be 
    hand-edited easily; it appears at the end of the CSS list.</p>
    <p>Speaking of CSS, <code>GPGenerator</code> creates a massive embedded stylesheet with each document. More pens than any one utility could <i>ever</i> use. 
    If this becomes a problem the unused pens (CSS rules) are easily deleted. The internal stylesheet is just that, <i>internal</i>!</p>
    <h3>Why no methods here?!?</h3>
    <p>Each type of graph paper is generated by its own dedicated {@link cox.jmatt.java.MathTools.demo.gpgdemo.GraphPanel GraphPanel}. Configuration methods are 
    there. All this class really does is manage them and prepare the <code>SVGTag</code> used for drawing.</p>
    <h3>Graph Paper and Scripting</h3>
    <p>Besides being good for generating graph paper the individual generators are also scriptable. The <code>demo.gpgdemo.GraphPanelFactory</code> class 
    ('Notebook' to scripts). facilitates this. When run from the JAR each generator in the GUI is the instance exposed to scripts. This means scripts can set 
    the GUI and GUI settings can affect script output. <code>GraphPanel</code> defines two complementary methods <code>toPizza()</code> and <code>eatPizza()</code>. 
    The former turns the current GUI settings for a generator into <code>java.util.Properties</code>-formatted lines of text. The former takes a <code>Properties</code> 
    object and extracts configuration information from it. Panel 'B' in this demo provides an area to put such information.</p>
    <p>Each generator has a '[To Pizza]' button, an '[Eat Pizza]' button and a text field for key prefixes. When the text output from a generator is placed into 
    the pizza area and its '[Eat Pizza]' button clicked it will configure from the data there. The prefix, if provided, is prepended to the property key. This 
    allows multiple configurations to be stored in a single file or placed into the pizza area. Even though it is not a <code>GraphPanel</code> the margin and 
    paper size Panel can also create and consume a pizza.</p>

    <h2>Available Graph Paper</h2>
    <p>Each graph paper generator is independent of all the others. The only thing they really share is the pens and those are output with each document. The 
    default values listed on each panel give 'pretty' graph paper but by all means experiment!</p>
    <h3>A. The Configuration Panel</h3>
    <p>This class actually has very little to configure. The margins can be set but should be set carefully. Some sanity-checking is done but carelessness with 
    the margins may result in unparseable or invalid documents. As with other quantities the units are 1/100 of an inch.</p>
    <h3>B. Graph Paper Pizzaria</h3>
    <p>This panel is for pizza configuration information for the other generators and the Config Panel. Pizzas placed here can be used to configure the other 
    panels.</p>
    <p>Clicking the 'B' panel '[Eat Pizza]' button does parse the Properties object BUT it uses it to set default config properties for the <code>getMDraw()</code> 
    method. This means any tag- or pen-class pizza-settable properties can be specified here. All graph paper generators here use this method to get their 
    <code>MathDrawSVG</code> instances so any properties 'eaten' from this panel go there.</p>
    <h3>C. Square-Grid (Quadrille) Paper</h3>
    <p>This panel generates plain vanilla square-grid paper. Details are {@link cox.jmatt.java.MathTools.demo.gpgdemo.QuadrillePanel here}.</p>
    <h3>D. Polar Paper</h3>
    <p>This panel generates polar-grid paper. Details are {@link cox.jmatt.java.MathTools.demo.gpgdemo.PolarPanel here}.</p>
    <h3>E. Log and Semilog Paper</h3>
    <p>This panel generates logarithmic and semilog paper. Details are {@link cox.jmatt.java.MathTools.demo.gpgdemo.LogPanel here}.</p>
    <p>The default settings have a logarithmic vertical scale (horizontal lines) and a quarter-inch linear horizontal scale (vertical lines). For a decent 
    horizontal log scale use 4 cycles at length 10 with an interval of 55.</p>
    <h3>F. Multicolumn or Ledger Paper</h3>
    <p>This generator produces multi-column paper with optional headers and row lines.  See {@link cox.jmatt.java.MathTools.demo.gpgdemo.LedgerPanel here}.</p>
    <h3>G. BPS Chart</h3>
    <p>BP/S stands for 'Blood Pressure / Sugar' and creates a {@link cox.jmatt.java.MathTools.demo.gpgdemo.BPSPanel chart} for tracking blood pressure or blood 
    sugar (or both!). </p>
    <h3>I. Hexagonal-Grid Paper</h3>
    <p> {@link cox.jmatt.java.MathTools.demo.gpgdemo.HexGridPanel This} is a quick and dirty generator for hex-grid paper. The size and number of hexes can be 
    adjusted and the rows and columns numbered.</p>
    <h3>I. Bar Chart</h3>
    <p>This generator generates bar charts. Check {@link cox.jmatt.java.MathTools.demo.gpgdemo.BarChartPanel here} for details.</p>
    <h3>J. Musical Notation Paper</h3>
    <p><code>MusicPanel</code> provides a simple, flexible generator for {@link cox.jmatt.java.MathTools.demo.gpgdemo.MusicPanel music paper}.</p>
    <h3>K. Lettering Guide Paper</h3>
    <p>This panel generates configurable-line paper usable to design {@link cox.jmatt.java.MathTools.demo.gpgdemo.LetteringPanel lettering guides} children use 
    when learning to write.</p>
    <h3>L. Offset Quadrille Paper</h3>
    <p>{@link cox.jmatt.java.MathTools.demo.gpgdemo.OffsetQuadPanel This} generator produces square- or rectangular-grid paper with alternate rows offset, 
    resuilting in a brick-like grid.</p>
    <h3>M. Symmetric Log Paper</h3>
    <p>{@link cox.jmatt.java.MathTools.demo.gpgdemo.SymmetricLogPanel This} is a modified type of logarithmic paper. The intervals begin at the axis and shrink 
    as they move away from it.</p>
    <h3>N. Isometric Paper</h3>
    <p>{@link cox.jmatt.java.MathTools.demo.gpgdemo.IsometricPanel Isometric} paper consists of a grid of equilateral triangles, which is what this generator 
    creates.</p>
    <p>The {@link cox.jmatt.java.MathTools.demo.gpgdemo.PerspectivePanel perspective} generator produces a one-point perspective grid. Both horizontal and 
    vertical perspective can be configured independently.</p>
*/

public class GPGenerator extends Panel implements ActionListener {

  /** Static constant for the number of the last defined pen. */
  public static final int MAX_PEN = 30;

  /** <p>This constant is defined to assist in developing the various graph paper generators. It is settable via the '[]GP Debug' Checkbox on the margin panel here,
      <code>GraphPanel</code> and <code>GraphPanelFactory</code> Its only effect here is to draw a border along the margins of the <code>SVGTag</code> delivered 
      to the panels when in GUI mode. The value is set any time the '[Go!]' button is clicked.</p>
  */
  public static boolean GP_DEBUG = false;

  /** This constant holds the name of the &lt;clipPath&gt; defined around the margins of the paper. This tag is not actually applied, it is defined in case it 
      is needed and its ID is here. This String does NOT start with the '#' character!
  */
  public static final String GPG_CLIP = "GPGMarginClip";

  private static final String BTN_NEXT = "|>>";
  private static final String BTN_PREV = "<<|";
  private static final String KEY_RESET = "RESET_";
  private static final String KEY_TOPIZZA = "TOPIZZA_";
  private static final String KEY_EATPIZZA = "EATPIZZA_";
  private static final String KEY_MARGINS = "A. Margin Configuration (Graph Paper Generator):Config";
  private static final String KEY_PIZZA = "B. Graph Paper Pizzaria:Pizza";
  private static final String KEY_QUAD = "C. Linear Square-Grid (Quadrille) Paper:Quadrille";
  private static final String KEY_POLAR = "D. Polar Graph Paper:Polar";
  private static final String KEY_LOG = "E. Semilog and Log Paper:Log";
  private static final String KEY_LEDGER = "F. Multicolumn / Ledger Paper:Ledger";
  private static final String KEY_BPS = "G. BP/S Chart:BP/S";
  private static final String KEY_HEX = "H. Hexagonal Graph Paper:Hex";
  private static final String KEY_BARCHART = "I. Bar Chart:Bar Chart";
  private static final String KEY_MUSIC = "J. Music Paper:Music";
  private static final String KEY_LETTERS = "K. Lettering Guide Paper:Lettering";
  private static final String KEY_BRICKS = "L. Offset Quadrille Paper:Bricks";
  private static final String KEY_SYMLOG = "M. Symmetric Log Paper:Sym. Log";
  private static final String KEY_ISO = "N. Isomatric Paper:Isometric";
  private static final String KEY_PERSPECTIVE = "O. Perspective Paper:Perspective";

  private static final String[] SVG_UNITS = "in cm mm pt pc em ex px".split(" ");

  private static MathDrawSVG myMDraw = null;
  private static Properties myPizza = null;
  private static String myPrefix = null;

  private CardLayout cdlMain = new CardLayout();
  private Checkbox cbxClip = new Checkbox("process(result)", true), cbxGPDebug = new Checkbox("GP Debug");
  private TextField txfMarginTop = new TextField("50"), txfMarginBottom = new TextField("1050"), txfMarginLeft = new TextField("50"), txfMarginRight = new TextField("800");
  private TextField txfPaperHeight = new TextField("11"), txfPaperWidth = new TextField("8.5");
  private Choice chcWidth = new Choice(), chcHeight = new Choice();
  private TextArea txaPizza = new TextArea();
  private Menu gpMenu = new Menu("Graph Paper");

  private int marginTop = 50, marginLeft = 50, marginBottom = 1050, marginRight = 800;
  private Hashtable<String, TextField> htbPrefixes = new Hashtable<String, TextField>();
  private Hashtable<String, GraphPanel> htbPanels = new Hashtable<String, GraphPanel>();

  /** Standard constructor. Nothing else needed. */
  public GPGenerator() {
    super();
    cox.jmatt.java.MathTools.demo.gpgdemo.GPickle.gpgMain = this;
  }

  /** <p>This method is used in conjunction with the margin clipPath tag that is automatically defined. It is ONLY defined within this class, so this is of 
      most use when subclassing. The <code>StylePen</code> returned has NO attributes set other than the clip URL, which refers to the clipPath tag.</p?
  */
  public static StylePen getClipPen() {
    StylePen retval = new StylePen();
    retval.setClipURL("#"+GPG_CLIP);
    return retval;
  }

  /** This class maintains ONE <code>MathDrawSVG</code> instance. This is done to allow pre-configuration if needed. This method maintains it. Clicking the '[Reset]' 
      button on the Pizzaria panel clears the current instance and forces a new one the next time this method is called.
  */
  public static MathDrawSVG getMDraw() {
    if(myMDraw == null) {
      myMDraw = new MathDrawSVG();
      myMDraw.setConfigPizza(myPrefix, myPizza);
    }
    return myMDraw;
  }

  /** This is the menu that appears in the Demo Frame whenever 'View/Graph Paper' is selected. It is normally disabled but specifically selecting the 'Graph Paper' 
      option from the 'View' menu OR clicking any button on a Graph Paper Generator panel enables it.
  */
  public Menu getMenu() { return gpMenu; }

  /** Handle all button clicks. */
  public void actionPerformed(ActionEvent aev) {
    gpMenu.setEnabled(true);
    String sCmd = aev.getActionCommand();
    if(aev.getSource() instanceof MenuItem) {
      cdlMain.show(this, sCmd);
    } else {
      if(BTN_NEXT.equals(sCmd)) {
        cdlMain.next(this);
      } else if(BTN_PREV.equals(sCmd)) {
        cdlMain.previous(this);
      } else if(sCmd.startsWith(KEY_RESET)) {
        reset(sCmd.substring(KEY_RESET.length()));
      } else if(sCmd.startsWith(KEY_TOPIZZA)) {
        makePizza(sCmd.substring(KEY_TOPIZZA.length()));
      } else if(sCmd.startsWith(KEY_EATPIZZA)) {
        consumePizza(sCmd.substring(KEY_EATPIZZA.length()));
      } else {
        drawPaper(sCmd);
      }
    }
  }

  /* Do the actual drawing. */
  private final void drawPaper(String pCmd) {
    if(!Question.validString(pCmd)) return;
    char cType = pCmd.charAt(0);
    CapCom.debug("drawPaper: "+pCmd);
    GP_DEBUG = cbxGPDebug.getState();
    MathDrawSVG MDraw = getMDraw();
    SVGTag graphPaper = MDraw.getSVGTag((txfPaperWidth.getText()+chcWidth.getSelectedItem()).trim(), (txfPaperHeight.getText()+chcHeight.getSelectedItem()).trim());
    String viewPort = "0 0 "+((int) (asDouble(txfPaperWidth, 8.5) * 100))+" "+((int) (asDouble(txfPaperHeight, 11)*100));
    graphPaper.setViewBox(viewPort);
    addPens(graphPaper);
    marginTop = asInt(txfMarginTop, 50); marginLeft = asInt(txfMarginLeft, 50);
    marginRight = asInt(txfMarginRight, 800); marginBottom = asInt(txfMarginBottom, 1050);
    if(GP_DEBUG) {
      PolygonTag borderz = MDraw.getPolygonTag();
      borderz.setPen(0);
      borderz.setID("GPGDebug_"+CapCom.hitCount());
      borderz.addVertex(marginLeft, marginTop).addVertex(marginRight, marginTop).addVertex(marginRight, marginBottom).addVertex(marginLeft, marginBottom);
      graphPaper.add(borderz);
    }
    ClipTag marginClip = MDraw.getClipTag(GPG_CLIP);
    marginClip.add(MDraw.getPolygonTag().addVertex(marginLeft, marginTop).addVertex(marginRight, marginTop).addVertex(marginRight, marginBottom).addVertex(marginLeft, marginBottom));
    graphPaper.add(marginClip);

    if(htbPanels.containsKey(pCmd)) {
      htbPanels.get(pCmd).drawGraph(marginTop, marginBottom, marginLeft, marginRight, graphPaper);
      String sResult = graphPaper.toString();
      if(cbxClip.getState()) CapCom.processMathObject(sResult);
      CapCom.println(sResult);
    } else {
      switch(cType) {
      case 'A': // Set margins
        CapCom.println("- - Printing current margins - -\nTop:\t"+marginTop+"\nLeft:\t"+marginLeft+"\nBottom:\t"+marginBottom+"\nRight:\t"+marginRight);
        CapCom.println("processMathObject(result) is "+(cbxClip.getState()?"enabled.":"disabled."));
        CapCom.println("Paper size: "+txfPaperWidth.getText()+chcWidth.getSelectedItem()+" w. by "+txfPaperHeight.getText()+chcHeight.getSelectedItem()+" h.");
        CapCom.println("ViewBox: "+viewPort);
        CapCom.println("GP Debug Mode is: "+(GP_DEBUG?"on":"off."));
        break;
      case 'B':
        CapCom.println("The '[Go!]' button triggers graph paper generation in the generator panels.\nThe margin config panel prints out its config information.");
        CapCom.println("Checking the '[]process...' and '[] Clip' boxes puts SVG output in the clipboard.");
        CapCom.println("The '[Reset]' button restores default settings to the generator panels.\nResetting clears the pizza area.");
        break;
      default:
        CapCom.println("Unknown command: "+pCmd);
      }
    }    
  }

  /* Reset a panel. */
  private void reset(String pCmd) {
    if(htbPrefixes.containsKey(pCmd)) htbPrefixes.get(pCmd).setText("");
    if(htbPanels.containsKey(pCmd)) {
      htbPanels.get(pCmd).reset();
    } else {
      switch(pCmd.charAt(0)) {
      case 'A':
        CapCom.debug("Reset: margins.");
        txfMarginTop.setText("50"); txfMarginBottom.setText("1050"); txfMarginLeft.setText("50"); txfMarginRight.setText("800");
        txfPaperWidth.setText("8.5"); txfPaperHeight.setText("11"); chcWidth.select(0); chcHeight.select(0);
        break;
      case 'B':
        CapCom.debug("Clear pizza area & reset config pizza.");
        txaPizza.setText("");
        myPizza = null;
        myPrefix = null;
        CapCom.println("Config pizza cleared.\nPrefix cleared.");
        break;
      default:
        CapCom.debug("Unknown panel: "+pCmd);
      }
    }
  }

  private void makePizza(String pCmd) {
    CapCom.debug("toPizza(): "+pCmd);
    if(!Question.validString(pCmd)) return;
    TextField txfTemp = htbPrefixes.get(pCmd);
    String retval = "";
    if(txfTemp != null) retval = txfTemp.getText();
    CapCom.debug("Prefix: "+retval);
    if(htbPanels.containsKey(pCmd)) {
      retval = htbPanels.get(pCmd).toPizza(retval);
    } else {
      switch(pCmd.charAt(0)) {
      case 'A':
        retval = toPizza(retval);
        break;
      case 'B':
        CapCom.println("The '[To Pizza]' button causes a generator to create Properties-style config data.\nThis is copied to the clipboard.");
        CapCom.println("This output can then be saved in a text file or put into the pizza area.");
        CapCom.println("The key-prefix TextField applies to this option and '[Eat Pizza]'\nPrefixes are prepended to property keys.");
        CapCom.println("Clicking '[Eat Pizza]' pulls config information from here into a generator.\nThe config information expected is per the 'toPizza' method");
        CapCom.println("or the '[To Pizza]' button.");
        break;
      default:
        CapCom.debug("Unknown panel: "+pCmd);
      }

    }
    CapCom.println(retval);
    if(cbxClip.getState()) CapCom.processMathObject(retval);
  }
  
  private void consumePizza(String pCmd) {
    CapCom.debug("eatPizza(): "+pCmd);
    if(!Question.validString(pCmd)) return;
    TextField txfTemp = htbPrefixes.get(pCmd);
    if(txfTemp == null) return;
    String sPrefix = txfTemp.getText();
    CapCom.debug("Prefix: "+sPrefix);
    Properties thePizza = new Properties();
    try {
      StringReader srIn = new StringReader(txaPizza.getText());
      thePizza.load(srIn);
      srIn.close();
    } catch(Exception ex) { CapCom.error("Error reading margin pizza", ex); }
    
    if(htbPanels.containsKey(pCmd)) {
        (htbPanels.get(pCmd)).eatPizza(thePizza, sPrefix);
    } else {
      switch(pCmd.charAt(0)) {
      case 'A':
        eatPizza(thePizza, sPrefix);
        break;
      case 'B':
        myPizza = thePizza;
        myPrefix = sPrefix;
        CapCom.println("Prefix set to: \""+sPrefix+"\"\nProperties loaded: "+thePizza.size());
        break;
      default:
        CapCom.debug("Unknown panel: "+pCmd);
      }
    }
    
    
  }

  /** <p>Add the standard pens to the SVGTag supplied. The detailed descriptions are here.
        <dl>
          <dt><b>Pen_0</b> to <b>Pen_4</b>: Line Pens</dt>
            <dd>'stroke-width: 1; fill: none'</dd>
            <dd>'stroke' (colors): black, #333, #555, #777, #bbb</dd>
          <dt><b>Pen_5</b> to <b>Pen_9</b></dt>
            <dd>'stroke-width: 1; fill: none'</dd>
            <dd>'stroke-dasharray: 10 10'</dd>
            <dd>'stroke' (colors): black, #333, #555, #777, #bbb</dd>
          <dt><b>Pen_10</b> to <b>Pen_14</b></dt>
            <dd>'stroke-width: 1; fill: none'</dd>
            <dd>'stroke-dasharray: 5 5'</dd>
            <dd>'stroke' (colors): black, #333, #555, #777, #bbb</dd>
          <dt><b>Pen_15</b> to <b>Pen_19</b></dt>
            <dd>'stroke-width: 1; fill: none'</dd>
            <dd>'stroke-dasharray: 2 2'</dd>
            <dd>'stroke' (colors): black, #333, #555, #777, #bbb</dd>
          <dt><b>Pen_20</b> to <b>Pen_24</b>: Text Pens</dt>
            <dd>'stroke-width: 1; stroke: none; fill: black;'</dd>
            <dd>Text anchor point: center.</dd>
            <dd>Font size: 10, 15, 20, 25, 30</dd>
          <dt><b>Pen_25</b> to <b>Pen_29</b>: Thick Line Pens</dt>
            <dd>'stroke-width: 4; fill: none'</dd>
            <dd>'stroke' (colors): black, #333, #555, #777, #bbb</dd>
          <dt><b>Pen_30</b>: Suppression Pen</dt>
            <dd>'stroke: none; fill: none;'</dd>
        </dl></p>
      <p>The font sizes range from (about) 8-point to (roughly) 24-point. The 'stroke: none' makes the letters look nice.</p>
      @param gPaper The SVGTag to receive the standard <code>StylePen</code>s.
  */
  public static final void addPens(SVGTag gPaper) {
    if(gPaper == null) return;
    MathDrawSVG MDraw = new MathDrawSVG();
    StylePen sPen = MDraw.getStylePen();
    sPen.setStroke("black").setFill("none").setStrokeWidth(1.0).setPen(0); gPaper.add(sPen);
    sPen.setStroke("#333").setFill("none").setStrokeWidth(1.0).setPen(1); gPaper.add(sPen);
    sPen.setStroke("#555").setFill("none").setStrokeWidth(1.0).setPen(2); gPaper.add(sPen);
    sPen.setStroke("#777").setFill("none").setStrokeWidth(1.0).setPen(3); gPaper.add(sPen);
    sPen.setStroke("#bbb").setFill("none").setStrokeWidth(1.0).setPen(4); gPaper.add(sPen);
    sPen.setStroke("black").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("10 10").setPen(5); gPaper.add(sPen);
    sPen.setStroke("#333").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("10 10").setPen(6); gPaper.add(sPen);
    sPen.setStroke("#555").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("10 10").setPen(7); gPaper.add(sPen);
    sPen.setStroke("#777").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("10 10").setPen(8); gPaper.add(sPen);
    sPen.setStroke("#bbb").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("10 10").setPen(9); gPaper.add(sPen);
    sPen.setStroke("black").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("5 5").setPen(10); gPaper.add(sPen);
    sPen.setStroke("#333").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("5 5").setPen(11); gPaper.add(sPen);
    sPen.setStroke("#555").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("5 5").setPen(12); gPaper.add(sPen);
    sPen.setStroke("#777").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("5 5").setPen(13); gPaper.add(sPen);
    sPen.setStroke("#bbb").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("5 5").setPen(14); gPaper.add(sPen);
    sPen.setStroke("black").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("2 2").setPen(15); gPaper.add(sPen);
    sPen.setStroke("#333").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("2 2").setPen(16); gPaper.add(sPen);
    sPen.setStroke("#555").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("2 2").setPen(17); gPaper.add(sPen);
    sPen.setStroke("#777").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("2 2").setPen(18); gPaper.add(sPen);
    sPen.setStroke("#bbb").setFill("none").setStrokeWidth(1.0).setStrokeDasharray("2 2").setPen(19); gPaper.add(sPen);
    sPen = (MDraw.getTextPen().setTextAnchor('m').setFontSize(10.0)).setStroke("none").setFill("black").setStrokeWidth(1.0).setPen(20); gPaper.add(sPen);
    sPen = (MDraw.getTextPen().setTextAnchor('m').setFontSize(15.0)).setStroke("none").setFill("black").setStrokeWidth(1.0).setPen(21); gPaper.add(sPen);
    sPen = (MDraw.getTextPen().setTextAnchor('m').setFontSize(20.0)).setStroke("none").setFill("black").setStrokeWidth(1.0).setPen(22); gPaper.add(sPen);
    sPen = (MDraw.getTextPen().setTextAnchor('m').setFontSize(25.0)).setStroke("none").setFill("black").setStrokeWidth(1.0).setPen(23); gPaper.add(sPen);
    sPen = (MDraw.getTextPen().setTextAnchor('m').setFontSize(30.0)).setStroke("none").setFill("black").setStrokeWidth(1.0).setPen(24); gPaper.add(sPen);
    sPen = MDraw.getStylePen().setStroke("black").setFill("none").setStrokeWidth(4.0).setPen(25); gPaper.add(sPen);
    sPen.setStroke("#333").setFill("none").setStrokeWidth(4.0).setPen(26); gPaper.add(sPen);
    sPen.setStroke("#555").setFill("none").setStrokeWidth(4.0).setPen(27); gPaper.add(sPen);
    sPen.setStroke("#777").setFill("none").setStrokeWidth(4.0).setPen(28); gPaper.add(sPen);
    sPen.setStroke("#bbb").setFill("none").setStrokeWidth(4.0).setPen(29); gPaper.add(sPen);
    sPen.setStroke("none").setFill("none").setStrokeWidth(1.0).setPen(30); gPaper.add(sPen);
  }

  /** Build the standard GUI. */
  public final Panel standardGUI() {
    setLayout(cdlMain);
    Panel pnlTemp = new Panel();
    pnlTemp.add(txaPizza);
    add(boilerPlate(KEY_MARGINS, marginPanel()), KEY_MARGINS);
    add(boilerPlate(KEY_PIZZA,pnlTemp), KEY_PIZZA);
    gpMenu.addSeparator();
    add(boilerPlate(KEY_QUAD, new QuadrillePanel()), KEY_QUAD);
    add(boilerPlate(KEY_POLAR, new PolarPanel()), KEY_POLAR);
    add(boilerPlate(KEY_LOG, new LogPanel()), KEY_LOG);
    add(boilerPlate(KEY_LEDGER, new LedgerPanel()), KEY_LEDGER);
    add(boilerPlate(KEY_BPS, new BPSPanel()), KEY_BPS);
    add(boilerPlate(KEY_HEX, new HexGridPanel()), KEY_HEX);
    add(boilerPlate(KEY_BARCHART, new BarChartPanel()), KEY_BARCHART);
    add(boilerPlate(KEY_MUSIC, new MusicPanel()), KEY_MUSIC);
    add(boilerPlate(KEY_LETTERS, new LetteringPanel()), KEY_LETTERS);
    add(boilerPlate(KEY_BRICKS, new OffsetQuadPanel()), KEY_BRICKS);
    add(boilerPlate(KEY_SYMLOG, new SymmetricLogPanel()), KEY_SYMLOG);
    add(boilerPlate(KEY_ISO, new IsometricPanel()), KEY_ISO);
    add(boilerPlate(KEY_PERSPECTIVE, new PerspectivePanel()), KEY_PERSPECTIVE);
    
    return this;
  }

  private final Panel boilerPlate(String myKey, Panel myCenter) {
    String[] keyz = myKey.split(":");
    myKey = keyz[0];
    Panel retval = new Panel(new BorderLayout());
    Button btnTemp = new Button(BTN_NEXT); retval.add(btnTemp, BorderLayout.EAST); btnTemp.addActionListener(this);
    btnTemp = new Button(BTN_PREV); retval.add(btnTemp, BorderLayout.WEST); btnTemp.addActionListener(this);
    retval.add(new Label(myKey, Label.CENTER), BorderLayout.NORTH);
    Panel pnlTemp = new Panel(new GridLayout(1,0));
    String mySelf = ""+myKey.trim().substring(0,4)+"_KEY";
    btnTemp = new Button("To Pizza"); btnTemp.setActionCommand(KEY_TOPIZZA+mySelf); btnTemp.addActionListener(this); pnlTemp.add(btnTemp);
    btnTemp = new Button("Eat Pizza"); btnTemp.setActionCommand(KEY_EATPIZZA+mySelf); btnTemp.addActionListener(this); pnlTemp.add(btnTemp);
    TextField txfTemp = new TextField();
    htbPrefixes.put(mySelf, txfTemp);
    pnlTemp.add(txfTemp);
    btnTemp = new Button("Go!"); btnTemp.setActionCommand(mySelf); btnTemp.addActionListener(this); pnlTemp.add(btnTemp);
    btnTemp = new Button("Reset"); btnTemp.setActionCommand(KEY_RESET+mySelf); btnTemp.addActionListener(this); pnlTemp.add(btnTemp);
    retval.add(pnlTemp, BorderLayout.SOUTH);
    retval.add(myCenter, BorderLayout.CENTER);
    if(myCenter instanceof GraphPanel) htbPanels.put(mySelf, (GraphPanel) myCenter);
    if(keyz.length == 2) {
      MenuItem mniTemp = new MenuItem(keyz[1]);
      mniTemp.setActionCommand(keyz[0]+":"+keyz[1]);
      mniTemp.addActionListener(this);
      gpMenu.add(mniTemp);
    }
    return retval;
}
  private final Panel marginPanel() {
    Panel pnlTemp = new Panel(new GridLayout(0, 3));
    pnlTemp.add(new Label("Left")); pnlTemp.add(txfMarginLeft); pnlTemp.add(new Label("(1/100) unit"));
    pnlTemp.add(new Label("Right")); pnlTemp.add(txfMarginRight); pnlTemp.add(new Label("(1/100) unit"));
    pnlTemp.add(new Label("Top")); pnlTemp.add(txfMarginTop); pnlTemp.add(new Label("(1/100) unit"));
    pnlTemp.add(new Label("Bottom")); pnlTemp.add(txfMarginBottom); pnlTemp.add(new Label("(1/100) unit"));
    for(String sUnit : SVG_UNITS) {
      chcWidth.add(sUnit);
      chcHeight.add(sUnit);
    }
    chcWidth.add(" ");
    chcHeight.add(" ");
    pnlTemp.add(new Label("Width")); pnlTemp.add(txfPaperWidth); pnlTemp.add(chcWidth);
    pnlTemp.add(new Label("Height")); pnlTemp.add(txfPaperHeight); pnlTemp.add(chcHeight);
    pnlTemp.add(new Label(" ")); pnlTemp.add(cbxClip); pnlTemp.add(cbxGPDebug);
    return pnlTemp;
  }

  /** Even though this is not technically a <code>GraphPanel</code> this method was defined to allow persistence of margin and size settings. The two Checkboxes, 
      'processMathObject' and 'Debug mode' are not saved.
  */
  public String toPizza(String pPrefix) {
    StringBuffer retval = new StringBuffer("# MarginPanel configuration data.");
    if(Question.validString(pPrefix)) {
      if(!pPrefix.endsWith(".")) pPrefix += ".";
      retval.append(" Prefix: ").append(pPrefix);
    } else {
      pPrefix = "";
    }
    retval.append("\n");
    retval.append(pPrefix).append("margins.left=").append(txfMarginLeft.getText()).append("\n");
    retval.append(pPrefix).append("margins.right=").append(txfMarginRight.getText()).append("\n");
    retval.append(pPrefix).append("margins.top=").append(txfMarginTop.getText()).append("\n");
    retval.append(pPrefix).append("margins.bottom=").append(txfMarginBottom.getText()).append("\n");
    retval.append(pPrefix).append("margins.width.value=").append(txfPaperWidth.getText()).append("\n");
    retval.append(pPrefix).append("margins.height.value=").append(txfPaperHeight.getText()).append("\n");
    retval.append(pPrefix).append("margins.width.units=").append(chcWidth.getSelectedIndex()).append("\n");
    retval.append(pPrefix).append("margins.height.units=").append(chcHeight.getSelectedIndex()).append("\n");
    return retval.toString();
  }

  /** Load size and margin settings from a <code>Properties</code> object. If present the prefix is prepended to the property keys. */
  public void eatPizza(Properties pPizza, String pPrefix) {
    if((pPizza == null) || pPizza.isEmpty()) return;
    if(Question.validString(pPrefix)) {
      if(!pPrefix.endsWith(".")) pPrefix += ".";
    } else {
      pPrefix = "";
    }
    txfMarginLeft.setText(pPizza.getProperty(pPrefix+"margins.left", "50"));
    txfMarginRight.setText(pPizza.getProperty(pPrefix+"margins.right", "800"));
    txfMarginTop.setText(pPizza.getProperty(pPrefix+"margins.top", "50"));
    txfMarginBottom.setText(pPizza.getProperty(pPrefix+"margins.bottom", "1050"));
    txfPaperWidth.setText(pPizza.getProperty(pPrefix+"margins.width.value", "8.5"));
    txfPaperHeight.setText(pPizza.getProperty(pPrefix+"margins.height.value", "11"));
    try {
      chcHeight.select(Integer.parseInt(pPizza.getProperty(pPrefix+"margins.height.units")));
    } catch(Exception ignored) {
      chcHeight.select(0);
    }
    try {
      chcWidth.select(Integer.parseInt(pPizza.getProperty(pPrefix+"margins.width.units")));
    } catch(Exception ignored) {
      chcWidth.select(0);
    }
  }

  /** Load configuraton data from the Pizzaria panel. */
  public void eatPizza(String pPrefix) {
    Properties thePizza = new Properties();
    try {
      StringReader srIn = new StringReader(txaPizza.getText());
      thePizza.load(srIn);
      srIn.close();
    } catch(Exception ex) { CapCom.error("Error reading margin pizza", ex); }
    eatPizza(thePizza, pPrefix);
  }

  /** Put a non-null, non-blank String into the Pizzaria area. */
  public void setPizza(String pPizza) {
    if(Question.validString(pPizza)) txaPizza.setText(pPizza);
  }

  private int asInt(TextField pText, int pDef) {
    int retval = pDef;
    try { retval = Integer.parseInt(pText.getText()); } catch(Exception ignored) {}
    return retval;
  }
  private double asDouble(TextField pText, double pDef) {
    double retval = pDef;
    try { retval = Double.parseDouble(pText.getText()); } catch(Exception ignored) {}
    return retval;
  }

}

