package cox.jmatt.java.MathTools.demo;

import java.awt.*;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.graphing.*;
import cox.jmatt.java.MathTools.graphing.xml.*;

/** <p>This PizzaPanel demonstrates Graphing equipment. The parameter types are specified on the Choice component and the indicated method is called by clicking 
    the [Go!] button.</p>
    <p>The demos for specific graphing classes use the XML package for reference. The demos are:
    <ol type="A">
      <li>Type a String representation in String.0 to match a Requirement. This uses the <code>match()</code> method.</li>
      <li>Type a String name in String.0 to match a Requirement. This uses the <code>findRequirementByName()</code> method. This also prints the Requirement's 
        String representation.</li>
      <li>Print a simple Constraint with the Requirement from String.0 and the value from String.1. If the 'par.1=' parameter is deleted the Constraint is formed 
        with a null value Object.</li>
      <li>Create a chain of nested Conditions, with the Requirements in String.0, String.1 and String.2 and the value of the inner Constraint as String.3.</li>
      <li>String.0 is fed to <code>Constraint.parseFromString()</code> and the result printed. The result should match the input.</li>
      <li>Create a <code>RenderableXML()</code>, add the values of String.0 to String.4 as parameters, then <code>render()</code> the result.</li>
      <li>The Axis uses Strings for the scale and tick marks.</li>
      <li>The GraphableXML sets its expression from String.0, one limit as String.1 and the other as a Constraint(String.2, String.3).</li>
      <li>String.0 is <code>Arithmetic.splint(s.0, false, false)</code>'ed for the vertex values. Boolean.1 determines whether or not it is closed.</li>
      <li>The <code>GraphPaper</code> has the dimensions specified and also has a parameter and transform added internally.</li>
    </ol></p>
*/
public class GraphPizza extends PizzaPanel {

  private Choice myPicker;

  public Panel getNorthPanel() {
    myPicker = new Choice();
    Panel pnlTemp = new Panel(new BorderLayout());
    pnlTemp.add(new Label("Graphing Test Pizza", Label.CENTER), BorderLayout.NORTH);
    pnlTemp.add(myPicker, BorderLayout.SOUTH);
    
    myPicker.add("A. Match and print Requirement s.0. (String representation)");
    myPicker.add("B. Find and print Requirement s.0. (Requirement name)");
    myPicker.add("C. Create and print a Constraint(s.0, s.1).");
    myPicker.add("D. Create and print Constraint(s.0, Constraint(s.1, Constraint(s.2, s.3))).");
    myPicker.add("E. Constraint.parseFromString(s.0) and print it.");
    myPicker.add("F. Create a Renderable (XML) and add up to five parameters. Then print it.");
    myPicker.add("G. Create and print an Axis (XML) with scale s.0 and tick marks s.1.");
    myPicker.add("H. Create and print a Graphable (XML) with expression s.0, limits s.1 and Constraint(s.2, s.3).");
    myPicker.add("I. Create and print a Polygon (XML) with splint(s.0) vertices and b.1 closure.");
    myPicker.add("J. Create and print GraphPaperXML with width d.0, height d.1 and ranges (d.2, d.3) to (d.4, d.5).");
    
   return pnlTemp;
  }

  public void processGoButton() {
    String sChoice = myPicker.getSelectedItem()+"  ";
    eatPizza();
    Renderable renTemp = null;
    Constraint conTemp = new Constraint();
    Constraint.Requirement reqTemp = null;
    switch(sChoice.charAt(0)) {
      case 'A':
        CapCom.println(Constraint.match(asString(0)).toString());
        break;
      case 'B':
        reqTemp = Constraint.findRequirementByName(asString(0));
        CapCom.println(reqTemp.toString());
        CapCom.println("String representation: "+reqTemp.designation());
        break;
      case 'C':
        conTemp = new Constraint(asString(0), asNullString(1));
        CapCom.println(">"+conTemp.toString()+"<");
        break;
      case 'D':
        conTemp = new Constraint(asString(0), new Constraint(asString(1), new Constraint(asString(2), asNullString(3))));
        CapCom.println(">"+conTemp.toString()+"<");
        break;
      case 'E':
        conTemp = Constraint.parseFromString(asString(0));
        CapCom.println(conTemp.toString());
        break;
      case 'F':
        renTemp = new RenderableXML();
        for(int i=0; i < 5; i++) { renTemp.setParameter("Parameter_"+i, asString(i)); }
        CapCom.println(renTemp.render(null).toString());
        break;
      case 'G':
        renTemp = new AxisXML();
        ((AxisXML) renTemp).setScale(asString(0));
        ((AxisXML) renTemp).setTickMarks(asString(1));
        renTemp.setParameter("Sample Parameter", "Example Value");
        CapCom.println(renTemp.render(null).toString());
        break;
      case 'H':
        renTemp = new GraphableXML();
        ((GraphableXML) renTemp).addExpression("Exp01", asString(0));
        ((GraphableXML) renTemp).addLimit("String Limit", asString(1));
        //((GraphableXML) renTemp).addLimit("Constraint Limit", new Constraint(asString(2), asString(3)));
        renTemp.setParameter("Sample Parameter", "Example Value");
        CapCom.println(renTemp.render(null).toString());
        break;
      case 'I':
        cox.jmatt.java.MathTools.graphing.Polygon myPoly = new PolygonXML();
        int[] points = Arithmetic.splint(asString(0), false, false);
        for(int i=0; i < points.length / 2; i++) { myPoly.addVertex(points[2*i], points[2*i + 1]); }
        myPoly.setClosed(asBoolean(1));
        myPoly.setParameter("Sample Parameter", "Example Value");
        CapCom.println(myPoly.render(null).toString());
        break;
      case 'J':
        GraphPaper myPaper = new GraphPaperXML();
        myPaper.setSize(asDouble(0), asDouble(1));
        myPaper.setRanges(asDouble(2), asDouble(3), asDouble(4), asDouble(5));
        myPaper.addTransform("rotate(45)");
        myPaper.setParameter("Paper", "Quadrille");
        CapCom.println(myPaper.render(null).toString());
        break;
      case 'K':
        
        break;
      
      default:
    }
  }
}

