package cox.jmatt.java.MathTools.demo;

import java.awt.*;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.markup.latex.*;

/** <p>This panel demonstrates some of the LaTeX capabilities built into MathTools.
    <ol type="A">
    <li>Attempt to locate the <code>LaSymbol</code> String.0 and print information about it.</li>
    <li>Find LaSymbol String.0 and call format(String.1 split on spaces). The result is sent through <code>CapCom.processMathObject()</code>.</li>
    <li>The entire parameter area is parsed, split on newlines and added to a <code>LaListTag</code>. The result is printed and sent to the clipboard.</li>
    <li>Exactly per the previous selection but the list is ordered. The result is copied to the clipboard.</li>
    <li>Print detailed instructions for the table creation demo. The result is sent to the clipboard.</li>
    <li>Create a <code>LaTable</code>, print it and send it to the clipboard. The first line is the column alignment, the others are the table items. Each line 
      is split on ampersands ('&amp;') and the result added as items. The ampersand was chosen as a BREAK character to allow maximum flexibility.</li>
    <li>Create a blank document of the type specified in the first character of String.0. String.1 is the title, String.2 the author and String.3 the date. If 
      String.1 is not supplied there is no title.</li>
    </ol>
    </p
    <p>Although primarily meant as a demonstration this class can be used to generate production LaTeX markup. The 'do-something' selections ('B' through 'G') 
    all send their output to the clipboard. While it might be unwieldy this can provide a quick way to add lists or tables to an existing LaTeX document or even 
    to generate a blank document template! Just a thought.</p>
*/

public class LatexPizza extends PizzaPanel {

  private Choice myPicker;

  public Panel getNorthPanel() {
    myPicker = new Choice();
    Panel pnlTemp = new Panel(new BorderLayout());
    pnlTemp.add(new Label("LaTeX Test Pizza", Label.CENTER), BorderLayout.NORTH);
    pnlTemp.add(myPicker, BorderLayout.SOUTH);
    
    myPicker.add("A. Info on LaSymbol S.0.");
    myPicker.add("B. LaSymbol S.0 format() on S.1 split on spaces. Sent to clipboard.");
    myPicker.add("C. Create an unordered list from the lines in the parameter area. Sent to clipboard.");
    myPicker.add("D. Create an ordered list from the lines in the param area. Sent to clipboard.");
    myPicker.add("E. Table demo instructions.");
    myPicker.add("F. Table demo.");
    myPicker.add("G. Blank document type S.0. S.1 - S.3 are title, author, date. Sent to clipboard.");
    
   return pnlTemp;
  }

  public void processGoButton() {
    String sChoice = myPicker.getSelectedItem()+"  ";
    eatPizza();
    MathGenLatex.LaSymbol mySymbol;
    String sTemp = null;
    String[] sLines = null;
    LaListTag lisTemp = null;
    LaTable tempTable = null;
    LaDocument myDoc = null;
    switch(sChoice.charAt(0)) {
      case 'A':
        mySymbol = MathGenLatex.getSymbol(asString(0));
        CapCom.println("Symbol:\t\t"+mySymbol.toString());
        CapCom.println("Parameters:\t"+mySymbol.getParams());
        CapCom.println("Raw Format:\t"+mySymbol.getFormat());
        CapCom.println("- - - - -");
        break;
      case 'B':
        mySymbol = MathGenLatex.getSymbol(asString(0));
        CapCom.println(CapCom.processMathObject(mySymbol.format(asString(1).split(" "))).toString());
        break;
      case 'C':
        lisTemp = new LaListTag();
        lisTemp.addItems(null, getRawPizza().split("\n"));
        CapCom.println((String) CapCom.processMathObject(lisTemp.toString()));
        break;
      case 'D':
        lisTemp = new LaListTag();
        lisTemp.setEnumerate(true);
        lisTemp.addItems(null, getRawPizza().split("\n"));
        CapCom.println((String) CapCom.processMathObject(lisTemp.toString()));
        break;
      case 'E':
        CapCom.println("--- LaTeX Table Demo Instructions ---\nClear the parameter area!\nThe first line is the column alignment specification.");
        CapCom.println("Subsequent lines are split on '&' characters and added as lines on the table.\nThe ampersand was chosen as a SPLITTING character to allow flexibility.");
        CapCom.println("The result is printed and sent to the clipboard.");
        break;
      case 'F':
        sLines = getRawPizza().split("\n");
        if(sLines.length < 2) {
          CapCom.println("Not enough lines for a table!");
          break;
        }
        tempTable = new LaTable(sLines[0]);
        for(int i = 1; i < sLines.length; i++) { tempTable.addRow(sLines[i].split("&")); }
        CapCom.println((String) CapCom.processMathObject(tempTable.toString()));
        break;
      case 'G':
        myDoc = new LaDocument();
        myDoc.setType((asString(0)+"a").charAt(0));
        myDoc.setTitle(asString(1), asString(2), asString(3));
        CapCom.println((String) CapCom.processMathObject(myDoc.toString()));
        break;
      case 'H':

        break;
      case 'I':

        break;
      case 'J':

        break;

      default:
    }
  }
}

