package cox.jmatt.java.MathTools.demo;

import java.awt.*;
import cox.jmatt.java.MathTools.*;

/** <p>This PizzaPanel demonstrates MComplex class methods. The parameter types are specified on the Choice component and the indicated method is called by 
    clicking the [Go!] button.</p>
    <p>The demos here are:
    <ol type="A" >
      <li>Print information concerning the MComplex parsed from String.0.</li>
      <li>Construct complex numbers from String.0 and String.1. Next compare them and perform arithmetic operations on them.</li>
      <li>Construct the conjugate of String.0 and print it then print the product of the two.</li>
      <li>Format the MComplex reParsed in String.0 according to the template in String.1. The result is sent through <code>processMathObject()</code>.</li>
      <li>Print MComplex(String.0) using <code>MComplex.FORMAT_DEFAULT</code> and <code>MComplex.FORMAT_TYPICAL</code>.</li>
      <li>Raise 'i' to the power Int.0. The resulting complex number is printed using DEFAULT and TYPICAL format.</li>
      <li>Create and print a complex number from the Knobs in String.0.</li>
      <li>Divide MComplex(String.0) by MComplex(String.1) with String.2 as the separator and String.3 as the 'i' symbol.</li>
    </ol></p>
*/

public class MComplexPizza extends PizzaPanel {

  private Choice myPicker;

  public Panel getNorthPanel() {
    myPicker = new Choice();
    Panel pnlTemp = new Panel(new BorderLayout());
    pnlTemp.add(new Label("MComplex Test Pizza", Label.CENTER), BorderLayout.NORTH);
    pnlTemp.add(myPicker, BorderLayout.SOUTH);
    
    myPicker.add("A. reParse s.0 and print lots of information about it.");
    myPicker.add("B. Compare and operate on complex numbers s.0 and s.1.");
    myPicker.add("C. Display the conjugate of s.0 and then the product.");
    myPicker.add("D. Format MComplex(s.0) per template s.1 and send to the clipboard.");
    myPicker.add("E. Format MComplex(s.0) by DEFAULT, STANDARD and TYPICAL.");
    myPicker.add("F. Raise 'i' to the power i.0");
    myPicker.add("G. Create MComplex.fromKnobs(s.0) and print it.");
    myPicker.add("H. Print MComplex(s.0).divide(MComplex(s.1), s.2, s.3).");
    
   return pnlTemp;
  }

  public void processGoButton() {
    String sChoice = myPicker.getSelectedItem()+"  ";
    eatPizza();
    MComplex cLHS = MComplex.reParse(asString(0)), cRHS = MComplex.reParse(asString(1));
    int iTemp = 0;
    switch(sChoice.charAt(0)) {
      case 'A':
        CapCom.println("As String:\t"+cLHS.toString());
        CapCom.println("As XML:\t\t"+cLHS.toXML(null));
        CapCom.println("Real part:\t"+cLHS.real()+"\nImaginary part:\t"+cLHS.imag()+"\nIs real:\t\t"+cLHS.isReal());
        CapCom.println("Is imaginary:\t"+cLHS.isImaginary()+"\nIs complex:\t"+cLHS.isComplex()+"\nIs zero:\t\t"+cLHS.isZero());
        CapCom.println("Magnitude:\t"+cLHS.abs());
        break;
      case 'B':
        CapCom.println("Equal:\t\t"+cLHS.equals(cRHS));
        CapCom.println("Conjugates:\t"+cLHS.isConjugate(cRHS));
        CapCom.println("Sum:\t\t"+cLHS.add(cRHS).format());
        CapCom.println("Difference:\t"+cLHS.subtract(cRHS).format());
        CapCom.println("Product:\t\t"+cLHS.multiply(cRHS).format());
        break;
      case 'C':
        cRHS = cLHS.conjugate();
        CapCom.println("Conjugate:\t"+cRHS.format()+"\nProduct:\t\t"+cLHS.multiply(cRHS).format());
        break;
      case 'D':
        CapCom.println("- - - Formatting MComplex(String.0) by String.1 - - -");
        CapCom.println((String) CapCom.processMathObject(cLHS.format(asString(1))));
        break;
      case 'E':
        CapCom.println("Default:\t"+cLHS.format(MComplex.FORMAT_DEFAULT));
        CapCom.println("Std.:\t"+cLHS.format(MComplex.FORMAT_STANDARD));
        CapCom.println("Typical:\t"+cLHS.format(MComplex.FORMAT_TYPICAL));
        break;
      case 'F':
        iTemp = asInt(0);
        CapCom.println("Raising: i^"+iTemp);
        cLHS = MComplex.iPow(iTemp);
        CapCom.println("Default:\t"+cLHS.format(MComplex.FORMAT_DEFAULT));
        CapCom.println("Typical:\t"+cLHS.format(MComplex.FORMAT_TYPICAL));
        break;
      case 'G':
        cLHS = MComplex.fromKnobs(asString(0));
        CapCom.println("From Knobs: "+cLHS.format(MComplex.FORMAT_DEFAULT));
        break;
      case 'H':
        CapCom.println("Dividing: "+cLHS.divide(cRHS, asString(2), asString(3)));
        break;
      case 'I':

        break;
      case 'J':

        break;

      default:
    }
  }
}


