package cox.jmatt.java.MathTools.demo;

import java.awt.*;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.util.MMatrixBuilder;

/** <p>This PizzaPanel demonstrates MMatrices. The parameter types are specified on the Choice component and the indicated method is called by clicking 
    the [Go!] button.<p>
    <p>Selections in this demo use a MMatrix created around String.0 and, where another is needed, one created around String.1. The row and column separators are, 
    respectiely, a semicolon and a comma. The format String '1,2,3,4;5,6,7,8' would yield a 2x4 MMatrix with rows '1 2 3 4' and '5 6 7 8'. The demos are:
    <ol type="A" >
      <li>Print a short help screen on MMatrix formatting and entry.</li>
      <li>Print the MMatrices built around String.0 and String.1. If either parameter is blank, the default is MMatrixBuilder.TEST_MATRIX(). This uses the 
      MMatrixBuilder.printMMatrix() method.</li>
      <li>Add MMatrix(String.0) to MMatrix(String.1). The orders must be identical for a sum to be calculated.</li>
      <li>Subtract MMatrix(String.1) from MMatrix(String.0). The orders must be identical or the subtraction will fail.</li>
      <li>Multiply MMatrix(String.0) by MMatrix(String.1). The rows in the first matrix MUST equal the columns in the second or the product will fail.</li>
      <li>Calculate and print the determinant of MMatrix(String.0). The matrix must be <b>square</b>.</li>
      <li>Print the scalar product of MMatrix(String.0) and <b>double.2</b>.</li>
      <li>For MMatrix(s.0) call toString(), toXML(), toMathML() and toLatex().<li>
      <li>Call the <code>createRandMMatrix() method with int.0 rows and int.1 columns. The element range is [int.2, int.3) and boolean.4 allows zero or not.</li>
      <li>Create MMatrix around String.0, print it and print the transpose.</li>
      <li>Specialized to format the MMatrix for OpenOffice Math.</li>
      <li>In this call the column separator is a tab and the row separator a newline. The augmentation separator is a vertical pipe.</li>
      <li>This format uses default variables, a newline row-separator and no column separator.</li>
    </ol></p>
*/
public class MMatrixPizza extends PizzaPanel {

  private Choice myPicker;

  public Panel getNorthPanel() {
    myPicker = new Choice();
    Panel pnlTemp = new Panel(new BorderLayout());
    pnlTemp.add(new Label("MMatrix Test Pizza", Label.CENTER), BorderLayout.NORTH);
    pnlTemp.add(myPicker, BorderLayout.SOUTH);
    
    myPicker.add("A. Print Help.");
    myPicker.add("B. Print MMatrix(s.0) and MMatrix(s.1).");
    myPicker.add("C. MMatrix(s.0).add(MMatrix(s.1));.");
    myPicker.add("D. MMatrix(s.0).subtract(MMatrix(s.1));.");
    myPicker.add("E. MMatrix(s.0).multiply(MMatrix(s.1));.");
    myPicker.add("F. MMatrix(s.0).determinant()");
    myPicker.add("G. Scalar product: d.2 * MMatrix(s.0).");
    myPicker.add("H. MMatrix(s.0) toString(), toXML(), toMathML() and toLatex().");
    myPicker.add("I. Random MMatrix i.0 x i.1, elements [i.2, i.3) b.4 allow zero");
    myPicker.add("J. Print MMatrix(s.0) and its transpose.");
    myPicker.add("K. Format MMatrix(s.0) for OpenOffice and send to the clipboard.");
    myPicker.add("L. Format MMatrix(s.0) as augmented matrix with s.2 split on spaces.");
    myPicker.add("M. Format MMatrix(s.0) as a system with s.2 split on spaces.");
    
   return pnlTemp;
  }

  public void processGoButton() {
    String sChoice = myPicker.getSelectedItem()+"  ";
    eatPizza();
    MMatrix matrix1 = MMatrixBuilder.TEST_MATRIX();
    MMatrix matrix2 = MMatrixBuilder.TEST_MATRIX();
    if(Question.validString(asString(0))) matrix1 = MMatrix.createMMatrix(asString(0));
    if(Question.validString(asString(1))) matrix2 = MMatrix.createMMatrix(asString(1));
    switch(sChoice.charAt(0)) {
      case 'A':
        printHelp();
        break;
      case 'B':
        CapCom.println("MMatrix(s.0)");
        MMatrixBuilder.printMMatrix(matrix1);
        CapCom.println("MMatrix(s.1)");
        MMatrixBuilder.printMMatrix(matrix2);
        break;
      case 'C':
        CapCom.println("Sum:");
        MMatrixBuilder.printMMatrix(matrix1.add(matrix2));
        break;
      case 'D':
        CapCom.println("Difference:");
        MMatrixBuilder.printMMatrix(matrix1.subtract(matrix2));
        break;
      case 'E':
        CapCom.println("Product:");
        MMatrixBuilder.printMMatrix(matrix1.multiply(matrix2));
        break;
      case 'F':
        CapCom.println("Determinant of s.0: "+matrix1.determinant());
        break;
      case 'G':
        CapCom.println("Scalar Product: ");
        MMatrixBuilder.printMMatrix(matrix1.multiply(asDouble(2)));
        break;
      case 'H':
        CapCom.println("As string: "+matrix1.toString());
        CapCom.println("As XML: "+matrix1.toXML(null));
        CapCom.println("As MathML:\n"+matrix1.toMathML());
        CapCom.println("As LaTeX:\n"+matrix1.toLatex(null, null, null));
        break;
      case 'I':
        matrix1 = MMatrixBuilder.createRandMMatrix(asInt(0), asInt(1), asInt(2), asInt(3), asBoolean(4));
        MMatrixBuilder.printMMatrix(matrix1);
        break;
      case 'J':
        CapCom.println("-- Original --");
        MMatrixBuilder.printMMatrix(matrix1);
        CapCom.println("-- Transpose --");
        MMatrixBuilder.printMMatrix(matrix1.transpose());
        break;
      case 'K':
        CapCom.println("Generating OO matrix and sending to clipboard.");
        CapCom.println(((String) CapCom.processMathObject(matrix1.formatMatrix("##", "#", "matrix{", "}"))));
        break;
      case 'L':
        CapCom.println(matrix1.format(MathStat.sparse(asString(2)), "\n", "\t", null, null, " | ", null, null));
        break;
      case 'M':
        String[] pRels = new String[0], pVars = new String[0];
        CapCom.println(matrix1.format(MathStat.sparse(asString(2)), "\n", " ", pVars, pRels, null, null, null));
        break;
      case 'N':

        break;
      case 'O':

        break;
      
      default:
    }
  }
  private void printHelp() {
    CapCom.println("Enter a matrix using standard notation: 'a00, a01, ... a0n;a10, a11, ...;'\nUse a comma as an element separator.\nUse a semicolon as a row separator.");
    CapCom.println("Most MMatrix methods apply to MMatrix(s.0) and MMatrix(s.1)");
  }
}
