package cox.jmatt.java.MathTools.demo;

import java.awt.*;
import java.awt.event.*;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.util.*;

/** This class demonstrates the features of MathTools. */
public class MTDemo extends Frame implements WindowListener, ActionListener {

  private static MTDemo MYSELF = null;

  private CardLayout cdlDeck = new CardLayout();
  private Button btnPrev = new Button("<<|");
  private Button btnNext = new Button("|>>");
  private Panel pnlDemos = new Panel();
  private Menu mnuGPG = null;
  
  private static final String DEMO_CALC = "Calculator";
  private static final String DEMO_QUESTION = "Question";
  private static final String DEMO_MENGINE = "MathEngine";
  private static final String DEMO_ARITHMETIC = "Arithmetic";
  private static final String DEMO_MATH_XML = "MathXML";
  private static final String DEMO_POLYNOMIAL = "Polynomial";
  private static final String DEMO_COMPLEX = "MComplex";
  private static final String DEMO_MLINEAR = "MLinear";
  private static final String DEMO_MFRACTION = "MFraction";
  private static final String DEMO_PERCENT = "Percent";
  private static final String DEMO_SCRIPT = "Scripting";
  private static final String DEMO_MMATRIX = "MMatrix";
  private static final String DEMO_MEQUATION = "MEquation";
  private static final String DEMO_MRADICAL = "MRadical";
  private static final String DEMO_GRAPH = "Graphing";
  private static final String DEMO_SVG = "SVG Classes";
  private static final String DEMO_GPGEN = "Graph Paper";
  private static final String DEMO_FORMATTER = "MTFormatter";
  private static final String DEMO_STAT = "MathStat";
  private static final String DEMO_SQL = "MathDBC & SQL";
  private static final String DEMO_NOTEWORTHY = "Noteworthy";
  private static final String DEMO_LATEX = "LaTeX";
  private static final String DEMO_XML = "XML Utilities";
  private static final String DEMO_MATHSET = "MathSet";
  private static final String DEMO_DIRPROC = "Dir. Processor";

  /** Standard constructor. Nothing special. */
  public MTDemo() {
    super();
    MYSELF = this;
  }
  
  /** This is where all the components are added and the demo itself is shown and started. */
  public void start() {
    setTitle("Matt's MathTools Demo");
    Panel pnlTemp = new Panel(new BorderLayout());
    BasicMathPrinter MPTemp = new BasicMathPrinter();
    CapCom.setPrinter((MathPrinter) MPTemp);
    pnlTemp.add((Panel) MPTemp.standardGUI(), BorderLayout.CENTER);
    btnPrev.addActionListener(this);
    pnlTemp.add(btnPrev, BorderLayout.WEST);
    btnNext.addActionListener(this);
    pnlTemp.add(btnNext, BorderLayout.EAST);
    pnlTemp.add(getDemoPanels(), BorderLayout.SOUTH);
    
    add(pnlTemp);
    addWindowListener(this);
    pack();
    setVisible(true);
  }
  // Set up the demo panels and MenuBar
  private final Panel getDemoPanels() {
    pnlDemos = new Panel(cdlDeck);
    Menu mnuView = new Menu("View");
    MenuItem mniTemp;
    pnlDemos.add(new QuickRPNCalculator(), DEMO_CALC);
    mniTemp = new MenuItem(DEMO_CALC); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new MathEnginePizza(), DEMO_MENGINE);
    mniTemp = new MenuItem(DEMO_MENGINE); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new QuestionPizza(), DEMO_QUESTION);
    mniTemp = new MenuItem(DEMO_QUESTION); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new MTFormatterDemo(this).standardGUI(), DEMO_FORMATTER);
    mniTemp = new MenuItem(DEMO_FORMATTER); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new ArithmeticPizza(), DEMO_ARITHMETIC);
    mniTemp = new MenuItem(DEMO_ARITHMETIC); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new MathXMLPizza(), DEMO_MATH_XML);
    mniTemp = new MenuItem(DEMO_MATH_XML); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new XMLUtils(this), DEMO_XML);
    mniTemp = new MenuItem(DEMO_XML); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new PolynomialPizza(), DEMO_POLYNOMIAL);
    mniTemp = new MenuItem(DEMO_POLYNOMIAL); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new MFractionPizza(), DEMO_MFRACTION);
    mniTemp = new MenuItem(DEMO_MFRACTION); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new MathStatPizza(), DEMO_STAT);
    mniTemp = new MenuItem(DEMO_STAT); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new MEquationPizza(), DEMO_MEQUATION);
    mniTemp = new MenuItem(DEMO_MEQUATION); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new MRadicalPizza(), DEMO_MRADICAL);
    mniTemp = new MenuItem(DEMO_MRADICAL); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new MComplexPizza(), DEMO_COMPLEX);
    mniTemp = new MenuItem(DEMO_COMPLEX); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new MathSetPizza(), DEMO_MATHSET);
    mniTemp = new MenuItem(DEMO_MATHSET); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new PercentPizza(), DEMO_PERCENT);
    mniTemp = new MenuItem(DEMO_PERCENT); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new GraphPizza(), DEMO_GRAPH);
    mniTemp = new MenuItem(DEMO_GRAPH); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new SVGPizza(), DEMO_SVG);
    mniTemp = new MenuItem(DEMO_SVG); mniTemp.addActionListener(this); mnuView.add(mniTemp);

    GPGenerator myGPG = new GPGenerator();
    pnlDemos.add(myGPG.standardGUI(), DEMO_GPGEN);
    mniTemp = new MenuItem(DEMO_GPGEN); mniTemp.addActionListener(this); mnuView.add(mniTemp);

    pnlDemos.add(new MathDBCDemo(this).standardGUI(), DEMO_SQL);
    mniTemp = new MenuItem(DEMO_SQL); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new DirectoryProcessorDemo(this), DEMO_DIRPROC);
    mniTemp = new MenuItem(DEMO_DIRPROC); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new LatexPizza(), DEMO_LATEX);
    mniTemp = new MenuItem(DEMO_LATEX); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new NoteworthyDemo(this).standardGUI(), DEMO_NOTEWORTHY);
    mniTemp = new MenuItem(DEMO_NOTEWORTHY); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new MMatrixPizza(), DEMO_MMATRIX);
    mniTemp = new MenuItem(DEMO_MMATRIX); mniTemp.addActionListener(this); mnuView.add(mniTemp);
    pnlDemos.add(new ScriptPanel(this).standardGUI(), DEMO_SCRIPT);
    mniTemp = new MenuItem(DEMO_SCRIPT); mniTemp.addActionListener(this); mnuView.add(mniTemp);

    MenuBar mnbMain = new MenuBar();
    mnbMain.add(mnuView);
    mnuGPG = myGPG.getMenu();
    mnuGPG.setEnabled(false);
    mnbMain.add(mnuGPG);
    setMenuBar(mnbMain);
    return pnlDemos;
  }

  public void windowActivated(WindowEvent we){}
  public void windowClosed(WindowEvent we){}
  /** This is the only WindowListener method defined. Anything generating a closing event is set invisible and disposed. If MTDemo is the origin, it also calls 
      System.exit(0).
  */
  public void windowClosing(WindowEvent we){
    ((Window) we.getSource()).setVisible(false);
    ((Window) we.getSource()).dispose();
    if(this == we.getSource()) {
      System.exit(0);
    }
  }
  public void windowDeactivated(WindowEvent we){}
  public void windowDeiconified(WindowEvent we){}
  public void windowIconified(WindowEvent we){}
  public void windowOpened(WindowEvent we){}
  
  /** Detect demo-switching buttons and the menu. */
  public void actionPerformed(ActionEvent aev) {
    mnuGPG.setEnabled(false);
    if(aev.getSource() == btnPrev) {
      cdlDeck.previous(pnlDemos);
    } else if(aev.getSource() == btnNext) {
      cdlDeck.next(pnlDemos);
    } else {
      cdlDeck.show(pnlDemos, aev.getActionCommand());
      mnuGPG.setEnabled(DEMO_GPGEN.equalsIgnoreCase(aev.getActionCommand()));
    }
  }
  
  /** The Main method does nothing but instantiate an MTDemo and start() it. */
  public static void main(String[] args) { (new MTDemo()).start(); }

}
