package cox.jmatt.java.MathTools.demo;

import java.awt.*;
import java.util.*;
import cox.jmatt.java.MathTools.*;
/** <p>This PizzaPanel demonstrates MathEngine methods. The parameter types are specified on the Choice component and the indicated method is called by clicking 
    the [Go!] button.</p>
    <p>The demonstrations in this panel are:
      <ol type="A" >
        <li>Generate int.0 calls to random() and print them. A 'high-low' count is kept and printed: low is below 0.5, high is greater or equal. If greater than 
          zero, double.1 and double.2 are used to set <code>adjustRandom()</code>. CAUTION: This <code>does</code> stay in effect until specifically unset!</li>
        <li>Generate int.0 random numbers between int.1 (inclusive) and int.2 (exclusive). If boolean.3 is true, the method used is 
        {@link cox.jmatt.java.MathTools.MathEngine#nzRandom MathEngine.nzRandom()} otherwise, {@link cox.jmatt.java.MathTools.MathEngine#random(int, int) MathEngine.random()} 
        is called.</li>
        <li>Enter a Knobs string into String.0: An even number of numbers separated by spaces. MathEngine.setKnobs() is then called on this and the results printed.</li>
        <li>This demo splick()'s String.0 by the character entered in String.1. If String.1 is blank, a space is used. Once the entries are split, a random one 
        is picked.</li>
        <li>String.0 and String.1 are each split on spaces, a random element is chosen, and it is converted to an int. The results from splickInt()-ing String.0 and 
        String.1 are then added and printed.</li>
        <li>String.0 is fed into MathEngine.parseRPN() as a postfix-formatted calculation String. This is the actual method behind {@link cox.jmatt.java.MathTools.util.QuickRPNCalculator 
        QuickRPNCalculator}.</li>
        <li>Create and fill an expression. See {@link cox.jmatt.java.MathTools.MathEngine#createExpression MathEngine.createExpression()} for details.</li>
        <li>Read a block of 'par' lines and print them. The block starts at 'par.0' and stops at the first blank parameter line. First the lines are printed then they 
        are read into a single String and printed. Finally, a random line is chosen (<code>blick()</code>).</li>
        <li>The <code>isoKnobs()</code> method is called on String.0 and then on the resultint String five times and the output printed.</li>
        <li>Splint String.0 and <code>arCat()</code> it back together. boolean.1 determines 'and', and boolean.2 determines whether or not to number-word them.</li>
        <li>Split String.0 on spaces and use the String <code>arCat()</code> to concatenate it. String.1 is the separator, String.2 is the last separator and 
        Boolean.3 is descending/ascending.</li>
      </ol></p>
*/
public class MathEnginePizza extends PizzaPanel {

  private Choice myPicker;

  public Panel getNorthPanel() {
    myPicker = new Choice();
    Panel pnlTemp = new Panel(new BorderLayout());
    pnlTemp.add(new Label("MathEngine Test Pizza", Label.CENTER), BorderLayout.NORTH);
    pnlTemp.add(myPicker, BorderLayout.SOUTH);
    
    myPicker.add("A. Generate i.0 randoms. adjustRandom(d.1, d.2).");
    myPicker.add("B. i.0 randoms from [i.1, i.2); b.3 = true for nzRandom()");
    myPicker.add("C. setKnobs() on s.0");
    myPicker.add("D. splick(s.0, s.1)");
    myPicker.add("E. splickInt(s.0) + splickInt(s.1)");
    myPicker.add("F. parseRPN(s.0)");
    myPicker.add("G. createExpression() - s.0: Variable, s.1: Knobs, s.2: Template");
    myPicker.add("H. Print blines('par') then bread('par'), then blick('par').");
    myPicker.add("I. Demonstrate isoKnobs(s.0)");
    myPicker.add("J. Call arCat(splint(s.0), b.1, b.2) and print it.");
    myPicker.add("K. Call arCat(S.0 split on spaces, s.1, s.2, b.3");

    return pnlTemp;
  }
  /** <p>This is where things get interesting. Selecting an option and clicking the button triggers a call to the selected method. The parameter index and type 
      are specified as L.0, i.0, d.0, s.0 and b.0 for long, int, double, string and boolean respectively. The number indicates the parameter that must be of that 
      particular type. The delimiter is '&lt;&gt;'.</p>
  */
  public void processGoButton() {
    String sChoice = myPicker.getSelectedItem()+"  ";
    Properties pTemp = eatPizza();
    Enumeration eTemp = pTemp.keys();
    CapCom.setPizza(pTemp);
    int iLow = 0, iHigh = 0;
    double dTemp = 0.0;
    boolean bTemp = false;
    switch(sChoice.charAt(0)) {
      case 'A':
        MathEngine.adjustRandom(asDouble(1), asDouble(2));
        for(int i=0; i < asInt(0); i++) {
          dTemp = MathEngine.random();
          CapCom.println(""+dTemp);
          if(dTemp < 0.5) { iLow++; } else { iHigh++; }
        }
        CapCom.println("Low: "+iLow+"\t\tHigh: "+iHigh);
      break;
      case 'B':
        bTemp = asBoolean(3);
        iLow = asInt(1); iHigh = asInt(2);
        for(int i=0; i <= asInt(0); i++) {
          if(bTemp) {
            CapCom.println(""+MathEngine.nzRandom(iLow, iHigh));
          } else {
            CapCom.println(""+MathEngine.random(iLow, iHigh));
          }
        }
      break;
      case 'C':
        int[] iaTemp = MathEngine.setKnobs(asString(0));
        for(int i=0; i < iaTemp.length; i++) {
          CapCom.println("Knob "+i+": "+iaTemp[i]);
        }
      break;
      case 'D':
        CapCom.println(MathEngine.splick(asString(0), asString(1)));
      break;
      case 'E':
        CapCom.println("Sum: "+(MathEngine.splickInt(asString(0)) + MathEngine.splickInt(asString(1))));
      break;
      case 'F':
        CapCom.println("Result: "+MathEngine.parseRPN(asString(0)));
      break;
      case 'G':
        Question qTemp = MathEngine.createExpression(asString(0), asString(1), asString(2));
        CapCom.println("Expression created\nQ.ID: "+qTemp.setFormat(Question.FORMAT_PROPERTIES).format());
      break;
      case 'H':
        String[] sLines = MathEngine.blines("par");
        CapCom.println("--- As individual lines ---");
        for(int i=0; i < sLines.length; i++) {
          CapCom.println("["+i+"]= "+sLines[i]);
        }
        CapCom.println("--- As a block ---\n"+MathEngine.bread("par", "<>"));
        CapCom.println("--- From blick()-ing ---\n"+MathEngine.blick("par"));
      break;
      case 'I':
        String sTemp = MathEngine.isoKnobs(asString(0));
        CapCom.println("First call: "+sTemp);
        for(int i=0; i < 5; i++) {
          CapCom.println("Result "+i+": "+sTemp);
          sTemp = MathEngine.isoKnobs(sTemp);
        }
      break;
      case 'J':
        CapCom.println(MathEngine.arCat(Arithmetic.splint(asString(0), false, false), asBoolean(1), asBoolean(2)));
      break;
      case 'K':
        CapCom.println("Concatenated: "+MathEngine.arCat(asString(0).split(" "), asNullString(1), asNullString(2), asBoolean(3)));
      break;
      case 'L':
        
      break;
      case 'M':
        
      break;
      
      default:
    }
  }
}
