package cox.jmatt.java.MathTools.demo;

import java.awt.*;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.util.MMatrixBuilder;

/** <p>This PizzaPanel demonstrates MathStat methods. The parameter types are specified on the Choice component and the indicated method is called by clicking 
    the [Go!] button. The specific demos are:
    <ol type="A">
      <li>Call sparse(String.0) and print the array at Debug level.</li>
      <li>Create <code>MathStat(String.0, String.1)</code>, print it as a String and as XML.</li>
      <li>Create <code>MathStat(String.0, String.1)</code> and print the available statistical calculations.</li>
      <li>This demo is exactly like 'C' save that <code>MathStat.fromPoints(String.0)</code> is used to create the instance.</li>
      <li>Create a <code>MathStat</code> with random values:
        <ul>
          <li>Int.0 (Int.4) - The number of values in the data set (second set).</li>
          <li>Double.1 (Double.5) - The minimum data value (second set), inclusive.</li>
          <li>Double.2 (Double.6) - The maximum data value (second set), exclusive.</li>
          <li>Boolean.3 (Boolean.7) - 'true' for floating-point values, false for integer (second set).</li>
        </ul>
        The first parameers are for the primary data set, the parenthesized ones are for the secondary data set. If Int.4 is zero there is no second set.</li>
      <li>Generate a linear-regression <code>MathStat</code>. The line is 'int.0' points along the line (double.1, double.2) to (double.3, double.4) with int.5 
        percent variation. Boolean.6 allows/disallows floating-point x-values in the data set.</li>
    </ol></p>
*/

public class MathStatPizza extends PizzaPanel {

  private Choice myPicker;

  public Panel getNorthPanel() {
    myPicker = new Choice();
    Panel pnlTemp = new Panel(new BorderLayout());
    pnlTemp.add(new Label("MathStat Test Pizza", Label.CENTER), BorderLayout.NORTH);
    pnlTemp.add(myPicker, BorderLayout.SOUTH);
    
    myPicker.add("A. Dump sparse(s.0) at DEBUG level.");
    myPicker.add("B. Create MathStat(s.0, s.1) and print it several ways.");
    myPicker.add("C. Create MathStats(s.0, s.1) and show some calculations using it.");
    myPicker.add("D. Create MathStat.fromPoints(s.0) and show some calculations using it.");
    myPicker.add("E. Create a random MathStat(i.0, d.1, d.2, b.3, i.4, d.5, d.6, b.7)");
    myPicker.add("F. Create a linear MathStat(i.0, d.1, d.2, d.3, d.4, i.5, b.6)");
    
   return pnlTemp;
  }

  public void processGoButton() {
    String sChoice = myPicker.getSelectedItem()+"  ";
    eatPizza();
    MathStat myStat = null;
    boolean bTemp = false;
    switch(sChoice.charAt(0)) {
      case 'A':
        CapCom.println("Dumping sparse(s.0) at Debug level.");
        MMatrixBuilder.dump(MathStat.sparse(asString(0)));
        break;
      case 'B':
        myStat = new MathStat(asString(0), asString(1));
        CapCom.println("String: "+myStat.toString());
        CapCom.println("XML: "+myStat.toXML(null));
        CapCom.println("FORMAT_DEBUG\n"+myStat.format(MathStat.FORMAT_DEBUG, null, null));
        break;
      case 'C':
        myStat = new MathStat(asString(0), asString(1));
        CapCom.println("-- Dumping Stat Calculations --");
        CapCom.println("Count 1, 2: \t"+myStat.count()+", "+myStat.yCount());
        CapCom.println("Mean 1, 2: \t"+myStat.mean()+", \t"+myStat.yMean());
        CapCom.println("Pop Var 1, 2: \t"+myStat.pVar()+", \t"+myStat.ypVar());
        CapCom.println("Samp Var 1, 2: \t"+myStat.sVar()+", \t"+myStat.ysVar());
        CapCom.println("Pop S Dev 1, 2: \t"+myStat.pDev()+", \t"+myStat.ypDev());
        CapCom.println("Samp SD 1, 2: \t"+myStat.sDev()+", \t"+myStat.ysDev());
        CapCom.println("Correlation: \t"+myStat.correlation());
        CapCom.println("Regr. Line: \t"+myStat.regressionLine());
        break;
      case 'D':
        myStat = MathStat.fromPoints(asString(0));
        CapCom.println("-- Dumping Stat Calculations --");
        CapCom.println("Count 1, 2: \t"+myStat.count()+", "+myStat.yCount());
        CapCom.println("Mean 1, 2: \t"+myStat.mean()+", \t"+myStat.yMean());
        CapCom.println("Pop Var 1, 2: \t"+myStat.pVar()+", \t"+myStat.ypVar());
        CapCom.println("Samp Var 1, 2: \t"+myStat.sVar()+", \t"+myStat.ysVar());
        CapCom.println("Pop S Dev 1, 2: \t"+myStat.pDev()+", \t"+myStat.ypDev());
        CapCom.println("Samp SD 1, 2: \t"+myStat.sDev()+", \t"+myStat.ysDev());
        CapCom.println("Correlation: \t"+myStat.correlation());
        CapCom.println("Regr. Line: \t"+myStat.regressionLine());
        break;
      case 'E':
        myStat = MathStat.newRandomInstance(asInt(0), asDouble(1), asDouble(2), asBoolean(3), asInt(4), asDouble(5), asDouble(6), asBoolean(7));
        CapCom.println(myStat.format(MathStat.FORMAT_DEBUG, ", ", ", "));
        break;
      case 'F':
        myStat = MathStat.newLinearInstance(asInt(0), asDouble(1), asDouble(2), asDouble(3), asDouble(4), asInt(5), asBoolean(6));
        CapCom.println(myStat.format(MathStat.FORMAT_DEBUG, ", ", ", "));
        break;
      case 'G':
        
        break;
      case 'H':
        
        break;
      case 'I':
        
        break;
      case 'J':
        
        break;
        
      default:
    }
  }
}

