package cox.jmatt.java.MathTools.demo;

import java.awt.*;
import cox.jmatt.java.MathTools.*;
/** <p>This PizzaPanel demonstrates MathXML and MTXPro methods. The parameter types are specified on the Choice component and the indicated method is called by 
    clicking the [Go!] button.</p>
    <p>The demos here are:
      <ol type="A" >
        <li>All dangerous XML characters in the parameter area are escaped: greater- and less-than brackets, ampersand, quote, and apostrophe. They are replaced 
        with their XML entity escapes. This is automatically sent to <code>processMathObject()</code>, which in Basic MathPrinter sends it to the clipboard. Clearing 
        the area first allows arbitrary text to be XML-escaped.</li>
        <li>String.0 is wrapped in &lt;math&gt; tags. String.1 is an optional 'display=' attribute. The only accepted values are 'block' and 'inline' but the method 
        does not enforce this.</li>
        <li>Call wrapTag() with String.0 as the tag name and String.1 as the content. The tags are formed and the content put between them without checking or 
        escaping XML entities; use with caution!</li>
        <li>Demonstrate wrapMN() by wrapping String.0 in number tags.</li>
        <li>Demonstrate wrapMI() by wrapping String.0 in identifier (variable) tags.</li>
        <li>Demonstrate wrapMO() by wrapping String.0 in operation tags.</li>
        <li>Wrap String.0 in &lt;mrow&gt; tags. Content is NOT escaped before it is added!</li>
        <li>Parse the entire parameter panel content and convert it to MathML. The characters and other tokens MUST be space-separated, and parser format tokens can 
        be included. For a list of format tokens see {@link cox.jmatt.java.MathTools.MathXML#parseMathML MathXML.parseMathML()}. Multiple lines are preserved 
        but not &lt;mrow&gt;'ed. Any blank lines are ignored.</li>
        <li>Create a fraction tag, with or without a whole part. String.0 is the whole part, String.1 is the numerator and String.2 is the denominator. These Strings 
        ARE parsed, so format tokens are acceptable. The resulting content should be automatically interpreted and tagged correctly.</li>
        <li>Look up the <code>MathSymbol</code> specified in String.0 and print information about it.</li>
        <li>(MTXProcessor) Check the file specified in String.0 for structural errors; i.e. parse via <code>checkXML()</code>.</li>
        <li>(MTXProcessor) Parse the file specified in String.0 for MathTools markup and print the results.</li>
        <li>(MTXProcessor) Transform the file String.0 using the stylesheet String.1. If the '... not available...' message appears the Math ClassLoader must be 
        set explicitly from an URLLoaderPanel (Scripting or MathDBC demo). As long as the application is not termiated the Math ClassLoader remains active. This 
        demo automatically uses it and always checks for availability before transforming. Similarly the Global Pizza must be set manually if it is to be used. 
        NOTE: The entire file paths must be entered. The result of the transform is printed and sent to <code>processMathObject()</code>; BasicMathPrinter will 
        send it to the clipboard.</li>
      </ol></p>
*/
public class MathXMLPizza extends PizzaPanel {

  private Choice myPicker;

  public Panel getNorthPanel() {
    myPicker = new Choice();
    Panel pnlTemp = new Panel(new BorderLayout());
    pnlTemp.add(new Label("MathXML Test Pizza", Label.CENTER), BorderLayout.NORTH);
    pnlTemp.add(myPicker, BorderLayout.SOUTH);
    
    myPicker.add("A. Call escapeXML() on the panel contents.");
    myPicker.add("B. Call wrapMath(s.0, s.1).");
    myPicker.add("C. Call wrapTag(s.0, s.1). Generate '<s.0 > s.1 </s.0>'.");
    myPicker.add("D. Number tag: wrapMN(s0).");
    myPicker.add("E. Identifier (variable) tag: wrapMI(s0).");
    myPicker.add("F. Operation tag: wrapMO(s0).");
    myPicker.add("G. MathML 'mrow' tag: wrapRow(s0).");
    myPicker.add("H. MathML expression: parseMathML() on the panel contents.");
    myPicker.add("I. Fraction expression: mFrac(s.0, s.1, s.2).");
    myPicker.add("J. Information on MathSymbol(String.0).");
    myPicker.add("K. MTXPro.checkXML() on the file s.0.");
    myPicker.add("L. MTXPro.parseFile(s.0) and dump the resulting list.");
    myPicker.add("M. MTXPro.transformFiles(s.0, s.1). SEE DOCS!!");
    
   return pnlTemp;
  }

  public void processGoButton() {
    String sChoice = myPicker.getSelectedItem()+"  ";
    eatPizza();
    int iLow = 0, iHigh = 0;
    double dTemp = 0.0;
    boolean bTemp = false;
    String strTemp = null;
    StringBuffer sbTemp = null;
    String[] linez = null;
    MathXML.MathSymbol mySymbol = null;
    cox.jmatt.java.MathTools.util.MTXProcessor myPro = new cox.jmatt.java.MathTools.util.MTXProcessor();
    switch(sChoice.charAt(0)) {
      case 'A':
        strTemp = MathXML.escapeXML(getRawPizza());
        CapCom.println(strTemp);
        CapCom.processMathObject(strTemp);
        break;
      case 'B':
        CapCom.println(MathXML.wrapMath(asString(0), asString(1)));
        break;
      case 'C':
        CapCom.println(MathXML.wrapTag(asString(0), asString(1)));
        break;
      case 'D':
        CapCom.println(MathXML.wrapMN(asString(0)));
        break;
      case 'E':
        CapCom.println(MathXML.wrapMI(asString(0)));
        break;
      case 'F':
        CapCom.println(MathXML.wrapMO(asString(0)));
        break;
      case 'G':
        CapCom.println(MathXML.wrapRow(asString(0)));
        break;
      case 'H':
        sbTemp = new StringBuffer();
        linez = getRawPizza().split("\n");
        for(int i = 0; i < linez.length; i++) {
          if(Question.validString(linez[i])) sbTemp.append(MathXML.parseMathML(linez[i])).append("\n");
        }
        CapCom.println(sbTemp.toString());
        CapCom.processMathObject(sbTemp.toString());
        break;
      case 'I':
        CapCom.println(MathXML.mFrac(asString(0), asString(1), asString(2)));
        break;
      case 'J':
        mySymbol = MathXML.getSymbol(asString(0));
        CapCom.println("The value of "+mySymbol.toString()+" = "+mySymbol.decValue());
        CapCom.println("Unicode entity: "+MathXML.getEntity(asString(0)));
        CapCom.println("As char: "+(mySymbol.charValue()));
        break;
      case 'K':
        myPro.checkXML(asString(0), false);
        break;
      case 'L':
        String sFile = asString(0);
        try {
          java.util.List lTemp = myPro.parseFile(sFile);
          for(Object obj : lTemp) { CapCom.println((obj == null)?"null":obj.toString()); }
        } catch(Exception ex) { CapCom.error("Error parsing XML file: "+sFile, ex); }
        break;
      case 'M':
        bTemp = myPro.isTransformAvailable();
        CapCom.println(bTemp?"- - Starting XSL Transform - -":"- - XSLT is not available - -");
        if(!bTemp) break;
        myPro.setMathClassLoader(true);
        myPro.setParametersFromGlobalPizza();
        strTemp = myPro.transformFiles(asString(0), asString(1));
        CapCom.println(strTemp);
        CapCom.processMathObject(strTemp);
        break;
      case 'N':

        break;
      case 'O':

        break;
      
      default:
    }
  }
}
