package cox.jmatt.java.MathTools.demo;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.util.NoteworthyParser;

/** <p>This class demonstrates the use of the {@link cox.jmatt.java.MathTools.util.NoteworthyParser NoteworthyParser} class. Simply stated, <b>Noteworthy</b> 
    allows HTML content to be embedded into and extractef from easily-readable and structured plain text lecture notes. To use this demo load or type some 
    Noteworthy text into the <code>TextArea</code> and hit the '[Parse]' button. The output will be displayed on the printer and, optionall, copied to the 
    clipboard.</p>
*/
public class NoteworthyDemo implements ActionListener {

  private FileDialog myLoadDialog = null;
  private TextArea txaScript = null;
  private Button btnLoad = null, btnClear = null, btnGo = null;
  private Checkbox cbxClip = null, cbxClear = null;

  /** Simple constructor, 'Load' button not available. */
  public NoteworthyDemo() { super(); }

  /** Build around a Dialog. 'Load' button available. */
  public NoteworthyDemo(Dialog pParent) {
    super();
    myLoadDialog = new FileDialog(pParent, "Load Noteworthy file...", FileDialog.LOAD);
  }

  /** Build around a Frame. 'Load' button available. */
  public NoteworthyDemo(Frame pParent) {
    super();
    myLoadDialog = new FileDialog(pParent, "Load Noteworthy file...", FileDialog.LOAD);
  }

  public void actionPerformed(ActionEvent aev) {
    if(aev.getSource() == btnLoad) {
      loadFile();
    } else if(aev.getSource() == btnClear) {
      clearContent(!cbxClear.getState());
      cbxClear.setState(true);
    } else if(aev.getSource() == btnGo) {
      parseDocument();
    } else {
      CapCom.debug("Unknown button: "+aev.getActionCommand());
    }
  }

  /** Build and return the GUI panel. */
  public final Panel standardGUI() {
    Panel retval = new Panel(), pnlTemp;
    retval.setLayout(new BorderLayout());
    retval.add(new Label("Noteworthy Parser Demo", Label.CENTER), BorderLayout.NORTH);
    txaScript = new TextArea(); retval.add(txaScript, BorderLayout.CENTER);
    pnlTemp = new Panel(new GridLayout(1,0));
    if(myLoadDialog == null) {
      pnlTemp.add(new Label(" "));
    } else {
      btnLoad = new Button("Load");
      btnLoad.addActionListener(this);
      pnlTemp.add(btnLoad);
    }
    btnGo = new Button("Parse"); btnGo.addActionListener(this); pnlTemp.add(btnGo);
    cbxClip = new Checkbox("process(result)", true); pnlTemp.add(cbxClip);
    cbxClear = new Checkbox("Lock", true); pnlTemp.add(cbxClear);
    btnClear = new Button("Clear"); btnClear.addActionListener(this); pnlTemp.add(btnClear);
    retval.add(pnlTemp, BorderLayout.SOUTH);
    return retval;
  }

  /** Load a file from the <code>FileDialog</code> if one is defined. */
  public void loadFile() {
    if(myLoadDialog == null) return;
    myLoadDialog.setVisible(true);
    String theDir = myLoadDialog.getDirectory();
    if(Question.validString(theDir)) loadFile(theDir + myLoadDialog.getFile());
  }

  /** Load a file into the parser area. */
  public void loadFile(String pFile) {
    if(!Question.validString(pFile)) return;
    StringBuffer retval = new StringBuffer();
    try {
      FileReader frIn = new FileReader(pFile);
      int theChar = 0;
      while((theChar = frIn.read()) > -1) retval.append((char) theChar);
      frIn.close();
      txaScript.setText(retval.toString());
    } catch(Exception ex) { CapCom.error("Error loading file: "+pFile, ex); }
  }

  /** Set the content to be parsed. Ignored if null or blank. */
  public void setContent(String pContent) {
    if(Question.validString(pContent)) txaScript.setText(pContent);
  }

  /** Fetch the content of the parser area. */
  public String getContent() { return txaScript.getText(); }

  /** Clear the parser area, if bClear is 'true'. */
  public void clearContent(boolean bClear) {
    if(bClear) txaScript.setText("");
  }

  /** Parse the document in the parser area into HTML. */
  public void parseDocument() {
    CapCom.debug("Noteworthy parse starting");
    NoteworthyParser theParser = new NoteworthyParser();
    String theResult = theParser.parse(getContent()).toString();
    CapCom.println(theResult);
    if(cbxClip.getState()) CapCom.processMathObject(theResult);
  }

}

