package cox.jmatt.java.MathTools.demo;

import java.awt.*;
import cox.jmatt.java.MathTools.*;
/** <p>This PizzaPanel demonstrates Polynomial methods. The parameter types are specified on the Choice component and the indicated method is called by clicking 
    the [Go!] button.</p>
    <p>The format used for entering Polynomials is {@link cox.jmatt.java.MathTools.Polynomial#createPolynomial here} and it is consistent throughout the demos. 
    The demos in this Panel are:
    <ol type="A">
      <li>Create a new Polynomial from String.0 and print it via its toString() method. The input and output will be very similar BUT if no variable or function 
      name is supplied, the defaults will appear on the output.</li>
      <li>Format Polynomial(String.0) and print it. Boolean.1 true prints in ascending order and boolean.2 prints it as a polynomial function. Then its XML form 
      is printed. String.3, if present, becomes the tag's ID. NOTE: This is <b>not</b> the same as its MathML representation!</li>
      <li>Format Polynomial(String.0) and then format it per the term template in String.1, boolean.2 ascending and boolean.3 to include zero terms.</li>
      <li>Create a random Polynomial by calling setPolyKnobs() with String.0 as the Knobs to use.</li>
      <li>Add Polynomial(String.0) to Polynomial(String.1).</li>
      <li>Subtract Polynomial(String.1) from Polynomial(String.0).</li>
      <li>Multiply Polynomial(String.0) by the constant int.1.</li>
      <li>Multiply Polynomial(String.0) by Polynomial(String.1).</li>
      <li>Evaluate Polynomial(String.0) as a function at the value int.1.</li>
      <li>Print the first and second derivative of Polynomial(String.0)</li>
      <li>Call the createQuotient() method on Polynomial(String.0) and Polynomial(String.1). String.2 is the division indicator, String.3 is the left fence and 
      String.4 is the right fence.</li>
      <li>Convert Polynomial(String.0) to MathML markup. Boolean.1 true formats in ascending order and boolean.2 true activates 'smart' mode. In smart mode the 
      exponent is not rendered if it is 1 and neither the exponent nor variable appear on the constant term.</li>
      <li><code>unFormat()</code> Polynomial(String.0) based on the formatting String in String.1.</li>
      <li>Create Polynomial function F(x) from s.0, G(x) from s.1 and evaluate F(G(x)).</li>
      <li>Print the relation <code>formatEquation()</code> with Polynomial(String.0) and Polynomial(String.1). The relation is String.2 and the fence Strings are 
      String.3 and String.4.</li>
      <li>Create Polynomials from String.0 and String.1 and test them for equality per the <code>equals()</code> method.<li>
    </ol></p>
*/
public class PolynomialPizza extends PizzaPanel {

  private Choice myPicker;

  public Panel getNorthPanel() {
    myPicker = new Choice();
    Panel pnlTemp = new Panel(new BorderLayout());
    pnlTemp.add(new Label("Polynomial Test Pizza", Label.CENTER), BorderLayout.NORTH);
    pnlTemp.add(myPicker, BorderLayout.SOUTH);
    
    myPicker.add("A. new Polynomial(s.0).toString()");
    myPicker.add("B. createPolynomial(s.0) then printPolynomial(b.1, b.2), toXML(s.3) and as LaTeX.");
    myPicker.add("C. Create on s.0, then format(s.1, b.2, b.3).");
    myPicker.add("D. setPolyKnobs(s.0) then print it out.");
    myPicker.add("E. Add: Polynomial(s.0) + Polynomial(s.1).");
    myPicker.add("F. Subtract: Polynomial(s.0) - Polynomial(s.1).");
    myPicker.add("G. Multiply: Polynomial(s.0) by constant i.1.");
    myPicker.add("H. Multiply: Polynomial(s.0).multiply(Polynomial(s.1)).");
    myPicker.add("I. Evaluate Polynomial(s.0) at i.1.");
    myPicker.add("J. Create Polynomial(s.0) then show the first and second derivatives.");
    myPicker.add("K. Call createQuotient(Poly(s.0), Poly(s.1), s.2, s.3, s.4) and print the Question.");
    myPicker.add("L. Print Polynomial(s.0) as MathML, b.1 true is ascending and b.2 true is smart mode.");
    myPicker.add("M. Polynomial(s.0).unFormat(s.1, b.2).");
    myPicker.add("N. Evaluate Polynomial(s.0) (f) at Polynomial(s.1) (g).");
    myPicker.add("O. Print formatEquation(Poly(s.0), Poly(s.1), s.2, s.3, s.4).");
    myPicker.add("P. Print Poly(s.0).equals(Poly(s.1)).");
    
   return pnlTemp;
  }

  public void processGoButton() {
    String sChoice = myPicker.getSelectedItem()+"  ";
    eatPizza();
    Polynomial poly1 = new Polynomial(asString(0));
    Polynomial poly2 = new Polynomial(asString(1));
    switch(sChoice.charAt(0)) {
      case 'A':
        CapCom.println(poly1.toString());
        break;

      case 'B':
        poly1 = Polynomial.createPolynomial(asString(0));
        CapCom.println(poly1.printPolynomial(asBoolean(1), asBoolean(2)));
        CapCom.println("XML:\t"+poly1.toXML(asString(3)));
        CapCom.println("LaTeX:\t"+poly1.format(Polynomial.FORMAT_LATEX, asBoolean(1), asBoolean(2)));
        break;
      case 'C':
        CapCom.println(poly1.format(asString(1), asBoolean(2), asBoolean(3)));
        break;
      case 'D':
        CapCom.println(Polynomial.createPolyKnobs(asString(0)).toString());
        break;
      case 'E':
        CapCom.println((poly1.add(poly2)).toString());
        break;
      case 'F':
        CapCom.println((poly1.subtract(poly2)).toString());
        break;
      case 'G':
        CapCom.println(poly1.multiply(asInt(1)).toString());
        break;
      case 'H':
        CapCom.println(poly1.multiply(poly2).toString());
        break;
      case 'I':
        CapCom.println("P("+asInt(1)+") = "+poly1.evaluate(asInt(1)));
        break;
      case 'J':
        CapCom.println(poly1.printPolynomial(false, true));
        poly1 = poly1.derivative();
        CapCom.println(poly1.printPolynomial(false, true));
        poly1 = poly1.derivative();
        CapCom.println(poly1.printPolynomial(false, true));
        break;
      case 'K':
        CapCom.println("Format Quotient");
        CapCom.println(Polynomial.formatQuotient(poly1, poly2, asString(2), asString(3), asString(4)).format("Prob:\t<1>\nAns:\t<2>"));
        break;
      case 'L':
        CapCom.println(poly1.toMathML(asBoolean(1), asBoolean(2)));
        break;
      case 'M':
        CapCom.println("Polynomial: "+poly1.printPolynomial(false, false));
        CapCom.println(poly1.unFormat(asString(1), asBoolean(2)));
        break;
      case 'N':
        poly1.setFName("F"); poly2.setFName("G");
        CapCom.println((poly1.evaluate(poly2)).printPolynomial(false, true));
        break;
      case 'O':
        CapCom.println(Polynomial.formatEquation(poly1, poly2, asString(2), asString(3), asString(4)));
        break;
      case 'P':
        CapCom.println("Equals: "+poly1.equals(poly2));
        break;
      case 'Q':

        break;
      case 'R':

        break;
      
      default:
    }
  }
}
