package cox.jmatt.java.MathTools.demo.gpgdemo;

import cox.jmatt.java.MathTools.CapCom;
import cox.jmatt.java.MathTools.markup.svg.SVGTag;
import cox.jmatt.java.MathTools.demo.GPGenerator;

/** <p>This class exists to provide scripts with access to the various <code>GraphPanel</code> subclasses. These are the static instances set and maintained in 
    <code>CapCom</code>. If MathTools is run from its JAR then these instances <i>are</i> the ones in the 'Graph Paper' demo panel. Setting values in a script 
    <b>will</b> affect the ones in the GUI and vise versa. Prudent use of the <code>reset()</code> method is critical!</p>
    <p>If the scripting framework is embedded in a separate program or if the 'Graph Paper' demo panel is not defined (for whatever reason) instances are created 
    as required.</p>
*/
public class GraphPanelFactory {

  public GraphPanelFactory() { super(); }

  /** Set or clear the <code>GPGenerator.GP_DEBUG</code> field. */
  public static final void setGPDebugMode(boolean pEnable) { cox.jmatt.java.MathTools.demo.GPGenerator.GP_DEBUG = pEnable; }
  /** Instance debug mode method. */
  public final void setGPDebug(boolean pEnable) { cox.jmatt.java.MathTools.demo.GPGenerator.GP_DEBUG = pEnable; }

  /** Add the {@link cox.jmatt.java.MathTools.demo.GPGenerator#addPens standard pens} to a <code>SVGTag</code>. */
  public void addPens(SVGTag pTag) { GPGenerator.addPens(pTag); }

  /** Set the Pizzaria area on the <code>GPGenerator</code> if it exists. If not, nothing happens. */
  public void setMarginPizza(String pPizza) {
    if(GPickle.gpgMain != null) GPickle.gpgMain.setPizza(pPizza);
  }

  /** Fetch the margin panel config, if it exists. If not an error String is returned. */
  public String getMarginPizza(String pPrefix) {
    if(GPickle.gpgMain == null) return "# GUI GPGenerator not defined.";
    return GPickle.gpgMain.toPizza(pPrefix);
  }

  /** Read the margin config from the Pizzaria area. */
  public void eatMarginPizza(String pPrefix) {
    if(GPickle.gpgMain != null) GPickle.gpgMain.eatPizza(pPrefix);
  }

  /** Fetch a <code>QuadrillePanel</code>. */
  public QuadrillePanel quadrille() {
    if(GPickle.gpgQuadrillePanel == null) GPickle.gpgQuadrillePanel = new QuadrillePanel();
    return GPickle.gpgQuadrillePanel;
  }

  /** Fetch a <code>PolarPanel</code>. */
  public PolarPanel polar() {
    if(GPickle.gpgPolarPanel == null) GPickle.gpgPolarPanel = new PolarPanel();
    return GPickle.gpgPolarPanel;
  }

  /** Fetch a <code>LogPanel</code>. The method uses the full name to avoid possible confusion with logging. */
  public LogPanel logarithmic() {
    if(GPickle.gpgLogPanel == null) GPickle.gpgLogPanel = new LogPanel();
    return GPickle.gpgLogPanel;
  }

  /** Fetch a <code>LedgerPanel</code>. */
  public LedgerPanel ledger() {
    if(GPickle.gpgLedgerPanel == null) GPickle.gpgLedgerPanel = new LedgerPanel();
    return GPickle.gpgLedgerPanel;
  }

  /** Fetch a <code>BPSPanel</code>. */
  public BPSPanel bps() {
    if(GPickle.gpgBPSPanel == null) GPickle.gpgBPSPanel = new BPSPanel();
    return GPickle.gpgBPSPanel;
  }

  /** Fetch a <code>BarChartPanel</code>. */
  public BarChartPanel barChart() {
    if(GPickle.gpgBarChart == null) GPickle.gpgBarChart = new BarChartPanel();
    return GPickle.gpgBarChart;
  }

  /** Fetch a <code>HexGridPanel</code>. */
  public HexGridPanel hex() {
    if(GPickle.gpgHexGrid == null) GPickle.gpgHexGrid = new HexGridPanel();
    return GPickle.gpgHexGrid;
  }

  /** Fetch a <code>MusicPanel</code>. */
  public MusicPanel music() {
    if(GPickle.gpgMusic == null) GPickle.gpgMusic = new MusicPanel();
    return GPickle.gpgMusic;
  }

  /** Fetch a <code>LetterngPanel</code>. */
  public LetteringPanel lettering() {
    if(GPickle.gpgLettering == null) GPickle.gpgLettering = new LetteringPanel();
    return GPickle.gpgLettering;
  }

  /** Fetch an <code>OffsetQuadPanel</code>. */
  public OffsetQuadPanel offsetQuad() {
    if(GPickle.gpgOffsetQuad == null) GPickle.gpgOffsetQuad = new OffsetQuadPanel();
    return GPickle.gpgOffsetQuad;
  }

  /** Fetch a <code>SymmetricLogPanel</code>. */
  public SymmetricLogPanel symLog() {
    if(GPickle.gpgSymmetricLog == null) GPickle.gpgSymmetricLog = new SymmetricLogPanel();
    return GPickle.gpgSymmetricLog;
  }

  /** Fetch a <code>IsometricPanel</code>. */
  public IsometricPanel isometric() {
    if(GPickle.gpgIsometric == null) GPickle.gpgIsometric = new IsometricPanel();
    return GPickle.gpgIsometric;
  }

  /** Fetch a <code>PerspectivePanel</code>. */
  public PerspectivePanel perspective() {
    if(GPickle.gpgPerspective == null) GPickle.gpgPerspective = new PerspectivePanel();
    return GPickle.gpgPerspective;
  }

}

