package cox.jmatt.java.MathTools.demo.gpgdemo;

import java.awt.*;
import java.util.Properties;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.demo.*;
import cox.jmatt.java.MathTools.markup.svg.*;
/** <p><code>LedgerPanel</code> generates a rectangular grid consisting of evenly-spaced rows and columns of uneven width. The top row is assumed to be column 
    headers; it can be underlined with a different pen ('Row 1 Pen') and can have a different height ('Row 1 Height'). The 'Horizontal Pen' field sets the pen 
    number used for the other horizontal lines and the 'Vertical Pen' field is the pen number used for all vertical lines. Finally the 'Text Pen' field is the pen 
    number used for any text present.</p>
    <p>The columns and their headers are set in the small text area to the right of the other fields. Column entries are of the form '<i>NN</i> Column Text' where 
    <i>NN</i> is the width of the column, followed by a single space, followed by the text to appear at the top of the column. Each line is split on spaces with 
    a limit of two, so the number MUST be first, followed by ONE SPACE. The remainder of the line becomes the column header. Each line describes one column and 
    the lines are taken in the order they appear (contents initially split on newlines). If a column cannot be parsed or has width zero it is ignored.</p>
    <p>The '[]Draw top line' Checkbox determines whether or not a horizontal line appears <i>above</i> the first row and whether the vertical lines will stretch 
    to touch it. Enabling the '[]Stretch last column' Checkbox causes the last column to extend to the right margin. IMPORTANT NOTE: Regardless of the status of 
    this box the last column MUST have a width greater than zero or it will be ignored!</p>
    <p>The <code>clearColumns()</code> and <code>addColumn()</code> methods are defined for scripting purposes. The <code>reset()</code> method resets the default 
    data in the column header area, the 'clear' method clears it. The 'add' method allows scripts to add column definitions one at a time.</p>
*/
public class LedgerPanel extends GraphPanel {

  private TextField txfVertPen = null, txfHorizPen = null, txfRowHeight = null, txfFirstPen = null, txfFirstHeight = null, txfTextPen = null;
  private Checkbox cbxStretch = null, cbxTopLine = null;
  private TextArea txaColumns = null;

  /** Basic constructor. */
  public LedgerPanel() {
    super();
    buildGUI();
    GPickle.gpgLedgerPanel = this;
  }

  private void buildGUI() {
    txfVertPen = new TextField("0");
    txfHorizPen = new TextField("0");
    txfRowHeight = new TextField("25");
    txfFirstPen = new TextField("25");
    txfFirstHeight = new TextField("30");
    txfTextPen = new TextField("22");
    cbxStretch = new Checkbox("Stretch last column");
    cbxTopLine = new Checkbox("Draw top line", true);
    txaColumns = new TextArea(8, 30);
    setLayout(new GridBagLayout());
    GridBagConstraints gbc = new GridBagConstraints(); gbc.insets = new Insets(1,1,1,1);
    gbc.gridx = 1; gbc.gridy = 1; gbc.fill = GridBagConstraints.HORIZONTAL;
    add(new Label("Vert. Pen #"), gbc); gbc.gridx++;
    add(txfVertPen, gbc); gbc.gridx=1; gbc.gridy++;
    add(new Label("Horiz. Pen #"), gbc); gbc.gridx++;
    add(txfHorizPen, gbc); gbc.gridx=1; gbc.gridy++;
    add(new Label("Row Height"), gbc); gbc.gridx++;
    add(txfRowHeight, gbc); gbc.gridx=1; gbc.gridy++;
    add(new Label("Row 1 Pen"), gbc); gbc.gridx++;
    add(txfFirstPen, gbc); gbc.gridx=1; gbc.gridy++;
    add(new Label("Row 1 Height"), gbc); gbc.gridx++;
    add(txfFirstHeight, gbc); gbc.gridx=1; gbc.gridy++; gbc.gridwidth = 2;
    add(cbxStretch, gbc); gbc.gridy++;
    add(cbxTopLine, gbc);
    gbc.gridy = 1; gbc.gridx = 3; gbc.gridheight = 1; gbc.gridwidth = 1;
    add(new Label("Text Pen"), gbc); gbc.gridx++;
    add(txfTextPen, gbc); gbc.gridx = 3; gbc.gridwidth = 2;
    gbc.gridy++; gbc.gridheight = GridBagConstraints.REMAINDER;
    add(txaColumns, gbc);
    txaColumns.setText("200 Name\n50\n400 Mailing Address\n");
  }

  /** Reset to defaults. Includes the sample column headers. */
  public void reset() {
    super.reset();
    txfVertPen.setText("0"); txfHorizPen.setText("0"); txfRowHeight.setText("25"); txfFirstPen.setText("25");
    txfFirstHeight.setText("30"); txfTextPen.setText("22");
    cbxStretch.setState(false); cbxTopLine.setState(true);
    txaColumns.setText("200 Name\n50\n400 Mailing Address");
  }

  /** Draw the graph. */
  public void drawGraph(int mTop, int mBottom, int mLeft, int mRight, SVGTag pPaper) {
    if(pPaper == null) return;
    String[] sCols = (txaColumns.getText()).split("\n"), sParts = null;
    if((sCols == null) || (sCols.length < 1)) {
      CapCom.error("No columns set");
      return;
    }
    MathDrawSVG MDraw = cox.jmatt.java.MathTools.demo.GPGenerator.getMDraw();
    int vPen = getPen(txfVertPen), hPen = getPen(txfHorizPen);
    int textPen = getPen(txfTextPen, 22), rowHeight = asInt(txfRowHeight.getText(), 25, 5, null);
    int firstHeight = asInt(txfFirstHeight.getText(), 30, 10, null), firstPen = getPen(txfFirstPen, 25);
    int yMin = mTop + firstHeight, xMax = 0, pos = 0, width = 0;
    mBottom -= (mBottom - mTop - firstHeight) % rowHeight;
    MDraw.setLinePen(vPen);
    TextTag texTag = MDraw.getTextTag(null, (double) (yMin - 7), null);
    texTag.setPen(textPen);
    if(cbxTopLine.getState()) {
      yMin = mTop;
      MDraw.setLinePen(vPen);
    }
    pos = mLeft;
    for(int i = 0; i < sCols.length; i++) {
      pPaper.drawLine((double) pos, (double) yMin, (double) pos, (double) mBottom);
      sParts = sCols[i].split(" ", 2);
      width = 0;
      try { width = Integer.parseInt(sParts[0]); } catch(Exception ex) { CapCom.debug("Error parsing column: "+sCols[i], ex); }
      if(width == 0) continue; // Zero length. Boring!
      if((sParts.length > 1) && Question.validString(sParts[1])) pPaper.add(texTag.setData((double) (pos + width/2), null, sParts[1]));
      pos += width;
    }
    if(cbxStretch.getState()) pos = mRight;
    pPaper.drawLine((double) pos, (double) yMin, (double) pos, (double) mBottom);
    MDraw.setLinePen(firstPen);
    pPaper.drawLine((double) mLeft, (double) mTop + firstHeight, (double) pos, (double) mTop + firstHeight);
    MDraw.setLinePen(hPen);
    pPaper.drawLine((double) mLeft, (double) yMin, (double) pos, (double) yMin);
    for(yMin = mTop + firstHeight; yMin <= mBottom; yMin += rowHeight) {
      pPaper.drawLine((double) mLeft, (double) yMin, (double) pos, (double) yMin);
    }
  }

  /** Set the pens used to draw the chart.
      @param pHorizontal The pen used for (standard) horizontal lines.
      @param pVertical The pen used for all vertical lines.
      @param pFirst The pen used to underline the first row (header pen).
      @param pText The pen used for column header text.
  */
  public void setPens(int pHorizontal, int pVertical, int pFirst, int pText) {
    txfVertPen.setText("" + pVertical);
    txfHorizPen.setText("" + pHorizontal);
    txfFirstPen.setText("" + pFirst);
    txfTextPen.setText("" + pText);
  }

  /** Set the columns for the chart. This must be a String of newline-separated column descriptors, each of which is of the form '<i>NN</i> Column Text' where 
      <i>NN</i> is the column width in 1/100ths of an inch and 'Column Text' is the column header text. The text is optional and any descriptor that cannot be 
      parsed is ignored.
      @param pHeaderText The String header formatted as described above. Ignored if null or blank.
  */
  public void setColumns(String pHeaderText) {
    if(Question.validString(pHeaderText)) txaColumns.setText(pHeaderText);
  }

  /** This scripting-only method clears the entire column header area. */
  public void clearColumns() { txaColumns.setText(""); }

  /** This scripting method adds a single entry to the column header area. If no text is given it is blank. If the number is zero or less nothing happens.
      @param pWidth The width of the column to add.
      @param pHeader The column label. If null or blank the column header is blank.
  */
  public void addColumn(int pWidth, String pHeader) {
    if(pWidth < 1) return;
    txaColumns.append(""+pWidth+(Question.validString(pHeader)?" "+pHeader.trim():"")+"\n");
  }

  /** Enable (true) or disable (false) the top line and boxes around the header row. */
  public void setTopLine(boolean pEnableTopLine) { cbxTopLine.setState(pEnableTopLine); }

  /** Set whether or not the last column does (true) or does not (false) stretch to the right margin. */
  public void setStretch(boolean pStretchLastColumn) { cbxStretch.setState(pStretchLastColumn); }

  /** Set the row height and the first row height.
      @param pRowHeight The height of all rows after the first.
      @param pFirstHeight The height of the first (header) row.
  */
  public void setHeights(int pRowHeight, int pFirstHeight) {
    txfRowHeight.setText("" + pRowHeight);
    txfFirstHeight.setText("" + pFirstHeight);
  }

  public String toPizza(String pPrefix) {
    StringBuffer retval = new StringBuffer("# LedgerPanel configuration data.");
    if(Question.validString(pPrefix)) {
      if(!pPrefix.endsWith(".")) pPrefix += ".";
      retval.append(" Prefix: ").append(pPrefix);
    } else {
      pPrefix = "";
    }
    retval.append("\n");
    retval.append(getKey(txfVertPen, pPrefix, "ledger.pen.vertical"));
    retval.append(getKey(txfHorizPen, pPrefix, "ledger.pen.horizontal"));
    retval.append(getKey(txfFirstPen, pPrefix, "ledger.pen.hFirst"));
    retval.append(getKey(txfTextPen, pPrefix, "ledger.pen.text"));
    retval.append(getKey(txfRowHeight, pPrefix, "ledger.rowHeight"));
    retval.append(getKey(txfFirstHeight, pPrefix, "ledger.pen.firstHeight"));
    retval.append(getKey(cbxStretch, pPrefix, "ledger.stretchLast"));
    retval.append(getKey(cbxTopLine, pPrefix, "ledger.topLine"));
    String[] linez = txaColumns.getText().split("\n");
    for(int i=0; i < linez.length; i++) { retval.append(pPrefix).append("ledger.column.").append(i).append("=").append(linez[i]).append("\n"); }
    return retval.toString();
  }

  public void eatPizza(Properties pPizza, String pPrefix) {
    if((pPizza == null) || pPizza.isEmpty()) return;
    if(Question.validString(pPrefix)) {
      if(!pPrefix.endsWith(".")) pPrefix += ".";
    } else {
      pPrefix = "";
    }
    txfVertPen.setText(pPizza.getProperty(pPrefix+"ledger.pen.vertical", "0"));
    txfHorizPen.setText(pPizza.getProperty(pPrefix+"ledger.pen.horizontal", "0"));
    txfFirstPen.setText(pPizza.getProperty(pPrefix+"ledger.pen.hFirst", "25"));
    txfTextPen.setText(pPizza.getProperty(pPrefix+"ledger.pen.text", "22"));
    txfRowHeight.setText(pPizza.getProperty(pPrefix+"ledger.rowHeight", "25"));
    txfFirstHeight.setText(pPizza.getProperty(pPrefix+"ledger.firstHeight", "30"));
    cbxStretch.setState("true".equalsIgnoreCase(pPizza.getProperty(pPrefix+"ledger.stretchLast", "false")));
    cbxTopLine.setState("true".equalsIgnoreCase(pPizza.getProperty(pPrefix+"ledger.topLine", "true")));
    
    setKey(pPizza, txfVertPen, pPrefix, "ledger.pen.vertical", "0");
    setKey(pPizza, txfHorizPen, pPrefix, "ledger.pen.horizontal", "0");
    setKey(pPizza, txfFirstPen, pPrefix, "ledger.pen.hFirst", "25");
    setKey(pPizza, txfTextPen, pPrefix, "ledger.pen.text", "22");
    setKey(pPizza, txfRowHeight, pPrefix, "ledger.rowHeight", "25");
    setKey(pPizza, txfFirstHeight, pPrefix, "ledger.pen.firstHeight", "30");
    setKey(pPizza, cbxStretch, pPrefix, "ledger.stretchLast");
    setKey(pPizza, cbxTopLine, pPrefix, "ledger.topLine");
    
    int i=0;
    String sLine = null;
    while((sLine = pPizza.getProperty(pPrefix+"ledger.column."+i, null)) != null) {
      txaColumns.append(sLine+"\n");
      i++;
    }
  }

}

