package cox.jmatt.java.MathTools.demo.gpgdemo;

import java.awt.*;
import java.util.Properties;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.demo.*;
import cox.jmatt.java.MathTools.markup.svg.*;

/** <p>This panel generates logarithmic and semilog graph paper. Either or both scales can be linear or logarithmic, but setting both to linear is the same as 
    standard quadrille paper. Every aspect of the paper is configurable.The '[]Linear' Checkboxes ('Scale Type') cause that particular scale to be linear. The 
    minor pen is used for linear scale and the 'Initial Interval' is used for the (fixed and constant) length of the linear scale.</p>
    <p>Logarithmic scales are rendered as a fixed number of cycles all of constant length but decreasing interval size. The settings are used as follows:
      <ul>
        <li><b>Number of Cycles</b> is the number of major divisions. The cycles repeat until this number is reached or space runs out.</li>
        <li><b>Cycle Length</b> is the number of (minor) divisions within each cycle. The number of lines drawn is actually one less than this number.</li>
        <li><b>Initial Interval</b> is the size of the first interval in each cycle. This value decreases as the cycle progresses. For linear scales this is the 
          length of each interval.</li>
        <li><b>Scale Factor</b> is the multiplier for scale size. It must be between 0.1 and 2.0, this is enforced. The current cycle interval equals the product 
          of this value and the previous interval.</li>
        <li>The <b>Major Pen Number</b> is used for the initial line of each cycle. Subsequent lines use the <b>Minor Pen</b>.</li>
      </ul></p>
    <p>The default settings have a logarithmic vertical scale (horizontal lines) and a quarter-inch linear horizontal scale (vertical lines). For a decent 
    horizontal log scale use 4 cycles at length 10 with an interval of 55.</p>
*/
public class LogPanel extends GraphPanel {

  private Checkbox cbxVertLinear = null, cbxHorizLinear = null;
  private TextField txfNumHCycles = null, txfNumVCycles = null, txfHorizCycLen = null, txfVertCycLen = null, txfHInterval = null, txfVInterval = null;
  private TextField txfHScale = null, txfVScale = null, txfPenVMaj = null, txfPenHMaj = null, txfPenVMin = null, txfPenHMin = null;

  /** Basic constructor. */
  public LogPanel() {
    super();
    buildGUI();
    GPickle.gpgLogPanel = this;
  }

  private void buildGUI() {
    cbxVertLinear = new Checkbox("Linear"); cbxHorizLinear = new Checkbox("Linear", true);
    txfNumHCycles = new TextField("4"); txfNumVCycles = new TextField("5");
    txfHorizCycLen = new TextField("10"); txfVertCycLen = new TextField("10");
    txfHInterval = new TextField("25"); txfVInterval = new TextField("55");
    txfHScale = new TextField("0.8"); txfVScale = new TextField("0.8");
    txfPenVMaj = new TextField("25"); txfPenHMaj = new TextField("25");
    txfPenVMin = new TextField("0"); txfPenHMin = new TextField("0");
    setLayout(new GridLayout(0, 3));
    add(new Label(" ")); add(new Label("Horizontal")); add(new Label("Vertical"));
    add(new Label("Scale Type")); add(cbxHorizLinear); add(cbxVertLinear);
    add(new Label("Total Cycles")); add(txfNumHCycles); add(txfNumVCycles);
    add(new Label("Cycle Length")); add(txfHorizCycLen); add(txfVertCycLen);
    add(new Label("Initial Interval")); add(txfHInterval); add(txfVInterval);
    add(new Label("Scale Factor")); add(txfHScale); add(txfVScale);
    add(new Label("Major Line Pen")); add(txfPenHMaj); add(txfPenVMaj);
    add(new Label("Minor Line Pen")); add(txfPenHMin); add(txfPenVMin);
  }

  public String toPizza(String pPrefix) {
    StringBuffer retval = new StringBuffer("# LogPanel configuration data.");
    if(Question.validString(pPrefix)) {
      if(!pPrefix.endsWith(".")) pPrefix += ".";
      retval.append(" Prefix: ").append(pPrefix);
    } else {
      pPrefix = "";
    }
    retval.append("\n");
    retval.append(getKey(cbxHorizLinear, pPrefix, "log.isLinear.horizontal"));
    retval.append(getKey(cbxVertLinear, pPrefix, "log.isLinear.vertical"));
    retval.append(getKey(txfNumHCycles, pPrefix, "log.cycles.horizontal"));
    retval.append(getKey(txfHorizCycLen, pPrefix, "log.length.horizontal"));
    retval.append(getKey(txfHInterval, pPrefix, "log.interval.horizontal"));
    retval.append(getKey(txfHScale, pPrefix, "log.scale.horizontal"));
    retval.append(getKey(txfPenHMaj, pPrefix, "log.pen.major.horizontal"));
    retval.append(getKey(txfPenHMin, pPrefix, "log.pen.minor.horizontal"));
    retval.append(getKey(txfNumVCycles, pPrefix, "log.cycles.vertical"));
    retval.append(getKey(txfVertCycLen, pPrefix, "log.length.vertical"));
    retval.append(getKey(txfVInterval, pPrefix, "log.interval.vertical"));
    retval.append(getKey(txfVScale, pPrefix, "log.scale.vertical"));
    retval.append(getKey(txfPenVMaj, pPrefix, "log.pen.major.vertical"));
    retval.append(getKey(txfPenVMin, pPrefix, "log.pen.minor.vertical"));
    return retval.toString();
  }

  public void eatPizza(Properties pPizza, String pPrefix) {
    if((pPizza == null) || pPizza.isEmpty()) return;
    if(Question.validString(pPrefix)) {
      if(!pPrefix.endsWith(".")) pPrefix += ".";
    } else {
      pPrefix = "";
    }
    setKey(pPizza, cbxHorizLinear, pPrefix, "log.isLinear.horizontal");
    setKey(pPizza, cbxVertLinear, pPrefix, "log.isLinear.vertical");
    setKey(pPizza, txfNumHCycles, pPrefix, "log.cycles.horizontal", "4");
    setKey(pPizza, txfHorizCycLen, pPrefix, "log.length.horizontal", "10");
    setKey(pPizza, txfHInterval, pPrefix, "log.interval.horizontal", "25");
    setKey(pPizza, txfHScale, pPrefix, "log.scale.horizontal", "0.8");
    setKey(pPizza, txfPenHMaj, pPrefix, "log.pen.major.horizontal", "25");
    setKey(pPizza, txfPenHMin, pPrefix, "log.pen.minor.horizontal", "0");
    setKey(pPizza, txfNumVCycles, pPrefix, "log.cycles.vertical", "5");
    setKey(pPizza, txfVertCycLen, pPrefix, "log.length.vertical", "10");
    setKey(pPizza, txfVInterval, pPrefix, "log.interval.vertical", "55");
    setKey(pPizza, txfVScale, pPrefix, "log.scale.vertical", "0.8");
    setKey(pPizza, txfPenVMaj, pPrefix, "log.pen.major.vertical", "25");
    setKey(pPizza, txfPenVMin, pPrefix, "log.pen.minor.vertical", "0");
  }

  /** Reset to default values. Everything! */
  public void reset() {
    super.reset();
    cbxVertLinear.setState(false); cbxHorizLinear.setState(true);
    txfNumHCycles.setText("4"); txfNumVCycles.setText("5"); txfHorizCycLen.setText("10"); txfVertCycLen.setText("10");
    txfHInterval.setText("25"); txfVInterval.setText("55"); txfHScale.setText("0.8"); txfVScale.setText("0.8");
    txfPenVMaj.setText("25"); txfPenHMaj.setText("25"); txfPenVMin.setText("0"); txfPenHMin.setText("0");
  }

  /** Draw the graph. */
  public void drawGraph(int mTop, int mBottom, int mLeft, int mRight, SVGTag pPaper) {
    if(pPaper == null) return;
    MathDrawSVG MDraw = cox.jmatt.java.MathTools.demo.GPGenerator.getMDraw();
    int penHMaj = getPen(txfPenHMaj, 25), penVMaj = getPen(txfPenVMaj, 25), penHMin = getPen(txfPenHMin), penVMin = getPen(txfPenVMin);
    int hInterval = asInt(txfHInterval.getText(), 25, 5, null), vInterval = asInt(txfVInterval.getText(), 55, 5, null);
    int hCycles = asInt(txfNumHCycles.getText(), 4, 1, null), vCycles = asInt(txfNumVCycles.getText(), 5, 1, null);
    int hCycLen = asInt(txfHorizCycLen.getText(), 10, 2, null), vCycLen = asInt(txfVertCycLen.getText(), 10, 2, null);
    double hScale = asDouble(txfHScale.getText(), 0.8, 0.1, 2.0), vScale = asDouble(txfHScale.getText(), 0.8, 0.1, 2.0);
    double hMax = 0, vMax = 0, delta = hInterval;
    boolean trucking = true;
    GroupTag theGraph = MDraw.getGroupTag("graph");
    if(cbxHorizLinear.getState()) {
      // Linear horizontal scale, vertical lines.
      MDraw.setLinePen(penVMin);
      for(double pos = mLeft; pos < mRight; pos += delta) {
        theGraph.drawLine(pos, (double) mTop, pos, (double) mBottom);
        hMax = pos;
      }
      hMax -= mLeft;
    } else {
      double pos = mLeft;
      for(int cycle = 0; trucking && (cycle < hCycles); cycle++) {
        delta = hInterval;
        MDraw.setLinePen(penVMaj);
        for(int minorLine = 0; trucking && (minorLine < hCycLen - 1); minorLine++) {
          theGraph.drawLine(pos, (double) mTop, pos, (double) mBottom);
          MDraw.setLinePen(penVMin);
          pos += delta;
          trucking = (pos <= mRight);
          if(trucking) hMax = pos;
          delta *= hScale;
        }
      }
    }
    delta = vInterval;
    trucking = true;
    if(cbxVertLinear.getState()) {
      // Linear vertical scale, horizontal lines
      MDraw.setLinePen(penHMin);
      for(double pos = mBottom; pos > mTop; pos -= delta) {
        theGraph.drawLine((double) mLeft, pos, (double) mRight, pos);
        vMax = pos;
      }
    }  else {
      double pos = mBottom;
      for(int cycle = 0; trucking && (cycle < vCycles); cycle++) {
        delta = vInterval;
        MDraw.setLinePen(penHMaj);
        for(int minorLine = 0; trucking && (minorLine < vCycLen - 1); minorLine++) {
          theGraph.drawLine((double) mLeft, pos, (double) mRight, pos);
          MDraw.setLinePen(penHMin);
          pos -= delta;
          trucking = (pos >= mTop);
          if(trucking) vMax = pos;
          delta *= vScale;
        }
      }
    }
    pPaper.add(theGraph);
    StylePen sPen = MDraw.getStylePen().setClipURL("#box");
    ClipTag theView = MDraw.getClipTag("box");
    theView.drawRectangle((double) mLeft, vMax, hMax, (double) mBottom);
    pPaper.add(theView);
    UseTag theRef = MDraw.getUseTag("#graph");
    theRef.setStyle(sPen);
    pPaper.add(theRef);
  }

  /** Set the scale type for the vertical and horizontal axes. Set true to make the specified <b>scale</b> linear, false for logarithmic. */
  public void setScaleTypes(boolean pHorizLinear, boolean pVertLinear) {
    cbxHorizLinear.setState(pHorizLinear);
    cbxVertLinear.setState(pVertLinear);
  }

  /** Set the number of cycles for each axis. If linear, this is ignored. */
  public void setCycles(int pHorizontal, int pVertical) {
    txfNumHCycles.setText("" + pHorizontal);
    txfNumVCycles.setText("" + pVertical);
  }

  /** Set the length of each cycle. Ignored if a scale is linear. */
  public void setCycleLengths(int pHorizontal, int pVertical) {
    txfHorizCycLen.setText("" + pHorizontal);
    txfVertCycLen.setText("" + pVertical);
  }

  /** Set the initial interval for a lograrithmic scale and the fixed interval for a linear scale. */
  public void setIntervals(int pHorizontal, int pVertical) {
    txfHInterval.setText("" + pHorizontal);
    txfVInterval.setText("" + pVertical);
  }

  /** Set the scale factors for each scale. The values will be clamped if it is outside [0.0, 1.0]. Ignored if the scale is linear. */
  public void setScales(double pHorizontal, double pVertical) {
    txfHScale.setText("" + pHorizontal);
    txfVScale.setText("" + pVertical);
  }

  /** Set the pens used. The <i>vertical</i> pen is used for vertical lines, which are on the <i>horizontal</i> scale, and vise versa.
      @param pHorizontalMajor The pen used for major lines on the vertical scale (horizontal lines!).
      @param pHorizontalMinor The pen used for minor lines on the vertical scale.
      @param pVerticalMajor The cycle initial line pen (major) for the horizontal scale (vertical lines).
      @param pVerticalMinor The minor vertical line pen (horizontal scale).
  */
  public void setPens(int pHorizontalMajor, int pHorizontalMinor, int pVerticalMajor, int pVerticalMinor) {
    txfPenHMaj.setText("" + pHorizontalMajor);
    txfPenHMin.setText("" + pHorizontalMinor);
    txfPenVMaj.setText("" + pVerticalMajor);
    txfPenVMin.setText("" + pVerticalMinor);
  }

}

