package cox.jmatt.java.MathTools.demo.gpgdemo;

import java.awt.*;
import java.util.Properties;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.demo.*;
import cox.jmatt.java.MathTools.markup.svg.*;

/** <p>This generator produces square- or rectangular-grid paper with alternate rows offset, yielding a brick-like pattern. The horizontal and vertical line pens 
    can be selected as can the length of the horizontal and vertical sides. Ordinarily the first row is flush with the left margin and the second row is offset. 
    Checking the '[]Offset first row' Checkbox offsets the first row and puts the second flush.</p>
*/

public class OffsetQuadPanel extends GraphPanel {

  TextField txfHeight = new TextField(), txfWidth = new TextField(), txfHorizPen = new TextField(), txfVertPen = new TextField();
  Checkbox cbxShowHoriz = new Checkbox("Show horizontals", true), cbxShowVert = new Checkbox("Show verticals", true), cbxOffset = new Checkbox("Offset first row");

  /** Usual constructor. */
  public OffsetQuadPanel() {
    super();
    _setUpGUI();
    GPickle.gpgOffsetQuad = this;
  }

  /** Draw the paper. */
  public void drawGraph(int pTop, int pBot, int pLeft, int pRight, SVGTag pPaper) {
    double yMin = pTop, xMin = pLeft, yMax = pBot, xMax = pRight;
    double dHeight = asDouble(txfHeight.getText(), 25d, 0.1, null), dWidth = asDouble(txfWidth.getText(), 25d, 0.1, null);
    int vLine = getPen(txfVertPen), hLine = getPen(txfHorizPen);
    boolean showVert = cbxShowVert.getState(), showHoriz = cbxShowHoriz.getState(), offset = cbxOffset.getState();
    int xCount = ((int) ((xMax - xMin)/dWidth)) - 1, yCount = ((int) ((yMax - yMin)/dHeight)) - 1;
    double curPos = 0d;
    MathDrawSVG MDraw = cox.jmatt.java.MathTools.demo.GPGenerator.getMDraw();
    String rowLeft = "LeftRow_"+CapCom.hitCount(), rowRight = "RightRow_"+CapCom.hitCount(), rowLast = "LastRow_"+CapCom.hitCount();
    UseTag uTag = null;
    TransformPen tPen = null;
    GroupTag gt = MDraw.getGroupTag(rowLeft); // Left row.
    rowLeft = "#"+rowLeft;
    if(showHoriz) {
      MDraw.setLinePen(hLine);
      gt.drawLine(0d, 0d, dWidth * xCount, 0d);
      gt.drawLine(0d, dHeight, dWidth/2, dHeight);
    }
    if(showVert) {
      MDraw.setLinePen(vLine);
      curPos = 0d;
      for(int i=0; i <= xCount; i++, curPos += dWidth) gt.drawLine(curPos, 0d, curPos, dHeight);
    }
    pPaper.add(gt);
    gt = MDraw.getGroupTag(rowRight); // Right row.
    rowRight = "#"+rowRight;
    if(showHoriz) {
      curPos = dWidth * xCount;
      MDraw.setLinePen(hLine);
      gt.drawLine(0d, 0d, curPos, 0d);
      gt.drawLine(curPos - dWidth/2, dHeight, curPos, dHeight);
    }
    if(showVert) {
      MDraw.setLinePen(vLine);
      curPos = 0d;
      for(int i=0; i <= xCount; i++, curPos += dWidth) gt.drawLine(curPos, 0d, curPos, dHeight);
    }
    pPaper.add(gt);
    gt = MDraw.getGroupTag(rowLast); // Bottom row, works both ways.
    rowLast = "#" + rowLast;
    if(showHoriz) {
      curPos = dWidth * xCount;
      MDraw.setLinePen(hLine);
      gt.drawLine(0d, 0d, curPos, 0d);
      gt.drawLine(0d, dHeight, curPos, dHeight);
    }
    if(showVert) {
      MDraw.setLinePen(vLine);
      curPos = 0d;
      for(int i=0; i <= xCount; i++, curPos += dWidth) gt.drawLine(curPos, 0d, curPos, dHeight);
    }
    pPaper.add(gt);
    curPos = yMin;
    for(int i=0; i < yCount; i++, curPos += dHeight) {
      tPen = MDraw.getTransformPen();
      if(offset) {
        uTag = MDraw.getUseTag(rowRight);
        tPen.translate(xMin + dWidth/2, curPos);
      } else {
        uTag = MDraw.getUseTag(rowLeft);
        tPen.translate(xMin, curPos);
      }
      uTag.setTransform(tPen);
      pPaper.add(uTag);
      offset = !offset;
    }
    //offset = !offset;
    tPen = MDraw.getTransformPen();
    uTag = MDraw.getUseTag(rowLast);
    tPen.translate(xMin + (offset?dWidth/2:0), curPos);
    uTag.setTransform(tPen);
    pPaper.add(uTag);
  }

  private void _setUpGUI() {
    setLayout(new GridLayout(0,2));
    add(new Label("Height")); add(txfHeight);
    add(new Label("Width:")); add(txfWidth);
    add(new Label("")); add(cbxOffset);
    add(new Label("")); add(cbxShowVert);
    add(new Label("")); add(cbxShowHoriz);
    add(new Label("Horizontal line pen:")); add(txfHorizPen);
    add(new Label("Vertical line pen:")); add(txfVertPen);
    reset();
  }

  /** Reset starting values. */
  public void reset() {
    setSize(25, 25);
    setVisible(true, true);
    setPens(0, 0);
  }

  /** Set the height and width of the grid. Negative or zero values are ignored, allowing one quantity to be set without affecting the other.
      @param pHeight The length of the vertical side.
      @param pWidth The length of the horizontal side.
  */
  public void setSize(int pHeight, int pWidth) {
    if(pHeight > 0) txfHeight.setText(""+pHeight);
    if(pWidth > 0) txfWidth.setText(""+pWidth);
  }

  /** Set the pens to use when drawing the lines. Negative or zero values are ignored. */
  public void setPens(int pVertical, int pHorizontal) {
    if(pHorizontal >= 0) txfHorizPen.setText(""+pHorizontal);
    if(pVertical >= 0) txfVertPen.setText(""+pVertical);
  }

  /** Set the visibility of the veritcal and horizontal sides. */
  public void setVisible(boolean pVertical, boolean pHorizontal) {
    cbxShowVert.setState(pVertical); cbxShowHoriz.setState(pHorizontal);
  }

  /** Enable/disable first row offset. If 'true' the first row will be offset and the second flush. */
  public void setOffset(boolean pOffsetFirstRow) { cbxOffset.setState(pOffsetFirstRow); }

  public String toPizza(String pPrefix) {
    StringBuffer retval = new StringBuffer("# OffsetQuadPanel configuration data.");
    if(Question.validString(pPrefix)) {
      if(!pPrefix.endsWith(".")) pPrefix += ".";
      retval.append(" Prefix: ").append(pPrefix);
    } else {
      pPrefix = "";
    }
    retval.append("\n");
    retval.append(getKey(txfHeight, pPrefix, "brick.height"));
    retval.append(getKey(txfWidth, pPrefix, "brick.width"));
    retval.append(getKey(txfHorizPen, pPrefix, "brick.pen.horizontal"));
    retval.append(getKey(txfVertPen, pPrefix, "brick.pen.vertical"));
    retval.append(getKey(cbxShowHoriz, pPrefix, "brick.showHorizontal"));
    retval.append(getKey(cbxShowVert, pPrefix, "brick.showVertical"));
    retval.append(getKey(cbxOffset, pPrefix, "brick.offsetFirst"));
    return retval.toString();
  }

  public void eatPizza(Properties pPizza, String pPrefix) {
    if((pPizza == null) || pPizza.isEmpty()) return;
    if(Question.validString(pPrefix)) {
      if(!pPrefix.endsWith(".")) pPrefix += ".";
    } else {
      pPrefix = "";
    }
    setKey(pPizza, txfHeight, pPrefix, "brick.height", "25");
    setKey(pPizza, txfWidth, pPrefix, "brick.width", "25");
    setKey(pPizza, txfHorizPen, pPrefix, "brick.pen.horizontal", "0");
    setKey(pPizza, txfVertPen, pPrefix, "brick.pen.vertical", "0");
    setKey(pPizza, cbxShowHoriz, pPrefix, "brick.showHorizontal");
    setKey(pPizza, cbxShowVert, pPrefix, "brick.showVert");
    setKey(pPizza, cbxOffset, pPrefix, "brick.offsetFirst");
  }

}

