package cox.jmatt.java.MathTools.demo.gpgdemo;

import java.awt.*;
import java.util.Properties;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.demo.*;
import cox.jmatt.java.MathTools.markup.svg.*;

/** <p>This class generates graph paper for polar coordinates. The coordinate system is centered on the paper and extends to the maximum width (or least dimension, 
    if the margins are altered) in circular radius. The distance between the circles (radial increment) is 'Circle spacing' and is in user units (1/100 in.).</p>
    <p>NOTE: The angular divisions ('spokes') are drawn <i>diametrically</i> and not radially, so there will be twice as many as listed in the 'Radial divisions' 
    field. This is not a bug or feature, simply the way the program works.</p>
*/
public class PolarPanel extends GraphPanel {

  private TextField txfCircDivs = null, txfAngularDivs = null, txfCirclePen = null, txfLinePen = null;

  /** Nothing to see here. Just a constructor. */
  public PolarPanel() {
    super();
    setUpGUI();
    GPickle.gpgPolarPanel = this;
  }

  private void setUpGUI() {
    txfCircDivs = new TextField("25");
    txfAngularDivs = new TextField("10");
    txfCirclePen = new TextField("0");
    txfLinePen = new TextField("0");
    setLayout(new GridLayout(0,2));
    add(new Label("Circle spacing")); add(txfCircDivs);
    add(new Label("Radial divisions*")); add(txfAngularDivs);
    add(new Label("  (*Divisions are drawn diametrically)")); add(new Label(" "));
    add(new Label(" ")); add(new Label(" "));
    add(new Label("Circle pen")); add(txfCirclePen);
    add(new Label("Line pen")); add(txfLinePen);
  }

  public String toPizza(String pPrefix) {
    StringBuffer retval = new StringBuffer("# PolarPanel configuration data.");
    if(Question.validString(pPrefix)) {
      if(!pPrefix.endsWith(".")) pPrefix += ".";
      retval.append(" Prefix: ").append(pPrefix);
    } else {
      pPrefix = "";
    }
    retval.append("\n");
    retval.append(getKey(txfCircDivs, pPrefix, "polar.circles"));
    retval.append(getKey(txfAngularDivs, pPrefix, "polar.radials"));
    retval.append(getKey(txfCirclePen, pPrefix, "polar.pen.circle"));
    retval.append(getKey(txfLinePen, pPrefix, "polar.pen.lines"));
    return retval.toString();
  }

  public void eatPizza(Properties pPizza, String pPrefix) {
    if((pPizza == null) || pPizza.isEmpty()) return;
    if(Question.validString(pPrefix)) {
      if(!pPrefix.endsWith(".")) pPrefix += ".";
    } else {
      pPrefix = "";
    }
    setKey(pPizza, txfCircDivs, pPrefix, "polar.circles", "0");
    setKey(pPizza, txfAngularDivs, pPrefix, "polar.radials", "0");
    setKey(pPizza, txfCirclePen, pPrefix, "polar.pen.circle", "0");
    setKey(pPizza, txfLinePen, pPrefix, "polar.pen.lines", "0");
  }

  /** Reset all to default values and print a debug message about it. */
  public void reset() {
    super.reset();
    txfCircDivs.setText("25"); txfAngularDivs.setText("10"); txfCirclePen.setText("0"); txfLinePen.setText("0");
  }

  /** Draw the graph. */
  public void drawGraph(int mTop, int mBottom, int mLeft, int mRight, SVGTag pPaper) {
    double midX = (mLeft + mRight)/2, midY = (mTop + mBottom)/2;
    double maxRad = Math.min(midX, midY);
    double deltaC = asDouble(txfCircDivs.getText(), 25.0, 5.0, null);
    int spokes = asInt(txfAngularDivs.getText(), 10, 2, 180);
    double radius = deltaC, rotAngle = 180 / spokes;
    MathDrawSVG MDraw = new MathDrawSVG();
    MDraw.setCirclePen(getPen(txfCirclePen));
    for( ; radius < maxRad; radius += deltaC) pPaper.drawCircle(midX, midY, radius);
    MDraw.setLinePen(getPen(txfLinePen));
    LineTag diameter = MDraw.getLineTag( midX - radius +deltaC, midY, midX + radius - deltaC, midY);
    TransformPen tPen = MDraw.getTransformPen();
    pPaper.add(diameter);
    for(double curAngle = rotAngle; curAngle < 180.0; curAngle += rotAngle) {
      tPen.reset().rotate(curAngle, midX, midY);
      diameter.setTransform(tPen);
      pPaper.add(diameter);
    }
  }

  /** Set the radial spacing and the number of angular divisions. NOTE: 'angular divisions' here are subject to the same conditions cited in the class docs.
      @param pCircleSpace sets the 'Circle spacing' field.
      @param pRadialDivs sets the 'Radial divisions' field.
  */
  public void setScale(int pCircleSpace, int pRadialDivs) {
    txfCircDivs.setText("" + pCircleSpace);
    txfAngularDivs.setText("" + pRadialDivs);
  }

  /** Set the pen number used to draw the circles and lines.
      @param pCirclePen The pen used for the circles on the graph system.
      @param pLinePen The pen used for the lines on the graph system.
  */
  public void setPens(int pCirclePen, int pLinePen) {
    txfCirclePen.setText("" + pCirclePen);
    txfLinePen.setText("" + pLinePen);
  }

}

