package cox.jmatt.java.MathTools.demo.gpgdemo;

import java.awt.*;
import java.util.Properties;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.demo.*;
import cox.jmatt.java.MathTools.markup.svg.*;

/** <p>This panel generates standard, square-grid (quadrille) graph paper. The vertical and horizontal scales can be set independently and either can be disabled, 
    for paper with only vertical or only horizontal lines. It is important to note that the horizontal scale is the distance between <b>vertical</b> lines and the 
    vertical scale is the difference between <b>horizontal</b> lines.</p>
*/
public class QuadrillePanel extends GraphPanel {

  private TextField txfVScale = null, txfHScale = null, txfHLPen = null, txfVLPen = null;
  private Checkbox cbxShowVertical = null, cbxShowHorizontal = null;

  /** Standard constructor. */
  public QuadrillePanel() {
    super();
    buildGUI();
    GPickle.gpgQuadrillePanel = this;
  }

  private void buildGUI() {
    txfVScale = new TextField("25");
    txfHScale = new TextField("25");
    txfVLPen = new TextField("0");
    txfHLPen = new TextField("0");
    cbxShowVertical = new Checkbox("Show vertical lines", true);
    cbxShowHorizontal = new Checkbox("Show horizontal lines", true);
    setLayout(new GridLayout(0,2));
    add(new Label("Horizontal scale")); add(txfHScale);
    add(new Label("Vertical scale")); add(txfVScale);
    add(new Label(" ")); add(cbxShowVertical);
    add(new Label(" ")); add(cbxShowHorizontal);
    add(new Label("Vertical line pen")); add(txfVLPen);
    add(new Label("Horizontal line pen")); add(txfHLPen);
  }

  /** Reset to standard quadrille values with both dimensions visible. */
  public void reset() {
    super.reset();
    txfVScale.setText("25"); txfHScale.setText("25"); txfVLPen.setText("0"); txfHLPen.setText("0");
    cbxShowVertical.setState(true); cbxShowHorizontal.setState(true);
  }

  /** Draw the graph. */
  public void drawGraph(int mTop, int mBottom, int mLeft, int mRight, SVGTag pPaper) {
    if(pPaper == null) return;
    MathDrawSVG MDraw = cox.jmatt.java.MathTools.demo.GPGenerator.getMDraw();
    int deltaX = asInt(txfHScale.getText(), 25, 5, null);
    int deltaY = asInt(txfVScale.getText(), 25, 5, null);
    mRight -= ((mRight - mLeft) % deltaX);
    mBottom -= ((mBottom - mTop) % deltaY);
    if(cbxShowVertical.getState()) {
      MDraw.setLinePen(getPen(txfVLPen));
      for(int i = mLeft; i <= mRight; i += deltaX) {
        pPaper.drawLine((double) i, (double) mTop, (double) i, (double) mBottom);
      }
    }
    if(cbxShowHorizontal.getState()) {
      MDraw.setLinePen(getPen(txfHLPen));
      for(int i = mTop; i <= mBottom; i += deltaY) {
        pPaper.drawLine((double) mLeft, (double) i, (double) mRight, (double) i);
      }
    }
  }

  /** Set the horizontal and vertical scale. */
  public void setScale(int pHorizontal, int pVertical) {
    txfHScale.setText(""+pHorizontal);
    txfVScale.setText(""+pVertical);
  }

  /** Set the pens to use. */
  public void setPens(int pVertical, int pHorizontal) {
    txfVLPen.setText(""+pVertical);
    txfHLPen.setText(""+pHorizontal);
  }

  /** Enable/disable vertical and horizontal lines. */
  public void setShowLines(boolean pHorizontal, boolean pVertical) {
    cbxShowVertical.setState(pVertical);
    cbxShowHorizontal.setState(pHorizontal);
  }

  
  public String toPizza(String pPrefix) {
    StringBuffer retval = new StringBuffer("# QuadrillePanel configuration data.");
    if(Question.validString(pPrefix)) {
      if(!pPrefix.endsWith(".")) pPrefix += ".";
      retval.append(" Prefix: ").append(pPrefix);
    } else {
      pPrefix = "";
    }
    retval.append("\n");
    retval.append(getKey(txfHScale, pPrefix, "quad.scale.horizontal"));
    retval.append(getKey(txfVScale, pPrefix, "quad.scale.vertical"));
    retval.append(getKey(txfHLPen, pPrefix, "quad.pen.horizontal"));
    retval.append(getKey(txfVLPen, pPrefix, "quad.pen.vertical"));
    retval.append(getKey(cbxShowHorizontal, pPrefix, "quad.horizontal.enabled"));
    retval.append(getKey(cbxShowVertical, pPrefix, "quad.vertical.enabled"));
    return retval.toString();
  }

  public void eatPizza(Properties pPizza, String pPrefix) {
    if((pPizza == null) || pPizza.isEmpty()) return;
    if(Question.validString(pPrefix)) {
      if(!pPrefix.endsWith(".")) pPrefix += ".";
    } else {
      pPrefix = "";
    }
    setKey(pPizza, txfHScale, pPrefix, "quad.scale.horizontal", "25");
    setKey(pPizza, txfVScale, pPrefix, "quad.scale.vertical", "25");
    setKey(pPizza, txfHLPen, pPrefix, "quad.pen.horizontal", "0");
    setKey(pPizza, txfVLPen, pPrefix, "quad.pen.vertical", "0");
    setKey(pPizza, cbxShowHorizontal, pPrefix, "quad.horizontal.enabled");
    setKey(pPizza, cbxShowVertical, pPrefix, "quad.vertical.enabled");
  }

}

