package cox.jmatt.java.MathTools.graphing;

/** <p>This interface represents a graph's axis. The most common types are the X-axis and Y-axis on a Cartesian plane, but the <code>Axis</code> interface makes 
    no such limiting assumption. For the purposes of this package an Axis represents a graphing component against which actual graphs are drawn, measured, or 
    compared. This interface is designed to represent these concepts in the broadest sense and to leave the actual interpretation and usage to the implementation 
    defining it. So, if a graph required a grid to be drawn Axis can be used for that.</p>
    <p><code>Axis</code> extends <code>Renderable</code>, so anything required but not provided can be configured via the <code>setParameter()</code> or 
    <code>setType()</code> methods.</p>
*/
public interface Axis extends Renderable {

  /** The scale of an axis determines how measurements and comparisons are made. How, or if, that is used depends on the implementation but this method provides 
      the framework for it.
      @param pScale The Object representing the scale used for this Axis.
      @return true if the pScale makes sense and is set, false otherwise.
  */
  public boolean setScale(Object pScale);

  /** Tick marks are small marks placed on an axis at appropriate intervals. Use this method to configure them for this Axis.
      @param pMarks The Object representing the tick-mark configuration to be used for this Axis.
      @return true if pMarks makes sense, false otherwise.
  */
  public boolean setTickMarks(Object pMarks);

}
