package cox.jmatt.java.MathTools.graphing.Graph;

import java.util.*;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.graphing.*;

/** This is the main do-it class for the Graph implementation. It includes everything needed for a <b>.grf</b> file except the graph, and that is handled by the 
    <code>GraphablePDK</code> class. All other <code>Renderable</code> classes are no-op versions.
*/
public class GraphPaperPDK extends cox.jmatt.java.MathTools.graphing.noop.NOGraphPaper implements GraphPaper {

  private static final String HEADER = ";Changing this file may cause Graph to crash.\n[Graph]\nVersion = 4.2.0.332\nMinVersion = 2.5\nOS = Windows NT 6.0 Service Pack 1\n";
  private static final String AXIS_TEMPLATE = "[Axes]\nxMin = <0>\nxMax = <1>\nxTickUnit = 2\nxGridUnit = 2\nyMin = <2>\nyMax = <3>\nyTickUnit = 2\nyGridUnit = 2\n";
  private static final String AXIS_FOOTER = "AxesColor = clBlue\nGridColor = 0x00FF9999\nShowLegend = 0\nRadian = 1\n";
  private static final String DATA_TEMPLATE = "[Data]\nTextLabelCount = 0\nFuncCount = <4>\nPointSeriesCount = 0\nShadeCount = 0\nRelationCount = 0\nOleObjectCount = 0\n";

  private String[] myTokens = {"-10", "10", "-10", "10", "0"};
  private ArrayList<Graphable> myGraphs = new ArrayList<Graphable>();

  /** Standard constructor. Builds an instance with everything needed to graph but the graph. */
  public GraphPaperPDK() { super(); }

  /** Add a graph to the paper. The 'pKey' parameter is not used and is ignored. If pGraph is null this method returns false immediately, otherwise it returns true. */
  public boolean addGraph(String pKey, Graphable pGraph) {
    if(pGraph == null) return false;
    myGraphs.add(pGraph);
    return true;
  }

  /** This method sets the range values for the resulting graph. If the minimum values are not less than the maxima the method returns false immediately. */
  public boolean setRanges(double xMin, double yMin, double xMax, double yMax) {
    if((xMax <= xMin) || (yMax <= yMin)) return false;
    myTokens[0] = ""+xMin; myTokens[1] = ""+xMax; myTokens[2] = ""+yMin; myTokens[3] = ""+yMax;
    return true;
  }

  /** Render the graph. The result of a call to this method will be a properly-formatted String ready to be saved in a <b>.grf</b> file. The rendering context 
      Object is a <code>GraphKeysPDK</code> instance but if one is not furnished it will be handled automatically.
  */
  public Object render(Object pContext) {
    if((pContext == null) || !(pContext instanceof GraphKeysPDK)) pContext = new GraphKeysPDK();
    GraphKeysPDK myContext = (GraphKeysPDK) pContext;
    myTokens[4] = ""+myGraphs.size();
    StringBuffer retval = new StringBuffer(HEADER);
    retval.append(Question.fillTemplate(AXIS_TEMPLATE, myTokens));
    retval.append(AXIS_FOOTER);
    int iNdx = 1;
    myContext.setRange(myTokens[0], myTokens[1]);
    for(Iterator<Graphable> it = myGraphs.iterator(); it.hasNext(); ) {
      myContext.setCount(iNdx);
      retval.append(it.next().render(myContext).toString());
      iNdx++;
    }    
    retval.append(Question.fillTemplate(DATA_TEMPLATE, myTokens));
    return retval.toString();
  }

}
