package cox.jmatt.java.MathTools.graphing.Graph;

import java.util.*;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.graphing.*;

/** <p>This class supplies the graph part of the package. To create one, set an expression under the appropriate key. Parametric graphs require two expressions and 
    have two separate keys for the job. If the graph should not extend the entire ranges set in GraphPaper, the extents can be set using the <code>addLimit()</code> 
    method. It extends <code>NORenderable</code> since it requires no external functionality.</p>
*/
public class GraphablePDK extends cox.jmatt.java.MathTools.graphing.noop.NORenderable implements Graphable {

  private static final String TEMPLATE = "[Func<0>]\nFuncType = <1>\n<2><3>\nFrom = <4>\nTo = <5>\nSteps = <6>\nColor = clBlack\nSize = 3\n";
  private String[] myTokens = {"0", "0", "", "y = x", "", "", "1000"};

  /** The only constructor. */
  public GraphablePDK() { super(); }

  /** This method is used for the 'From', 'To', and 'Steps' settings for the graph. The pKey argument must be one of these words and the pValue argument a 
      <code>Number</code> instance. If pValue is not a Number the method returns false instantly. If pKey is not recognized nothing is set and the method returns 
      true.
  */
  public boolean addLimit(String pKey, Object pValue) {
    boolean retval = false;
    if(!(pValue instanceof Number) || !Question.validString(pKey)) return retval;
    if("from".equalsIgnoreCase(pKey)) {
      myTokens[4] = ""+((Number) pValue).doubleValue();
      retval = true;
    } else if("to".equalsIgnoreCase(pKey)) {
      myTokens[5] = ""+((Number) pValue).doubleValue();
      retval = true;
    } else if("steps".equalsIgnoreCase(pKey)) {
      myTokens[6] = ""+Math.abs(((Number) pValue).intValue());
    } else {}
    return retval;
  }

  /** Add an expression to the graph. The key argument MUST be one of <code>Graphable.STANDARD_GRAPH, POLAR_GRAPH, PARAMETRIC_X, or PARAMETRIC_Y</code>. The 
      'pExpr' argument MUST be a String. For STANDARD graphs it is an expression in the variable 'x'. For POLAR and PARAMETRIC graphs it must be an expression 
      in the variable 't'. <b>IMPORTANT NOTE:</b> PARAMETRIC graphs require both an X and Y expression in 't'. If the key or value is blank or null the method 
      returns false. If the key is not recognized the method returns false. If the method returns true, something was set.
  */
  public boolean addExpression(String pKey, Object pExpr) {
    boolean retval = false;
    if((pExpr == null) || !Question.validString(pExpr.toString())) return retval;
    if(Graphable.STANDARD_GRAPH.equals(pKey)) {
      myTokens[1] = "0";
      myTokens[3] = "y = "+pExpr.toString();
      retval = true;
    } else if(Graphable.POLAR_GRAPH.equals(pKey)) {
      myTokens[1] = "2";
      myTokens[3] = "r = "+pExpr.toString();
      retval = true;
    } else if(Graphable.PARAMETRIC_X.equals(pKey)) {
      myTokens[1] = "1";
      myTokens[2] = "x = "+pExpr.toString()+"\n";
      retval = true;
    } else if(Graphable.PARAMETRIC_Y.equals(pKey)) {
      myTokens[1] = "1";
      myTokens[3] = "y = "+pExpr.toString();
      retval = true;
    } else {}
    return retval;
  }

  /** This method requires a <code>GraphKeysPDK</code> as a context object. If it does not receive one, the return value will be blank. */
  public Object render(Object pContext) {
    if((pContext == null) || !(pContext instanceof GraphKeysPDK)) return "";
    GraphKeysPDK myContext = (GraphKeysPDK) pContext;
    myTokens[0] = "" + myContext.getCount();
    if(!Question.validString(myTokens[4])) myTokens[4] = myContext.getFrom();
    if(!Question.validString(myTokens[5])) myTokens[5] = myContext.getTo();
    return Question.fillTemplate(TEMPLATE, myTokens);
  }

}
