package cox.jmatt.java.MathTools.graphing;

/** <p>The <code>GraphEngine</code> provides all the components needed to create graphs. It and the graphing components it provides are specific to the implementation 
    they provide. When requested, <code>GraphEngineFactory</code> tries to provide an implementation of this interface.</p>
    <p>Runtime configuration is provided by means of Features. Exactly what these are, aside from generic Objects with String names, depends on the implementation. 
    By intent, setting a Feature affects in some way the graphing components that are generated. Please note: this interface does NOT extend <code>Renderable</code>. 
    It is not meant to appear on a graph; rather, it generates the items that <b>do</b>.</p>
    <p>the <code>pKey</code> argument in all of <code>GraphEngine</code>'s get__() methods can be used as a type for multiple classes implementing the interface 
    or it can be used as an ID. If used as a type, it may or may not interact with the <code>Renderable setType()</code> method. It is provided for implementations 
    that define, say, more than one type of <code>Graphable</code>. The <code>GraphEngine</code> for such an implementation could then furnish whichever was 
    desired based on the key sent in. NOTE: The <code>GraphKeys</code> class, if it exists, is assumed to be self-sufficient so no key value is supplied for its 
    getter.</p>
*/
public interface GraphEngine {

  /** Retrieve a GraphPaper instance for the implementation. */
  public GraphPaper getGraphPaper(Object pKey);

  /** Create and return an implementation-specific Axis. */
  public Axis getAxis(Object pKey);

  /** Create and return a Graphable suited to the implementation. */
  public Graphable getGraph(Object pKey);

  /** Return an instance of the implementation's GraphKeys, or null if none such exists. */
  public GraphKeys getGraphKeys();

  /** Return a blank Polygon. Implementations may give new Polygons default values but this is not required. */
  public Polygon getPolygon(Object pKey);

  /** Set a feature to be used when generating graph components. All such features are implementation-dependent and optional. By convention, setting a feature 
      to <code>null</code> should clear that feature or reset it to its default state.
      @param pFeature The String name of the feature to be set.
      @param pValue The value the feature is to have.
      @return true if the feature is set, false if it does not apply or is not set for whatever reason.
  */
  public boolean setFeature(String pFeature, Object pValue);

  /** Test whether or not a feature is set.
      @param pFeature The name of the feature to test.
      @return true if the feature is set, false otherwise.
  */
  public boolean testFeature(String pFeature);

}

