package cox.jmatt.java.MathTools.graphing;

/** <p>This interface provides one special component that may appear on a graph: a polygon. The definition here is used loosely; there is no absolute or implied 
    requirement that the <code>Polygon</code> not intersect itself, nor must it be closed. The Polygon class instead represents a two-dimensional collection of 
    line segments such that each shares at least one endpoint with another line segment and such that it may be drawn (rendered) as a consecutive and finite sequence 
    of points.</p>
    <p>The critical components of a <code>Polygon</code> are the points describing its vertices and whether or not it is closed. In a closed Polygon the last point 
    added is automatically considered connected to the very first point; this does not need to be done explicitly. Also, since this concept is very closely tied 
    to the two-dimensional nature of graphs in this package, point values are given as double and not Object.</p>
*/
public interface Polygon extends Renderable {

  /** Add a line segment to the Polygon. Due to the nature of polygons, the points added MUST be kept in order! This method should return a reference to itself or 
      a copy (for immutable Polygons) to allow convenient method chaining.
      @param pX The X-coordinate of the point.
      @param pY The Y-coordinate of the point.
      @return A self-reference to allow multiple vertex adds.
  */
  public Polygon addVertex(double pX, double pY);

  /** Determine whether or not the last point added will be 'connected' to the first point entered. Setting pClosed to 'true' has the same effect as adding the 
      first point again after the last one is added; this method simply makes things easier.
      @param pClosed true to 'close' the Polygon explicitly, false to leave it 'open'.
  */
  public void setClosed(boolean pClosed);

}
