package cox.jmatt.java.MathTools.graphing;

import java.util.*;

/** <p>This is a support class for any graphing implementation. Most methods assume, but do not enforce, that parameter and other Objects will in fact be Strings. 
    The X<L implementation makes and states this assumption. The <code>RenderEngine</code> class is provided for graphing implementations in which the 
    data cannot possibly be String or have a <code>toString()</code> method capable of re-creating it.</p>
    <p>This class is very simple in both design and function. It accepts Renderables, holds them in the order they were added, and returns them as an Object[] 
    array upon request.</p>
*/
public class RenderEngine {

  private ArrayList<Renderable> myStuff = new ArrayList<Renderable>();

  /** Constructor necessary for adding to scripts. */
  public RenderEngine() { super(); }

  /** Add a <code>Renderable</code> to the collection. The method returns false only if the supplied Renderable is null. */
  public boolean addRenderable(Renderable pItem) {
    if(pItem == null) return false;
    myStuff.add(pItem);
    return true;
  }

  /** Call <code>render()</code> on all the items currently held, in order, and collect the results.
      @param pContext The Rendering Context to be used for all the items rendered.
      @return The result of rendering all held <code>Renderable</code>s against the supplied context.
  */
  public Object[] render(Object pContext) {
    Object[] retval = {};
    if(myStuff.size() == 0) return retval;
    retval = new Object[myStuff.size()];
    int iCount = 0;
    for(Iterator it = myStuff.iterator(); it.hasNext(); ) {
      retval[iCount] = ((Renderable) it.next()).render(pContext);
      iCount++;
    }
    return retval;
  }

}

