package cox.jmatt.java.MathTools.graphing.noop;

import cox.jmatt.java.MathTools.graphing.*;

/** <p>This class serves two purposes. First of all it acts as a 'null' <code>GraphEngine</code>. It is fully functional as such but all instances it returns are 
    classes from the <code>graphing.noop</code> package.</p>
    <p>Its second and perhaps most useful purpose is to act as a base class for implementations to extend. Not all implementations will require or furnish all 
    <code>Renderable</code> classes, yet <code>GraphHandler</code> <b>demands</b> non-null instances or it might choke. Implementations for which this is the case 
    (e.g. <code>graphing.Graph</code>) can subclass this for their base <code>GraphEngine</code> and only override what they need.</p>
*/
public class NOGraphEngine implements GraphEngine {

  /** Simple constructor. */
  public NOGraphEngine() { super(); }
  /** Return a NOGraphPaper instance. */
  public GraphPaper getGraphPaper(Object pKey) { return new NOGraphPaper(); }

  /** Create and return a NOAxis. */
  public Axis getAxis(Object pKey) { return new NOAxis(); }

  /** Create and return a NOGraphable. */
  public Graphable getGraph(Object pKey) { return new NOGraphable(); }

  /** Return null. Always. */
  public GraphKeys getGraphKeys() { return null; }

  /** Return a NOPolygon. */
  public Polygon getPolygon(Object pKey) { return new NOPolygon(); }

  /** Always returns false. */
  public boolean setFeature(String pFeature, Object pValue) { return false; }

  /** Always returns false. */
  public boolean testFeature(String pFeature) { return false; }

}

