package cox.jmatt.java.MathTools.graphing.xml;

import java.util.*;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.graphing.*;

/** <p>This class is the XML package's Axis implementation. When rendered, it generates an &lt;Axis&gt; tag with an optional ID attribute, name attribute, and 
    type attribute, if these fields are defined. If no parameters are set the tag itself is empty: The scale and tick marks are represented as String attributes. 
    If the Axis is created with its name specified, that becomes its name. The <code>setType()</code> method determines its type and the ID is supplied as the 
    key when it is added to a GraphPaper.</p>
    <p>All Object arguments are assumed to be String. This is enforced.</p>
*/
public class AxisXML extends RenderableXML implements Axis {

  private Object myScale = null, myTicks = null;

  /** Standard constructor for an unnamed Axis. */
  public AxisXML() { super(); }

  /** Constructor used to create a named Axis. */
  public AxisXML(String pName) { super(pName); }

  /** Set or clear the scale for this Axis. Setting null clears any previously-set value. This method returns true if its scale is not null, false if it is. If 
      the scale is not defined it will not appear as an attribute when rendered.
  */
  public boolean setScale(Object pScale) {
    myScale = pScale;
    return (myScale != null);
  }

  /** Set or clear the tick marks for this Axis. Setting null clears any previously-set value. This method returns true if pMarks is not null, false if it is. If 
      this is not defined it will not appear as an attribute when rendered.
  */
  public boolean setTickMarks(Object pMarks) {
    myTicks = pMarks;
    return (myTicks != null);
  }

  /** Print this Axis as an XML tag. If name, type, scale, and tick marks are defined they appear as attributes with the same name (tick marks is tickMarks). 
      If no parameters are set the tag is empty, otherwise it includes them as children. No context object is required or used. If ID is not defined it will be 
      assigned automatically.
  */
  public Object render(Object pContext) {
    StringBuffer retval = new StringBuffer("    <Axis ");
    if(!Question.validString(myID)) myID = "Axis"+GraphKeysXML.hitCount();
    retval.append("id=\"").append(myID).append("\" ");
    if(Question.validString(myName)) retval.append("name=\"").append(myName).append("\" ");
    if(Question.validString(myType)) retval.append("type=\"").append(myType).append("\" ");
    if(myScale != null) retval.append("scale=\"").append(myScale.toString()).append("\" ");
    if(myTicks != null) retval.append("tickMarks=\"").append(myTicks.toString()).append("\" ");
    if(myParams.isEmpty()) {
      retval.append("/>\n");
    } else {
      retval.append(">\n");
      retval.append(super.render(pContext));
      retval.append("    </Axis>\n");
    }
    return retval.toString();
  }

}
