package cox.jmatt.java.MathTools.graphing.xml;

import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.graphing.*;

/** <p>This <code>GraphEngine</code> creates graphing classes that render XML output. The markup used is simplified, per other MathTools XML, and none of the 
    tags has deeply nested children. All components returned are of the 'XML' variety: AxisXML, GrapPaperXML, etc. None of these have any non-standard methods 
    with the exception of <code>RenderableXML</code>. That class has a <code>setID()</code> method which is used to set the necessary ID attributes on the tags 
    it generates. This is handled automatically and transparently, so the script does not need to worry about it! The ID is set when the <code>RenderableXML</code> 
    is added to its GraphPaper. The key used to add the component becomes its ID and this cannot be null or empty! (This is enforced.)</p>
    <p>The 'name=' attribute of the tag, if it exists, is the value used when the original component was created. That is, the 'pKey' arguments in this class, 
    which must be String, become the 'name=' attribute of the tag whenever the component is rendered. Since <code>GraphPaper</code> becomes the root element of 
    the XML document it does not have or require an 'id=' attribute. Its name and type, however, can still be set.</p>
*/
public class GraphEngineXML implements GraphEngine {

  /** The only constructor needed. */
  public GraphEngineXML() { super(); }

  /** Not used. Returns false. */
  public boolean setFeature(String pFeature, Object pValue) { return false; }

  /** Not used. Returns false. */
  public boolean testFeature(String pFeature) { return false; }

  public Axis getAxis(Object pKey) {
    if(pKey != null) return new AxisXML(pKey.toString());
    return new AxisXML();
  }

  public Graphable getGraph(Object pKey) {
    if(pKey != null) return new GraphableXML(pKey.toString());
    return new GraphableXML();
  }

  public GraphPaper getGraphPaper(Object pKey) {
    if(pKey != null) return new GraphPaperXML(pKey.toString());
    return new GraphPaperXML();
  }

  public Polygon getPolygon(Object pKey) {
    if(pKey != null) return new PolygonXML(pKey.toString());
    return new PolygonXML();
  }

  public GraphKeys getGraphKeys() { return new GraphKeysXML(); }

}
