package cox.jmatt.java.MathTools.graphing.xml;

import java.util.*;
import cox.jmatt.java.MathTools.*;
import cox.jmatt.java.MathTools.graphing.*;

/** <code>PolygonXML</code> contains a &lt;Polygon&gt; tag with the usual attributes along with one mandatory child tag. The &lt;Vertices&gt; tag contains the 
    'closed=' attribute along with the vertices, if any, expressed as a space-separated String. Any parameters added will follow.
*/
public class PolygonXML extends RenderableXML implements Polygon {

  private String myPoints = "";
  private boolean amIClosed = true;

  /** Constructor for an unnamed Polygon. */
  public PolygonXML() { super(); }

  /** Build a named Polygon. */
  public PolygonXML(String pName) { super(pName); }

  /** Add a vertex to the Polygon. It is stored and rendered as the content of the &lt;Vertices&gt; tag. Return value is a reference to itself. */
  public Polygon addVertex(double pX, double pY) {
    String sTemp = " "+pX+" "+pY+"";
    myPoints += sTemp;
    return this;
  }

  /** Set the closedness of the Polygon. Default value is true. */
  public void setClosed(boolean pClosed) { amIClosed = pClosed; }

  /** Rendering <code>Polygon</code> yields a &lt;Polygon&gt; tag with the usual 'id=', 'name=' and 'type=' attributes. Vertices are represented as a space-separated 
      String inside a &lt;Vertices&gt; child tag. The closure of the Polygon is a 'closed=' attribute on the Vertices tag. Any parameters will follow.
  */
  public Object render(Object pContext) {
    if(!Question.validString(myID)) myID = "Polygon"+GraphKeysXML.hitCount();
    StringBuffer retval = new StringBuffer("    <Polygon");
    retval.append(" id=\"").append(myID).append("\" ");
    if(Question.validString(myName)) retval.append("name=\"").append(myName).append("\" ");
    if(Question.validString(myType)) retval.append("type=\"").append(myType).append("\" ");
    retval.append(">\n");
    retval.append("      <Vertices closed=\"").append(amIClosed).append("\" >");
    if(myPoints.length() > 1) retval.append(myPoints.substring(1));
    retval.append("</Vertices>\n");
    retval.append(super.render(pContext));
    retval.append("    </Polygon>\n");
    return retval.toString();
  }

}
