package cox.jmatt.java.MathTools.markup.html;

/** <p>This class provides the anchor ('&lt;a&gt;') tag. The 'href=' attribute is <i>mandatory</i> so this class cannot be constructed without a valid value for 
    it. For this reason there is a static construction method and no public constructor.</p>
*/
public class AnchorTag extends SimpleDecorationTag {

  /* Copy constructor ONLY! */
  private AnchorTag() { super(BaseHTag.TagKeys.A_TAG); }

  private AnchorTag(String pHREF) {
    super(BaseHTag.TagKeys.A_TAG);
    _addAttribute("href", pHREF);
  }

  /** Use this method to obtain tag instances. If pHREF is null or blank the method returns null. If pContent is null or blank it is ignored.
      @param pHREF The value of the 'href=' attribute. Must not be null or blank.
      @param pContent The (text) content of the tag. Ignored if null or blank.
  */
  public static final AnchorTag newInstance(String pHREF, String pContent) {
    if(!validString(pHREF)) return null;
    AnchorTag retval = new AnchorTag(pHREF);
    retval.addContent(pContent);
    return retval;
  }

  /** Copy the current tag. */
  public AnchorTag copyOf() {
    _enableCopy();
    return new AnchorTag();
  }

  /** Overridden to preserve the 'href=' attribute. */
  public void reset() {
    String sHREF = _attributes().get("href");
    super.reset();
    _addAttribute("href", sHREF);
  }

}

