package cox.jmatt.java.MathTools.markup.html;

/** <p>This class provides the &lt;blockquote&gt; tag. It is classified as <code>SimpleDecorationTag</code> because in non-strict HTML it is allowed to contain 
    non-block elements. To generate strict HTML or XHTML avoid adding non-block elements to this tag. Also, because of its placement, this tag (only!) has methods 
    to add lists, definitions and tables to itself. Use with care.</p>
*/
public class BQTag extends SimpleDecorationTag {

  /** Standard constructor for editable tags. */
  public BQTag() { super(BaseHTag.TagKeys.BQ_TAG); }

  /** Construct a tag with content. */
  public BQTag(String pContent) { super(BaseHTag.TagKeys.EM_TAG, pContent); }

  /** Copy the current tag. */
  public BQTag copyOf() {
    _enableCopy();
    return new BQTag();
  }

  /** Add a <code>PTag</code>. &lt;div&gt;- and &lt;body&gt;-level tags are ignored. */
  public BQTag addP(PTag pTag) {
    if((pTag != null) && (pTag.level() >= PTag.P_LEVEL)) _addTag(pTag);
    return this;
  }

  /** Add a <code>ListTag</code> to the current block. */
  public BQTag addList(ListTag pTag) {
    _addTag(pTag);
    return this;
  }

  /** Add a <code>DLTag</code> to the current block. */
  public BQTag addDL(DLTag pTag) {
    _addTag(pTag);
    return this;
  }

  /** Add a <code>TableTag</code> to the current block. */
  public BQTag addTable(TableTag pTag) {
    _addTag(pTag);
    return this;
  }

}

