package cox.jmatt.java.MathTools.markup.html;

/** <p>This class provides the definition list tag and container for definition elements: &lt;dl&gt;, &lt;dt&gt; and &lt;dd&gt;. Although the term and definition 
    tags subclass <code>SimpleDecorationTag</code> they are used here.</p>
    <p>Terms and definitions are kept in the order in which they are given and may be intermixed. Since both <code>DTTag</code> and <code>DDTag</code> extend 
    <code>SimpleDecorationTag</code> their text content can contain bold, italic, emphasized, etc. formatting.</p>
*/
public class DLTag extends BaseHTag {

  /** Standard constructor for an empty definition list. */
  public DLTag() { super(BaseHTag.TagKeys.DL_TAG); }

  /** Create a copy of this tag. */
  public DLTag copyOf() {
    _enableCopy();
    return new DLTag();
  }

  /** Add a definition term directly to the container. All defaults apply, a self-reference is returned, and the tag is not created if the text is not valid. */
  public DLTag addTerm(String pDefTerm) {
    if(validString(pDefTerm)) _addTag(new DTTag(pDefTerm));
    return this;
  }

  /** Add a pre-constructed &lt;dt&gt; to the list and return a self-reference. */
  public DLTag addTerm(DTTag pTag) {
    _addTag(pTag);
    return this;
  }

  /** Add a definition to the container with defaults, self-reference, and disregarding of null or blank Strings. */
  public DLTag addDef(String pDefinition) {
    if(validString(pDefinition)) _addTag(new DDTag(pDefinition));
    return this;
  }

  /** Add an existing &lt;dd&gt; tag to the container, ignored if null, returns a self-reference for method chaining. */
  public DLTag addDef(DDTag pTag) {
    _addTag(pTag);
    return this;
  }

}

