package cox.jmatt.java.MathTools.markup.html;

/** <p>This class provides the &lt;hr&gt; tag. Since there is no content construction is fairly simple. The standard constructor produces a page-spanning rule. 
    If the one-argument constructor (width) is used the number is interpreted as follows. If the number is negative it is made positive. If it is between 
    0.0 (inclusive) and 1.0 (exclusive) it is interpreted as a percent and the '%' sign is appended. If the value is 1.0 or greater it is interpreted as a raw 
    pixel width and added as an integer.</p>
*/
public class HRTag extends SimpleDecorationTag {

  /** Standard constructor for editable tags. */
  public HRTag() { super(BaseHTag.TagKeys.HR_TAG); }

  /** Construct a tag with content. */
  public HRTag(double pWidth) {
    super(BaseHTag.TagKeys.HR_TAG);
    setWidth(pWidth);
  }

  /** Copy the current tag. */
  public HRTag copyOf() {
    _enableCopy();
    return new HRTag();
  }

  /** Set the width of this rule. The number is processed as described above. */
  public HRTag setWidth(double pWidth) {
    pWidth = Math.abs(pWidth);
    if(pWidth < 1.0) {
      _addAttribute("width", ((int) (pWidth * 100)) + "%");
    } else {
      _addAttribute("width", "" + ((int) pWidth));
    }
    return this;
  }

  /** This tag is always empty! This method always returns null. */
  protected String _content() { return null; }
  /** Always sets null (clears) content. Not that this tag would ever <i>have</i> content. */
  protected void _setContent(String pContent) { super._setContent(null); }

}

