package cox.jmatt.java.MathTools.markup.html;

/** <p>This class implements the &lt;p&gt; tag, the paragraph block. It also forms the superclass for the other block-structure tags &lt;div&gt; and &lt;body&gt;. 
    Each of these has a level constant associated with it and as a rule these tags can only contain other tags whose level is greater. The partial exception is 
    that &lt;div&gt; tags can contain other &lt;div&gt; tags. MathTools considers this tag the most atomic structural block-level tag so it can contain any of 
    the <code>SimpleDecorationTag, ListTag, DLTag, IMGTag,</code> and <code>TableTag</code> instances.</p>
    <p>For numeric comparison, BODY_LEVEL &lt; DIV_LEVEL &lt; P_LEVEL.</p>
*/
public class PTag extends ContainerTag {

  /** Static constant for the outermost container level ('&lt;body&gt;'). */
  public static final byte BODY_LEVEL = 0;
  /** &lt;div&gt;-level constant. */
  public static final byte DIV_LEVEL = 5;
  /** Level constant for paragraph blocks. */
  public static final byte P_LEVEL = 10;

  /** This field is used to hold the _LEVEL value of this tag. It is used for comparison with the <code>level()</code> method to determine containment. */
  protected byte myLevel = -1;

  public PTag() {
    super(BaseHTag.TagKeys.P_TAG);
    myLevel = P_LEVEL;
  }

  /** Protected subclass constructor. */
  protected PTag(BaseHTag.TagKeys pTag) { super(pTag); }

  /** Create a copy of this tag. */
  public PTag copyOf() {
    _enableCopy();
    return new PTag();
  }

  /** <p>This method is used to return the _LEVEL constant associated with this particular tag class. It is used to determine which tags can contain what.</p>
      <p>The standard rule is that no tag class can contain any tag of lesser or equal level to itself: &lt;div&gt; can contain &lt;p&gt; but not vise versa. 
      The only exception is that &lt;div&gt; tags can contain other &lt;div&gt; tags.</p>
  */
  public byte level() { return P_LEVEL; }

  /** Add a <code>PTag</code> subclass. */
  public final void add(PTag pTag) {
    if((pTag == null) || (pTag.level() <= myLevel)) return;
    _addTag(pTag);
  }

  /** Add a <code>ListTag</code>. */
  public void addList(ListTag pTag) { _addTag(pTag); }

  /** Add a <code>DLTag</code>. */
  public void addDef(DLTag pTag) { _addTag(pTag); }

  /** Add a <code>TableTag</code>. */
  public void addTable(TableTag pTag) { _addTag(pTag); }

  /** Add an <code>IMGTag</code>. */
  public void addImage(IMGTag pTag) { _addTag(pTag); }

  /** Add a <code>SimpleDecorationTag</code> instance. Item tags are not added. */
  public void add(SimpleDecorationTag pTag) {
    if((pTag != null) && (pTag.isInBody())) _addTag(pTag);
  }

}

