package cox.jmatt.java.MathTools.markup.html;

/** <p>This abstract class provides tags used for simple text decoration plus several others that are similarly straightforward. These classes are provided to 
    allow these tags to benefit from default attributes and settings. In addition to the obvious tag types ('&lt;b&gt;' et. al.) this class is also used for 
    <i>item tags</i>.
    <p><i>Item tags</i> are tags that appear as 'items' within other specialized containers: &lt;li&gt;, &lt;dd&gt; and similar tags. They may contain text 
    decoration tags but not instances of each other, therefor they are excluded from the <code>_addTag()</code> method. (One use of the <code>isInBody()</code> 
    method.) The specific container tags have methods to add their own tags.</p>
*/
public abstract class SimpleDecorationTag extends ContainerTag {

  /** Constructor for subclassing. */
  protected SimpleDecorationTag(BaseHTag.TagKeys pKey) { super(pKey); }

  /** Constructor that includes content. */
  protected SimpleDecorationTag(BaseHTag.TagKeys pKey, String pContent) {
    super(pKey);
    _setContent(pContent);
  }

  /** Add a decoration tag instance. Item tags will not be added. */
  public void add(SimpleDecorationTag pTag) {
    if((pTag != null) && (pTag.isInBody())) _addTag(pTag);
  }

}

