package cox.jmatt.java.MathTools.markup.html;

/** <p>This class provides the &lt;td&gt; and &lt;th&gt; tags. Both tag classes use the same defaults, so setting 'TD_TAG' also affects table headers. (This is 
    neither bug nor feature, simply the way of things.) To change a &lt;td&gt; to &lt;th&gt; call <code>setHeader(true)</code>. All other settings remain the 
    same.</p>
*/
public class TDTag extends SimpleDecorationTag {

  /** Standard constructor for editable tags. */
  public TDTag() { super(BaseHTag.TagKeys.TD_TAG); }

  /** Construct a tag with content. */
  public TDTag(String pContent) { super(BaseHTag.TagKeys.TD_TAG, pContent); }

  /** Copy the current tag. */
  public TDTag copyOf() {
    _enableCopy();
    return new TDTag();
  }

  /** Set the 'rowspan=' attribute. Ignored if negative, removed if null. */
  public void setRowSpan(Integer pSpan) { _clampNAttribute("rowspan", pSpan, 0.0, null, true); }

  /** Set the 'colspan=' attribute. Ignored if negative, removed if null. */
  public void setColSpan(Integer pSpan) { _clampNAttribute("colspan", pSpan, 0.0, null, true); }

  /** Use this method to change &lt;td&gt; to &lt;th&gt;. The change takes place immediately and does survive <code>reset()</code>. Returns a self-reference.
      @param isTH 'true' for &lt;th&gt;, false for &lt;td&gt;.
  */
  public TDTag setHeader(boolean isTH) {
    myTagName = (isTH?"th":"td");
    return this;
  }

}

