package cox.jmatt.java.MathTools.markup.html;

/** <p>This class represents a single row ('&lt;tr&gt;' ... '&lt;/tr&gt;') in a &lt;table&gt;. It makes provision for holding &lt;td&gt; elements and for creating 
    rows of them.</p>
*/
public class TRTag extends BaseHTag {

  /** Constructor for an empty row, ready to fill. */
  public TRTag() { super(BaseHTag.TagKeys.TR_TAG); }

  /** <p>Turn an Object[] array into a table row. If the array is null or empty null is returned. Non-null elements are <code>toString()</code>'ed to add them, 
      null elements are added as a single space so the resulting &lt;td&gt; tag will be empty.</p>
      <p>If 'cellIDprefix' is not null or blank it is used to set the ID of the cell. If used it is prepended to the cell number and set as the ID of the resulting 
      &lt;td&gt; tag. If null or blank it is ignored.</p>
      @param pRow The Object[] array to be converted to a row. If null or empty the return value is null.
      @param pCellID The cell ID prefix. If defined it is used to set the cell's 'id=' attribute.
      @param pIsTH 'true' for a header-cell row, false for a standard &lt;td&gt; row.
  */
  public static TRTag createRow(Object[] pRow, String pCellID, boolean pIsTH) {
    if((pRow == null) || (pRow.length < 1)) return null;
    boolean gotID = validString(pCellID);
    TRTag retval = new TRTag();
    TDTag cellTag = null;
    for(int i=0; i < pRow.length; i++) {
      cellTag = new TDTag((pRow[i]==null)?" ":pRow[i].toString());
      cellTag.setHeader(pIsTH);
      if(gotID) cellTag.setID(pCellID+i);
      retval.addCell(cellTag);
    }
    return retval;
  }

  /** Instance version of <code>createRow()</code>. */
  public TRTag newRow(Object[] pRow, String pCellID, boolean isTH) { return createRow(pRow, pCellID, isTH); }

  /** Copy this row verbatim. */
  public TRTag copyOf() {
    _enableCopy();
    return new TRTag();
  }

  /** Add a cell directly to the row. Defaults are in effect, nothing done if the String is null or blank. */
  public TRTag addCell(String pContent) {
    if(validString(pContent)) _addTag(new TDTag(pContent));
    return this;
  }

  /** Add an existing &lt;td&gt; to the row. This works both for cells <i>and</i> header cells. */
  public TRTag addCell(TDTag pTag) {
    _addTag(pTag);
    return this;
  }

  /** Add a <b>heading</b> cell to this row. If used once this should be used subsequently but this is not enforced! */
  public TRTag addHeading(String pContent) {
    if(validString(pContent)) _addTag(new TDTag(pContent).setHeader(true));
    return this;
  }

}

