package cox.jmatt.java.MathTools.markup.html;

/** <p>This class provides the &lt;table&gt; tag. It contains methods to add rows to an existing table or to create one from an Object[][] array. */
public class TableTag extends BaseHTag {

  /** Create an empty <code>TableTag</code>. */
  public TableTag() { super(BaseHTag.TagKeys.TABLE_TAG); }

  /** Produce a copy of this table. */
  public TableTag copyOf() {
    _enableCopy();
    return new TableTag();
  }

  /** <p>Create a &lt;table&gt; based on an Object[][] array. The first index is a row index, the second is the cells within that row. A row ID prefix can be 
      given as can a column ID prefix. The row number is appended to the end of the row ID prefix, if it is specified, and the cell ID prefix is sent into each 
      <code>TRTag.createRow()</code> call. If 'bUseRowIDforCells' is true the row ID prefix is concatenated with the cell ID prefix (if present) or simply sent 
      in (cell ID not specified) giving a way to uniquely ID both rows and cells.</p>
      <p>Null entries within a row become empty &lt;td&gt; elements, null rows are ignored. If a row is null, originally or when converted to a row, it is ignored 
      and it does not affect the row count.</p>
      @param pTable The Object[][] array to convert to a table. If null or empty the return value is null.
      @param pRowID The row ID prefix. If defined this is prepended to the row number for each &lt;tr&gt; tag ID.
      @param pCellID The cell ID prefix sent into each row.
      @param bUseRowIDforCells true to prefix the cell ID prefix with the (current) row ID, false to omit it.
      @param bFirstRowTH 'true' if the first row (pTable[0][]) should be &lt;th&gt; elements.
  */
  public static final TableTag createTable(Object[][] pTable, String pRowID, String pCellID, boolean bUseRowIDforCells, boolean bFirstRowTH) {
    if((pTable == null) || (pTable.length < 1)) return null;
    TableTag retval = new TableTag();
    boolean gotRowID = validString(pRowID), gotCellID = validString(pCellID);
    String sCellPrefix = (gotCellID?pCellID:"");
    int iRowCount = 0;
    TRTag trTemp = null;
    for(int i = 0; i < pTable.length; i++) {
      if((pTable[i] == null) || (pTable[i].length < 1)) continue;
      trTemp = TRTag.createRow(pTable[i], ((gotRowID && bUseRowIDforCells)?(pRowID+iRowCount+sCellPrefix):pCellID), ((i == 0) && bFirstRowTH));
      if(gotRowID) trTemp.setID(pRowID+iRowCount);
      retval.addRow(trTemp);
      if(trTemp != null) iRowCount++;
    }
    return retval;
  }

  /** Instance flavor of <code>createTable()</code>. */
  public TableTag newTable(Object[][] pTable, String pRowID, String pCellID, boolean bUseRowIDforCells, boolean bFirstRowTH) {
    return createTable(pTable, pRowID, pCellID, bUseRowIDforCells, bFirstRowTH);
  }

  /** Add an existing row to the current table. */
  public TableTag addRow(TRTag pTag) {
    if(pTag != null) _addTag(pTag);
    return this;
  }

  /** <p>Add a row based on a one-dimensional Object[] array. This uses the {@link cox.jmatt.java.MathTools.markup.html.TRTag#createRow TRTag.createRow} method. 
      Parameters are the Object[] array to create the row and the (optional) cell ID header. Return value is a self-reference.</p>
  */
  public TableTag addRow(Object[] pRow, String pCellID, boolean bIsTH) {
    _addTag(TRTag.createRow(pRow, pCellID, bIsTH));
    return this;
  }

}

