package cox.jmatt.java.MathTools.markup.latex;

import java.util.*;
import cox.jmatt.java.MathTools.*;

/** This class provides the '{equation}' and '{eqnarray}' blocks. Equation numbering can be enabled (default) or disabled. There is also a method to add '/label...' 
    tags.
*/

public class LEquation extends LaBaseTag {

  private String myCurName = "equation", lineBreak = "";
  private boolean myNumbering = true;

  /** Standard constructor. All information is internal. */
  public LEquation() { super(null, null, "equation", null, null); }

  private LEquation(String pEnv, String pOptions, String pContent) { super(null, null, pEnv, pOptions, pContent); }

  private void resetName() { myEnvironment = (myCurName + (myNumbering?"":"*")).trim(); }

  /** Toggle between 'equation' and 'eqnarray'. Set pArray to 'true' to enable an equation array. Returns a self-reference. */
  public LEquation setArray(boolean pArray) {
    myCurName = pArray?"eqnarray":"equation";
    resetName();
    return this;
  }

  /** Enable/disable equation numbering. It is on (true) by default. */
  public LEquation setNumbered(boolean pNumbered) {
    myNumbering = pNumbered;
    resetName();
    lineBreak = "";
    return this;
  }

  /** Add an equation and optional label. If the equation is null or blank nothing happens. If the label is null or blank it is ignored.
  */
  public LEquation add(String pEquation, String pLabel) {
    if(!Question.validString(pEquation)) return this;
    if(Question.validString(pLabel)) {
      _addContent(lineBreak + "\\label{"+pLabel+"}\n");
      lineBreak = "";
    }
    _addContent(lineBreak + pEquation);
    lineBreak = DBSN;
    return this;
  }

  /** Overridden to reset to 'equation' and enable numbering. */
  public void reset() {
    myCurName = "equation"; myNumbering = true;
    resetName();
    super.reset();
  }

  /** Return an exact copy of this tag. */
  public LEquation copyOf() {
    LEquation retval = new LEquation(myEnvironment, myOptions, _getContent());
    retval.setLabel(mySectionLabel);
    return retval;
  }

  /** Copy this tag with end tag disabled. */
  public LEquation copyOfSET() {
    _setETEn(false);
    return copyOf();
  }

}

