package cox.jmatt.java.MathTools.markup.latex;

import java.util.*;
import cox.jmatt.java.MathTools.*;

/** <p>This class provides LaTeX list blocks, both '{itemize}' and '{enumerate}'. By default an unordered list is created but the 'setEnumerate()' method changes 
    between the two types as needed.</p>
*/

public class LaListTag extends LaBaseTag {

  /** All the necessary information is internal. */
  public LaListTag() { super(null, null, "itemize", null, null); }

  private LaListTag(String pEnv, String pOptions, String pContent) { super(null, null, pEnv, pOptions, pContent); }

  /** Switch between '{itemize}' and '{enumerate}' modes. The change takes place immediately and is not graven in stone until a 'toString...' method is called. 
      A self-reference is returned to allow chaining.
      @param pEnumerate true for 'enumerate,' false for 'itemize.'
  */
  public LaListTag setEnumerate(boolean pEnumerate) {
    myEnvironment = pEnumerate?"enumerate":"itemize";
    return this;
  }

  /** Add an item to the list, full-power version. This method allows both the bullet character AND the item text to be specified. A null or blank bullet String 
      is disregarded and the item is added. If the item content is null or blank it is silently ignored. If an item is successfully added a newline is appended 
      at the end.
      @param pBullet The bullet symbol to use. If null or blank a standard '/item{}' tag is created.
      @param pContent The content of the item. NOTHING is done if this is null or blank.
  */
  public LaListTag addItem(String pBullet, String pContent) {
    if(!Question.validString(pContent)) return this;
    if(Question.validString(pBullet)) {
      _addContent("\\item["+pBullet+"]{"+pContent+"}\n");
    } else {
    _addContent("\\item{"+pContent+"}\n");
    }
    return this;
  }

  /** Add a String[] array of items to the list. Each element is added singly with the bullet character if one is provided. */
  public LaListTag addItems(String pBullet, String[] pItems) {
    if(pItems == null) return this;
    for(String sItem : pItems) { addItem(pBullet, sItem); }
    return this;
  }

  /** Overridden to reset the list type to unordered as well as clear the content. */
  public void reset() {
    myEnvironment = "itemize";
    super.reset();
  }

  /** Convenience method for <code>addItem(null, pContent)</code>. */
  public LaListTag addItem(String pContent) { return addItem(null, pContent); }

  /** Return a copy of this tag at its CURRENT state. */
  public LaListTag copyOf() {
    LaListTag retval = new LaListTag(myEnvironment, myOptions, _getContent());
    retval.setLabel(mySectionLabel);
    return retval;
  }

  /** Return a copy of this tag with end tag disabled. */
  public LaListTag copyOfSET() {
    _setETEn(false);
    return copyOf();
  }

}

