package cox.jmatt.java.MathTools.markup.latex;

import java.util.*;
import cox.jmatt.java.MathTools.*;

/** This class provides LaTeX table-generating ('tabular') capability. When instantiated it creates an empty table with a single center-aligned column. Any number 
    of columns are available but the column alignment String <b>must</b> be set.
*/

public class LaTable extends LaBaseTag {

  /** Basic constructor for an empty table. */
  public LaTable() {
    super(null, null, "tabular", null, null);
    myEnvParams = new String[1];
    myEnvParams[0] = "c";
  }

  /** Deluxe constructor that allows column alignment to be set. If 'pColAlign' is null or blank it defaults to a single centered column.
      @param pColAlign The column alignment specifier for this table.
  */
  public LaTable(String pColAlign) {
    super(null, null, "tabular", null, null);
    myEnvParams = new String[1];
    myEnvParams[0] = Question.validString(pColAlign)?pColAlign:"c";
  }

  /** Set the column alignment. THIS IS MANDATORY! If set to null or blank the default is a single center-aligned column: '{c}'. */
  public LaTable setColAlign(String pAlign) {
    myEnvParams[0] = Question.validString(pAlign)?pAlign:"c";
    return this;
  }

  /** Add a row to the table. No column-counting is enforced so it is up to the user to make certain each row contains the right number of entries. The column 
      break ('&amp;') and end-of-row delimiters are added automatically.
      @param pRow The row of elements to add to the table.
  */
  public LaTable addRow(String... pRow) {
    if((pRow == null) || (pRow.length < 1)) return this;
    StringBuffer theRow = new StringBuffer();
    for(int i=0; i < pRow.length; i++) {
      theRow.append((pRow[i] == null)?" ":pRow[i]);
      theRow.append((i < pRow.length - 1)?" & ":DBSN);
    }
    _addContent(theRow.toString());
    return this;
  }

  /** Add a horizontal line ('/hline') to the table. */
  public LaTable addLine() {
    _addContent("\\hline\n");
    return this;
  }

  /** Add a '/cline{}' tag to the table. If either number is less than one nothing happens.
      @param pLN The left-side number in the curly braces.
      @param pRN The right-side number.
  */
  public LaTable addLine(int pLN, int pRN) {
    if((pLN < 1) || (pRN < 1)) return this;
    _addContent("\\cline{"+pLN+" - "+pRN+"}\n");
    return this;
  }

  /** Overridden to reset local settings. */
  public void reset() {
    super.reset();
    myEnvParams[0] = "c";
  }

  /** Standard copy method. */
  public LaTable copyOf() { return null; }

  /** Copy method, end tag suppressed. */
  public LaTable copyOfSET() {
    _setETEn(false);
    return copyOf();
  }

}

