package cox.jmatt.java.MathTools.markup.latex;

import java.util.*;
import cox.jmatt.java.MathTools.*;

/** <p>This class can be used to generate any LaTeX tag. Although primarily designed for debugging and development it is useful for scripting and other applications. 
    Every aspect of <code>LaBaseTag</code> can be set and there are <b>no restrictions</b> on the values. USE WITH CARE! The only thing that cannot be set to null 
    or blank is the tag name; it is supplied with the constructor. The setter methods return a self-reference to allow method chaining.</p>
*/

public class LaTag extends LaBaseTag {

  /** Basic, default constructor. Calls <code>super(pName, null, null, null)</code>. 
    @param pName The tag name. This is the ONLY place it can be set and it still cannot be null or blank!
  */
  public LaTag(String pName) { super(pName, null, null, null, null); }

  /** Set or clear the initial environment. */
  public LaTag setEnvironment(String pEnv) {
    myEnvironment = pEnv;
    return this;
  }

  /** Add content to the tag, publicly. */
  public LaTag addContent(String pContent) {
    _addContent(pContent);
    return this;
  }

  /** Set the square-brace options. */
  public LaTag setOptions(String pOpts) {
    myOptions = pOpts;
    return this;
  }

  /** Set any extra curly-brace parameters. */
  public LaTag setEnvParams(String... pParams) {
    myEnvParams = pParams;
    return this;
  }

  /** Set or clear the end tag name. */
  public LaTag setEndTag(String pTag) {
    myEndTag = pTag;
    return this;
  }

  /** Enable/disable the end tag. */
  public LaTag setEndTagEnabled(boolean pEnable) {
    _setEndTagEnabled(pEnable);
    return this;
  }

  public LaTag copyOf() { return new LaTag(myTagName); }

  public LaTag copyOfSET() {
    _setETEn(false);
    return copyOf();
  }

}

