package cox.jmatt.java.MathTools.markup.svg;

import cox.jmatt.java.MathTools.markup.*;

/** <p>This class implements the &lt;clipPath&gt; tag. It lives inside the &lt;defs&gt; block and is used to describe a clipping path applied to another graphic. 
    Because of this a valid ID is mandatory. There is no public constructor and the factory method will NOT build a tag around a null or blank ID.</p>
*/
public class ClipTag extends DrawableContainerTag {

  private String myID = null;

  private ClipTag() { super(StyleTag.TagKeys.CLIP_TAG); }

  /** This is the only way to construct an instance of this class. If the ID argument is null or blank the return value is null! */
  public static ClipTag newInstance(String pTagID) {
    if(!validString(pTagID)) return null;
    ClipTag retval = new ClipTag();
    retval.setID(pTagID);
    return retval;
  }

  /** Overridden to ensure the ID set is valid. */
  public void setID(String pID) {
    if(!validString(pID)) return;
    myID = pID;
    super.setID(pID);
  }

  /** Overridden to preserve the ID. To change it use <code>reset(String)</code> or re-set manually afterward. */
  public void reset() {
    super.reset();
    setID(myID);
  }

  /** Call this method to clear data <b>and</b> re-set the ID in one fell swoop. */
  public void reset(String newID) {
    super.reset();
    setID(newID);
  }

  /** Set the unit spec for this tag. This is 'objectBoundingBox' or 'userSpaceOnUse'.
      @see DrawableContainerTag#_setOBUS
  */
  public ClipTag setClipPathUnits(boolean pUserSpace) {
    _setOBUS("clipPathUnits", pUserSpace);
    return this;
  }

  /** <p>Configure this tag. The only class-unique key it recognizes is 'markup.svg.Clip.pathUnits' which accepts a value of 'true' or 'false'. Of course the 
      <code>StyleTag</code> keys may also be set.</p>
  */
  public void configureTag(String pPrefix, java.util.Properties pPizza) {
    if(badPizza(pPizza)) return;
    pPrefix = _fixKey(pPrefix, "svg.Clip");
    if(pPizza.containsKey(pPrefix+".pathUnits")) setClipPathUnits(_asBoolean(pPizza, pPrefix, ".pathUnits"));
    super.configureTag(pPrefix, pPizza);
  }

}

