package cox.jmatt.java.MathTools.markup.svg;

import cox.jmatt.java.MathTools.markup.*;

/** <p>This class implements the group tag, &lt;g&gt;. Although this tag can appear in the body of a document MathTools considers it &lt;defs&gt;-only. For 
    that reason the constructor is private and the factory method requires a valid ID before it will create an instance. To add a <code>GroupTag</code> outside 
    the &lt;defs&gt; block call <code>addRaw(groupTagInstance.toString())</code> on this tag's container. Better yet, let things go where they should and create 
    a <code>UseTag</code> to reference the group!</p>
*/
public class GroupTag extends DrawableContainerTag {

  private String myID = null;

  /** Constructor provided for subclasses. */
  protected GroupTag(StyleTag.TagKeys pKey) { super(pKey); }

  private GroupTag() { super(StyleTag.TagKeys.GROUP_TAG); }

  /** This is the only way to construct an instance of this class. If the ID argument is null or blank the return value is null! */
  public static GroupTag newInstance(String pTagID) {
    if(!validString(pTagID)) return null;
    GroupTag retval = new GroupTag();
    retval.setID(pTagID);
    return retval;
  }

  /** Overridden to ensure the ID set is valid. */
  public void setID(String pID) {
    if(!validString(pID)) return;
    myID = pID;
    super.setID(pID);
  }

  /** Overridden to preserve the ID. To change it use <code>reset(String)</code> or re-set manually afterward. */
  public void reset() {
    super.reset();
    setID(myID);
  }

  /** Call this method to clear data <b>and</b> re-set the ID in one fell swoop. */
  public void reset(String newID) {
    super.reset();
    setID(newID);
  }

  /** Set or clear the title of this tag. */
  public GroupTag setTitle(String pTitle) { _setTitle(pTitle); return this; }

  /** Set or clear the description of this tag. */
  public GroupTag setDesc(String pDescription) { super._setDesc(pDescription); return this; }

  /** <p>Pizza-configure this tag. The prefix is 'svg.markup.Group' and the two settable keys are '.title' for the title and '.desc' for the description. */
  public void configureTag(String pPrefix, java.util.Properties pPizza) {
    if(badPizza(pPizza)) return;
    pPrefix = _fixKey(pPrefix, "svg.markup.Group");
    if(pPizza.containsKey(pPrefix+".title")) setTitle(pPizza.getProperty(pPrefix+".title"));
    if(pPizza.containsKey(pPrefix+".desc")) setDesc(pPizza.getProperty(pPrefix+".desc"));
    super.configureTag(pPrefix, pPizza);
  }

}

