package cox.jmatt.java.MathTools.markup.svg;

import cox.jmatt.java.MathTools.markup.*;

/** <p>This class generates SVG &lt;line&gt; tags; lines from one point to another. Once created all the data can be changed: endpoints, style and transforms. 
    Default values can also be set. Endpoint values are given as Object type Double; if one of these values is null it is <b>ignored</b> and not removed. This 
    allows one coordinate to be changed without affecting the others.</p>
*/
public class LineTag extends StyleTag {

  /** Standard constructor. No endpoints set. */
  public LineTag() { super(TagKeys.LINE_TAG); }

  /** Copy this class. */
  public LineTag copyOf() {
    _enableCopy();
    return new LineTag();
  }

  /** Create a LineTag with all data present. */
  public LineTag(Double X1, Double Y1, Double X2, Double Y2) {
    super(TagKeys.LINE_TAG);
    setData(X1, Y1, X2, Y2);
  }

  /** Set line-specific data: the endpoints! This draws a line from (X1, Y1) to (X2, Y2). */
  public LineTag setData(Double X1, Double Y1, Double X2, Double Y2) {
    _addNAttribute("x1", X1); _addNAttribute("y1", Y1);
    _addNAttribute("x2", X2); _addNAttribute("y2", Y2);
    return this;
  }

}

