package cox.jmatt.java.MathTools.markup.svg;

import java.util.*;
import cox.jmatt.java.MathTools.markup.*;

/** <p>Use this class to place markers on any entity able to accept them. The entire marker can be set or the beginning, middle, and end can be set separately. 
    This is also a <code>StylePen</code> subclass so any of those options are also available.</p>
*/
public class MarkerPen extends StylePen {

  /** Standard constructor. */
  public MarkerPen() { super(); }

  /** Return a copy of this <code>MarkerPen()</code> */
  public MarkerPen copyOf() {
    _enableCopy();
    return new MarkerPen();
  }

  
  /** Set the marker to be used at the start of the markable thing. */
  public MarkerPen setMarkerStart(String pURI) {
    if(Tag.validString(pURI)) {
      _addDeclaration("marker-start", "url("+pURI+")");
    } else {
      _addDeclaration("marker-start", null);
    }
    return this;
  }

  /** Set the marker to be used at the midpoint(s) of the markable thing. */
  public MarkerPen setMarkerMid(String pURI) {
    if(Tag.validString(pURI)) {
      _addDeclaration("marker-mid", "url("+pURI+")");
    } else {
      _addDeclaration("marker-mid", null);
    }
    return this;
  }

  /** Set the marker to be used at the end of the markable thing. */
  public MarkerPen setMarkerEnd(String pURI) {
    if(Tag.validString(pURI)) {
      _addDeclaration("marker-end", "url("+pURI+")");
    } else {
      _addDeclaration("marker-end", null);
    }
    return this;
  }

  /** Set the marker to be used on all markable points of the markable thing. */
  public MarkerPen setMarker(String pURI) {
    if(Tag.validString(pURI)) {
      _addDeclaration("marker", "url("+pURI+")");
    } else {
      _addDeclaration("marker", null);
    }
    return this;
  }


}

