package cox.jmatt.java.MathTools.markup.svg;

import java.util.Properties;
import cox.jmatt.java.MathTools.markup.*;

/** <p>This class furnishes instances of SVG tag classes. It can also be used to set default values. <code>MathDrawSVG</code> serves the same purpose as the 
    <code>graphing.GraphEngine</code> class except that it's a part of the SVG package and not <code>graphing.*</code>. The various 'get()' methods echo the 
    relevant constructors for the classes they furnish.</p>
*/
public class MathDrawSVG {

  private Properties myPizza = null;
  private String myPrefix = null;

  /** Scripting constructor and the only one needed. Ever! */
  public MathDrawSVG() { super(); }

  /** Set or clear a <code>Properties</code> object for tag configuration. If not null this pizza is used to configure all generated tag classes and pens with 
      the exception of <code>TransformPen</code>. Setting null clears it and empty instances are ignored.
      @param pPrefix The configuration key prefix if desired.
      @param pPizza The configuration properties for all generated tags and pens. Null clears it.
  */
  public void setConfigPizza(String pPrefix, Properties pPizza) {
    myPrefix = pPrefix;
    myPizza = pPizza;
  }

  /** Fetch a default <code>LineTag</code>. */
  public LineTag getLineTag() {
    LineTag retval = new LineTag();
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Create a <code>LineTag</code> with all specific data specified. */
  public LineTag getLineTag(Double X1, Double Y1, Double X2, Double Y2) {
    LineTag retval =  new LineTag(X1, Y1, X2, Y2);
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Set the default pen number for subsequent <code>LineTag</code> instances. */
  public void setLinePen(int pPen) { StyleTag.TagKeys.LINE_TAG.setDefaultPen(pPen); }
  /** Set the default <code>StylePen</code> for subsequent <code>LineTag</code> instances. */
  public void setLineStyle(StylePen pPen) { StyleTag.TagKeys.LINE_TAG.setDefaultStyle(pPen); }
  /** Set the default transforms for subsequent <code>LineTag</code>s. */
  public void setLineTransform(TransformPen pPen) { StyleTag.TagKeys.LINE_TAG.setDefaultTransforms(pPen); }

  /** Fetch a default <code>CircleTag</code>. */
  public CircleTag getCircleTag() {
    CircleTag retval =  new CircleTag();
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Create a <code>CircleTag</code> with all specific data specified. */
  public CircleTag getCircleTag(Double X1, Double Y1, Double rad) {
    CircleTag retval =  new CircleTag(X1, Y1, rad);
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Set the default pen number for subsequent <code>CircleTag</code> instances. */
  public void setCirclePen(int pPen) { StyleTag.TagKeys.CIRCLE_TAG.setDefaultPen(pPen); }
  /** Set the default <code>StylePen</code> for subsequent <code>CircleTag</code> instances. */
  public void setCircleStyle(StylePen pPen) { StyleTag.TagKeys.CIRCLE_TAG.setDefaultStyle(pPen); }
  /** Set the default transforms for subsequent <code>CircleTag</code>s. */
  public void setCircleTransform(TransformPen pPen) { StyleTag.TagKeys.CIRCLE_TAG.setDefaultTransforms(pPen); }
  /** (Re-)Set the configuration data for points. */
  public void setPointData(double pSize, String pColor) { CircleTag.setPointData(pSize, pColor); }
  /** Create a point at (pX, pY). */
  public String getPoint(double pX, double pY) { return CircleTag.getPoint(pX, pY); }

  /** Fetch a default <code>EllipseTag</code>. */
  public EllipseTag getEllipseTag() {
    EllipseTag retval = new EllipseTag();
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Create a <code>EllipseTag</code> with all specific data specified. */
  public EllipseTag getEllipseTag(Double X1, Double Y1, Double X2, Double Y2) {
    EllipseTag retval = new EllipseTag(X1, Y1, X2, Y2);
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Set the default pen number for subsequent <code>EllipseTag</code> instances. */
  public void setEllipsePen(int pPen) { StyleTag.TagKeys.ELLIPSE_TAG.setDefaultPen(pPen); }
  /** Set the default <code>StylePen</code> for subsequent <code>EllipseTag</code> instances. */
  public void setEllipseStyle(StylePen pPen) { StyleTag.TagKeys.ELLIPSE_TAG.setDefaultStyle(pPen); }
  /** Set the default transforms for subsequent <code>EllipseTag</code>s. */
  public void setEllipseTransform(TransformPen pPen) { StyleTag.TagKeys.ELLIPSE_TAG.setDefaultTransforms(pPen); }

  /** Fetch a default <code>RectangleTag</code>. */
  public RectangleTag getRectangleTag() {
    RectangleTag retval = new RectangleTag();
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Create a sharp-cornered <code>RectangleTag</code> with all specific data specified. */
  public RectangleTag getRectangleTag(Double X1, Double Y1, Double pWidth, Double pHeight) {
    RectangleTag retval = new RectangleTag(X1, Y1, pWidth, pHeight);
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Create a rounded <code>RectangleTag</code>. */
  public RectangleTag getRectangleTag(Double X1, Double Y1, Double pWidth, Double pHeight, Double radX, Double radY) {
    RectangleTag retval = new RectangleTag(X1, Y1, pWidth, pHeight, radX, radY);
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Set the default pen number for subsequent <code>RectangleTag</code> instances. */
  public void setRectanglePen(int pPen) { StyleTag.TagKeys.RECTANGLE_TAG.setDefaultPen(pPen); }
  /** Set the default <code>StylePen</code> for subsequent <code>RectangleTag</code> instances. */
  public void setRectangleStyle(StylePen pPen) { StyleTag.TagKeys.RECTANGLE_TAG.setDefaultStyle(pPen); }
  /** Set the default transforms for subsequent <code>RectangleTag</code>s. */
  public void setRectangleTransform(TransformPen pPen) { StyleTag.TagKeys.RECTANGLE_TAG.setDefaultTransforms(pPen); }

  /** Fetch a default <code>PolygonTag</code>. */
  public PolygonTag getPolygonTag() {
    PolygonTag retval = new PolygonTag();
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Create a <code>Poly<u>line</u>Tag</code>. It will not be closed: <code>setPolygon(false)</code>. */
  public PolygonTag getPolylineTag() {
    PolygonTag retval = new PolygonTag().setPolygon(false);
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Set the default pen number for subsequent <code>PolygonTag</code> instances. */
  public void setPolygonPen(int pPen) { StyleTag.TagKeys.POLYGON_TAG.setDefaultPen(pPen); }
  /** Set the default <code>StylePen</code> for subsequent <code>PolygonTag</code> instances. */
  public void setPolygonStyle(StylePen pPen) { StyleTag.TagKeys.POLYGON_TAG.setDefaultStyle(pPen); }
  /** Set the default transforms for subsequent <code>PolygonTag</code>s. */
  public void setPolygonTransform(TransformPen pPen) { StyleTag.TagKeys.POLYGON_TAG.setDefaultTransforms(pPen); }

  /** Fetch a default <code>TextTag</code>. */
  public TextTag getTextTag() {
    TextTag retval = new TextTag();
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Create a <code>TextTag</code> with all specific data specified. */
  public TextTag getTextTag(Double X1, Double Y1, String pContent) {
    TextTag retval = new TextTag(X1, Y1, pContent);
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Set the default pen number for subsequent <code>TextTag</code> instances. */
  public void setTextPen(int pPen) { StyleTag.TagKeys.TEXT_TAG.setDefaultPen(pPen); }
  /** Set the default <code>StylePen</code> for subsequent <code>TextTag</code> instances. */
  public void setTextStyle(StylePen pPen) { StyleTag.TagKeys.TEXT_TAG.setDefaultStyle(pPen); }
  /** Set the default transforms for subsequent <code>TextTag</code>s. */
  public void setTextTransform(TransformPen pPen) { StyleTag.TagKeys.TEXT_TAG.setDefaultTransforms(pPen); }
  /** Create and return a <code>TSpanTag</code> with nothing pre-set. */
  public TSpanTag getTSpanTag() {
    TSpanTag retval = new TSpanTag();
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Create a TSpanTag around a String of text. Since this is automatically a part of a &lt;text&gt; tag it has no methods for setting default values for 
      styling or transformation; these must be handled on a per-instance basis!
  */
  public TSpanTag getTSpanTag(String pText) {
    TSpanTag retval = new TSpanTag(pText);
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }

  /** Fetch a <code>UseTag</code> instance. The HREF is mandatory and must not be null or blank. */
  public UseTag getUseTag(String HREF) {
    UseTag retval = UseTag.newInstance(HREF);
    if(!((myPizza == null) || (retval == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Set the default pen number for subsequent <code>UseTag</code> instances. */
  public void setUseTagPen(int pPen) { StyleTag.TagKeys.USE_TAG.setDefaultPen(pPen); }
  /** Set the default <code>StylePen</code> for subsequent <code>UseTag</code> instances. */
  public void setUseTagStyle(StylePen pPen) { StyleTag.TagKeys.USE_TAG.setDefaultStyle(pPen); }
  /** Set the default transforms for subsequent <code>UseTag</code>s. */
  public void setUseTagTransform(TransformPen pPen) { StyleTag.TagKeys.USE_TAG.setDefaultTransforms(pPen); }

  /** Fetch a default <code>PathTag</code>. */
  public PathTag getPathTag() {
    PathTag retval = new PathTag();
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }
  /** Set the default pen number for subsequent <code>PathTag</code> instances. */
  public void setPathPen(int pPen) { StyleTag.TagKeys.PATH_TAG.setDefaultPen(pPen); }
  /** Set the default <code>StylePen</code> for subsequent <code>PathTag</code> instances. */
  public void setPathStyle(StylePen pPen) { StyleTag.TagKeys.PATH_TAG.setDefaultStyle(pPen); }
  /** Set the default transforms for subsequent <code>PathTag</code>s. */
  public void setPathTransform(TransformPen pPen) { StyleTag.TagKeys.PATH_TAG.setDefaultTransforms(pPen); }

  /** Create a &lt;clipPath&gt; tag with the ID given. The ID <b>must not</b> be blank or null otherwise the return value will be null. */
  public ClipTag getClipTag(String pID) {
    ClipTag retval = ClipTag.newInstance(pID);
    if(!((myPizza == null) || (retval == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }

  /** Create a &lt;mask&gt; tag with the ID given. If the ID is blank or null the return value will be null. */
  public MaskTag getMaskTag(String pID) {
    MaskTag retval = MaskTag.newInstance(pID);
    if(!((myPizza == null) || (retval == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }

  /** Create a &lt;marker&gt; tag. If the ID is null or blank the return value will be null. */
  public MarkerTag getMarkerTag(String pID) {
    MarkerTag retval = MarkerTag.newInstance(pID);
    if(!((myPizza == null) || (retval == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }

  /** Create a &lt;pattern&gt; tag. If the ID is null or blank the return value will be null. */
  public PatternTag getPatternTag(String pID) {
    PatternTag retval = PatternTag.newInstance(pID);
    if(!((myPizza == null) || (retval == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }

  /** Create a group tag ('&lt;g&gt;'). A non-null, non-blank ID is mandatory. */
  public GroupTag getGroupTag(String pID) {
    GroupTag retval = GroupTag.newInstance(pID);
    if(!((myPizza == null) || (retval == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }

  /** Create a &lt;symbol&gt;') tag. Must have a non-null, non-blank ID. */
  public SymbolTag getSymbolTag(String pID) {
    SymbolTag retval = SymbolTag.newInstance(pID);
    if(!((myPizza == null) || (retval == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }

  /** Create and return a standard (linear) <code>GradientTag</code>. */
  public GradientTag getLinearGradient() {
    GradientTag retval = new GradientTag();
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }

  /** Create and return a radial <code>GradientTag</code>. */
  public GradientTag getRadialGradient() {
    GradientTag retval = new GradientTag().setAsRadial(true);
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }

  /** Create a standard, no-frills root &lt;svg&gt; tag, ready for use. Must have valid width and height. */
  public SVGTag getSVGTag(String pWidth, String pHeight) {
    SVGTag retval = SVGTag.newInstance(pWidth, pHeight);
    if(!((myPizza == null) || (retval == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }

  /** Create a letter-sized SVG root tag with the viewport set for 100 units per inch in both directions. */
  public SVGTag getLetter() {
    SVGTag retval = SVGTag.newInstance("8.5in", "11in");
    retval.setViewBox("0 0 850 1100");
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configureTag(myPrefix, myPizza);
    return retval;
  }

  /** Create a plain StylePen, unnamed (unnumbered). */
  public StylePen getStylePen() {
    StylePen retval = new StylePen();
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configurate(myPizza);
    return retval;
  }
  /** Create a new numbered StylePen. */
  public StylePen getStylePen(int pNum) {
    StylePen retval = new StylePen().setPen(pNum);
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configurate(myPizza);
    return retval;
  }

  /** Create a plain unnamed MarkerPen. */
  public MarkerPen getMarkerPen() {
    MarkerPen retval = new MarkerPen();
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configurate(myPizza);
    return retval;
  }
  /** Create a new numbered MarkerPen. */
  public MarkerPen getMarkerPen(int pNum) {
    MarkerPen retval = ((MarkerPen) new MarkerPen().setPen(pNum));
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configurate(myPizza);
    return retval;
  }

  /** Create a plain unnamed TextPen. */
  public TextPen getTextPen() {
    TextPen retval = new TextPen();
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configurate(myPizza);
    return retval;
  }
  /** Create a new numbered TextPen. */
  public TextPen getTextPen(int pNum) {
    TextPen retval = ((TextPen) new TextPen().setPen(pNum));
    if(!((myPizza == null) || myPizza.isEmpty())) retval.configurate(myPizza);
    return retval;
  }

  /** Create a TransformPen. */
  public TransformPen getTransformPen() { return new TransformPen(); }

}

