package cox.jmatt.java.MathTools.markup.svg;

import cox.jmatt.java.MathTools.markup.*;

/** <p>This class provides the &lt;pattern&gt; tag. Add one or more &lt;path&gt; tags to establish the pattern, the add this to the ('defs' block of the) root 
    tag. To apply the pattern to a graphic and reference it by local URL ('id=' attribute)</p>
*/
public class PatternTag extends DrawableContainerTag {

  private String myID = null;

  private PatternTag() {
    super(StyleTag.TagKeys.PATTERN_TAG);
    _addAttribute("patternTransform", StyleTag.TagKeys.PATTERN_TAG.getDefaultTransforms());
    _addAttribute("transform", null);
  }

  /** This is the only way to construct an instance of this class. If the ID argument is null or blank the return value is null! */
  public static PatternTag newInstance(String pTagID) {
    if(!validString(pTagID)) return null;
    PatternTag retval = new PatternTag();
    retval.setID(pTagID);
    return retval;
  }

  /** Overridden to ensure the ID set is valid. */
  public void setID(String pID) {
    if(!validString(pID)) return;
    myID = pID;
    super.setID(pID);
  }

  /** Overridden to preserve the ID. To change it use <code>reset(String)</code> or re-set manually afterward. */
  public void reset() {
    super.reset();
    setID(myID);
  }

  /** Call this method to clear data <b>and</b> re-set the ID in one fell swoop. */
  public void reset(String newID) {
    super.reset();
    setID(newID);
  }

  /** Set the pattern units specifier. This is 'objectBoundingBox' or 'userSpaceOnUse'.
      @param pUserSpace true for 'userSpaceOnUse', false for 'objectBoundingBox'.
      @see DrawableContainerTag#_setOBUS
  */
  public PatternTag setPatternUnits(boolean pUserSpace) {
    _setOBUS("patternUnits", pUserSpace);
    return this;
  }

  /** Set the pattern content unit specifier. This is 'objectBoundingBox' or 'userSpaceOnUse'.
      @param pUserSpace true for 'userSpaceOnUse', false for 'objectBoundingBox'.
      @see DrawableContainerTag#_setOBUS
  */
  public PatternTag setPatternContentUnits(boolean pUserSpace) {
    _setOBUS("patternContentUnits", pUserSpace);
    return this;
  }

  /** Set the dimensions of this tag. It should be percent values or decimals between 0 and 1 for 'objectBoundingBox', or user units for 'userSpaceOnUse'.
      @param pX The X-coordinate of the pattern's starting point.
      @param pY The Y-coordinate of the pattern's initial point.
      @param pWidth The pattern width.
      @param pHeight The pattern height.
  */
  public PatternTag setSize(Double pX, Double pY, Double pWidth, Double pHeight) {
    _addNAttribute("x", pX); _addNAttribute("y", pY);
    _addNAttribute("width", pWidth); _addNAttribute("height", pHeight);
    return this;
  }

  /** Overridden to provide class-specific adjustments. */
  public void setTransform(TransformPen pPen) {
    if(pPen == null) {
      _addAttribute("patternTransform", null);
    } else {
      _addAttribute("patternTransform", pPen.toString(false));
    }
  }

}

