package cox.jmatt.java.MathTools.markup.svg;

import cox.jmatt.java.MathTools.markup.*;

/** <p>This class provides both the &lt;poly<u>gon</u>&gt; and &lt;poly<u>line</u>&gt: tags. The difference is in closure: the poly<u>gon</u> automatically 
    connects the last point added to the first; the poly<u>line</u> does not. Apart from that there is no difference between the two tags.</p>
*/
public class PolygonTag extends StyleTag {

  private StringBuffer myPoints = new StringBuffer();

  /** Only one constructor. No more are needed. */
  public PolygonTag() { super(StyleTag.TagKeys.POLYGON_TAG); }

  /** Determine whether this tag is a poly<b>gon</b> or poly<b>line</b>. The <code>reset()</code> method does not affect this! Returns a self-reference to allow 
      <code>new PolygonTag().setPolygon(false)</code> to generate a poly<u>line</u> in one statement.
      @param isClosed true for a polygon, false for a polyline.
  */
  public PolygonTag setPolygon(boolean isClosed) {
    myTagName = (isClosed)?"polygon":"polyline";
    return this;
  }

  /** Add a vertex (point) to the current poly<i>thing</i>. */
  public PolygonTag addVertex(double vX, double vY) {
    myPoints.append(vX).append(" ").append(vY).append(" ");
    return this;
  }

  /** Add a String of space-separated points directly. Returns a self-reference for method chaining. */
  public PolygonTag addPoints(String pPoints) {
    if(Tag.validString(pPoints)) myPoints.append(pPoints.trim()).append(" ");
    return this;
  }

  /** Overridden to clear the internal point data also. */
  public void reset() {
    super.reset();
    myPoints = new StringBuffer();
  }

  /** Copy this class. */
  public PolygonTag copyOf() {
    boolean isPolygon = "polygon".equals(myTagName);
    _enableCopy();
    PolygonTag retval = new PolygonTag();
    retval.setPolygon(isPolygon);
    return retval;
  }

  /** Overridden to add the points at the last possible instant. */
  public String toString() {
    if(myPoints.length() > 0) _addAttribute("points", myPoints.toString().trim());
    return super.toString();
  }

}

