package cox.jmatt.java.MathTools.markup.svg;

import cox.jmatt.java.MathTools.markup.*;

/** <p>This class provides the SVG &lt;rect&gt; tag: rectangles. Equipment is provided for both sharp-cornered and rounded rectangles. If the corner radii are set 
    they remain in effect until cleared via the <code>reset()</code> method. All coordinates are in user units.</p>
*/
public class RectangleTag extends StyleTag {

  /** Standard constructor. No data set. */
  public RectangleTag() { super(TagKeys.RECTANGLE_TAG); }

  /** Create a square RectangleTag with all data present. */
  public RectangleTag(Double X1, Double Y1, Double pWidth, Double pHeight) {
    super(TagKeys.RECTANGLE_TAG);
    setData(X1, Y1, pWidth, pHeight, null, null);
  }

  /** Create a <i>rounded</i> RectangleTag with all data. */
  public RectangleTag(Double X1, Double Y1, Double pWidth, Double pHeight, Double radX, Double radY) {
    super(TagKeys.RECTANGLE_TAG);
    setData(X1, Y1, pWidth, pHeight, radX, radY);
  }

  /** (Re-)Set the corner coordinates, width, and height. This does not alter corner radii. */
  public RectangleTag setData(Double X1, Double Y1, Double pWidth, Double pHeight) {
    return setData(X1, Y1, pWidth, pHeight, null, null);
  }

  /** <p>Set or reconfigure all rectangle data. This method contains all possible data, so all argument descriptions are here. If the values are null they are 
      ignored, so the only way to clear them is the <code>reset()</code> method. If the radii are specified they remain in effect until cleared, and if they 
      <i>are</i> specified the rectangles generated will be rounded at the corners.
      @param X1 The X-coordinate of the rectangle center.
      @param Y1 The Y-coordinate of the rectangle center.
      @param pWidth The width (horizontal size) of the rectangle.
      @param pHeight The height (vertical size) of the rectangle.
      @param radX The corner horizontal radius for a rounded rectangle.
      @param radY The corner vertical radius for a rounded rectangle.
  */
  public RectangleTag setData(Double X1, Double Y1, Double pWidth, Double pHeight, Double radX, Double radY) {
    _addNAttribute("x", X1); _addNAttribute("y", Y1);
    _addNAttribute("width", pWidth); _addNAttribute("height", pHeight);
    _addNAttribute("rx", radX); _addNAttribute("ry", radY);
    return this;
  }

  /** Copy this class. */
  public RectangleTag copyOf() {
    _enableCopy();
    return new RectangleTag();
  }

}

