package cox.jmatt.java.MathTools.markup.svg;

import cox.jmatt.java.MathTools.markup.*;

/** <p>This class encapsulates the &lt;symbol&gt; tag. </p>
*/
public class SymbolTag extends GroupTag {

  private String myID = null;

  /** Constructor provided for subclasses. */
  protected SymbolTag(StyleTag.TagKeys pKey) { super(pKey); }

  private SymbolTag() { super(StyleTag.TagKeys.SYMBOL_TAG); }

  /** This is the only way to construct an instance of this class. If the ID argument is null or blank the return value is null! */
  public static SymbolTag newInstance(String pTagID) {
    if(!validString(pTagID)) return null;
    SymbolTag retval = new SymbolTag();
    retval.setID(pTagID);
    return retval;
  }

  /** Overridden to ensure the ID set is valid. */
  public void setID(String pID) {
    if(!validString(pID)) return;
    myID = pID;
    super.setID(pID);
  }

  /** Overridden to preserve the ID. To change it use <code>reset(String)</code> or re-set manually afterward. */
  public void reset() {
    super.reset();
    setID(myID);
  }

  /** Call this method to clear data <b>and</b> re-set the ID in one fell swoop. */
  public void reset(String newID) {
    super.reset();
    setID(newID);
  }

  /** Set aspect ratio preservation for this symbol.
      @see DrawableContainerTag#_setPAR
  */
  public SymbolTag setPAR(String par) { _setPAR(par); return this; }

  /** Set the 'viewBox=' attribute. This should be four space-separated numbers given as a single String.
      @see DrawableContainerTag#_setViewBox
  */
  public SymbolTag setViewBox(String pVBox) { _setViewBox(pVBox); return this;}

}

