package cox.jmatt.java.MathTools.markup.svg;

import cox.jmatt.java.MathTools.markup.*;

/** <p>The <code>TSpanTag</code> class encapsulates the &lt;tspan&gt; tag, which is basically a &lt;text&gt; tag within another &lt;text&gt; tag. This is a very 
    loose analogy but good enough for MathTools' purposes. In function this tag allows some text within a &lt;text&gt; tag to be styled differently from the 
    rest. To that end it provides three methods and two constructors for the extra goodies the &lt;tspan&gt; tag possesses.</p>
    <p>Although this class inherits set-default methods from <code>TextTag</code> for style and transforms these should not be used! Each <code>TSpanTag</code> 
    should set these options individually! Or, when possible, rely on those set on the enclosing tag.</p>
*/
public class TSpanTag extends TextTag {

  /** Build a blank tag. */
  public TSpanTag() { super(StyleTag.TagKeys.TSPAN_TAG); }

  /** Build a tag with text in-place. */
  public TSpanTag(String pText) {
    super(StyleTag.TagKeys.TSPAN_TAG);
    super.addText(pText);
  }

  /** Copy this class. */
  public TSpanTag copyOf() {
    _enableCopy();
    return new TSpanTag();
  }

  /** Set the &lt;tspan&gt; 'dx=' and 'dy=' attributes. Null or blank values are ignored. A self-reference is returned for method chaining. */
  public TSpanTag setDelta(Double dx, Double dy) {
    _addNAttribute("dx", dx);
    _addNAttribute("dy", dy);
    return this;
  }

  /** This is used for super- and sub-scripting text. Although it can appear in the style attribute it is treated here as structure rather than presentation. */
  public TSpanTag setBaselineShift(Double pShift) {
    _addNAttribute("baseline-shift", pShift);
    return this;
  }

  /** This sets the 'rotate=' <b>attribute</b> for this tag. The value is an angle in degrees. Setting null clears the attribute and disables it.*/
  public TSpanTag setRotate(Double pDegrees) {
    _addNAttribute("rotate", pDegrees);
    return this;
  }

}

