package cox.jmatt.java.MathTools.markup.svg;

import cox.jmatt.java.MathTools.markup.*;

/** <p>This class implements the SVG &lt;use&gt; tag. It has no public constructors because the 'HREF=' attribute is <b>mandatory</b>. The purpose of this tag is 
    to allow something to be defined once and used multiple times, perhaps with multiple different styles and transformations. This class simplifies that process. 
    Since the HREF is mandatory it is preserved when <code>reset()</code> is called. This allows a <code>UseTag</code> to be defined around a particular reference 
    then styled and transformed, reset, then re-styled and re-transformed, all without losing the reference.</p>
*/
public class UseTag extends StyleTag {

  private UseTag(String HREF) {
    super(StyleTag.TagKeys.USE_TAG);
    _addAttribute("xlink:href", HREF);
  }

  /** Static factory method for constructing these tags. If 'HREF' is null or blank the return value is null. */
  public static final UseTag newInstance(String HREF) {
    if(!Tag.validString(HREF)) return null;
    return new UseTag(HREF);
  }

  /** Change the tag's 'href=' attribute. Null or blank values are ignored. */
  public UseTag setHREF(String HREF) {
    if(Tag.validString(HREF)) _addAttribute("xlink:href", HREF);
    return this;
  }

  /** Overridden to preserve the HREF even when all other data is cleared. */
  public void reset() {
    String HREF = _attributes().get("xlink:href");
    super.reset();
    setHREF(HREF);
  }

}

