package cox.jmatt.java.MathTools.test;

import cox.jmatt.java.MathTools.*;

/** This class exists to create specific <code>COFormatter</code> instances. The Math ClassLoader can be enabled or not and, as with similar classes, the 
    fully-qualified name of the class is required.
*/
public class COFormatterFactory {

  private ClassLoader myLoader = null;

  /** Scripting constructor. */
  public COFormatterFactory() { super(); }

  /** Enable/disable the Math ClassLoader. This method takes effect immediately, setting or clearing the Math Loader.
      @param pEnable true to use the Math ClassLoader, false to disable it.
  */
  public void setMathClassLoader(boolean pEnable) { myLoader = pEnable?CapCom.getMathClassLoader():null; }

  /** Create and return a <code>COFormatter</code>. The full classname must be supplied otherwise this method returns null. Any errors are reported at Error level 
      and the return value is null.
      @param pName The name of the COFormatter implementation to create.
      @return The proper instance or null if catastrophe strikes.
  */
  public COFormatter getFormatter(String pName) {
    COFormatter retval = null;
    if(!Question.validString(pName)) return retval;
    try {
      if(myLoader == null) {
        retval = (COFormatter) Class.forName(pName).newInstance();
      } else {
        retval = (COFormatter) Class.forName(pName, true, myLoader).newInstance();
      }
    } catch(Exception ex) { CapCom.error("Error creating COFormatter", ex); }
    return retval;
  }

  /** Static convenience method to create <code>COFormatter</code>s. If an error occurs or the class cannot be created null is returned.
      @param pName The name of the <code>COFormatter</code> class to create.
      @param pUseMTCL true to use the Math ClassLoader, false not to.
  */
  public static COFormatter createFormatter(String pName, boolean pUseMTCL) {
    COFormatter retval = null;
    if(!Question.validString(pName)) return retval;
    try {
      if(pUseMTCL) {
        retval = (COFormatter) Class.forName(pName, true, CapCom.getMathClassLoader()).newInstance();
      } else {
        retval = (COFormatter) Class.forName(pName).newInstance();
      }
    } catch(Exception ex) { CapCom.error("Error creating COFormatter: "+pName, ex); }
    return retval;
  }

}

