package cox.jmatt.java.MathTools.test;

/** This interface describes anything that might be included on a math test. Questions are MTestElements, obviously, but most tests also contain such things as 
    titles, class name/number, section number, etc. This interface makes provision for all of that, or for anything else that a test might need. Four formatting 
    methods are specified plus XML: three for the MTestElement's internal format and one that accepts a String parameter. The 'internal format' methods make 
    no assumptions about the internal formulas, merely that they exist.
*/
public interface MTestElement {

  /** Format MTestElement content. This method makes no requirement other than that it exists. It assumes that the nature of the format will be specified internally 
      by the MTestElement being formatted.
  */
  public String format();

  /** Format MTestElement content according to the format specified. The format String is exactly per the Question.format(String) formatting string with identical 
      replacement tokens. This method is used by MTest to separate the Problem and Answer parts when they are printed separately with custom formats.
  */
  public String format(String pFormat);

  /** Format MTestElement Problems according to their internal Problem format. */
  public String formatProblem();

  /** Format the Answer content according to the internal Answer format. */
  public String formatAnswer();

  /** Output the MTestElement as XML of some description. Again, this method assumes that the MTestElement is able to obey this constraint. Each MTestElement is 
      responsible for assuring the XML used is well-formed and that any dangerous characters are replaced by their XML escape codes.
  */
  public String toXML();

  /** This method is used in adding elements to a <code>MTest</code>. This is done so that changes to the original object do not affect elements already added. */
  public MTestElement copyOf();

}
