package cox.jmatt.java.MathTools.util;

import javax.xml.transform.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import cox.jmatt.java.MathTools.*;

/** This class extends <code>org.xml.sax.helpers.DefaultHandler</code> and implements <code>javax.xml.transform.ErrorListener</code> Its sole reason for existence 
    is to report XML errors. All XML Warn- and Error- level events are reported at DEBUG level and Fatal-level is reported at ERROR. No other methods are defined 
    so this class can be used to parse an XML document to check for errors.
*/
public class MTXErrorHandler extends DefaultHandler implements ErrorListener {

  /** Default constructor. Does nothing special. */
  public MTXErrorHandler() { super(); }

  /** Report parser warning at DEBUG level. */
  public void warning(SAXParseException ex) { CapCom.debug("SAX Warning", ex); }

  /** Report a recoverable parser error at DEBUG level. */
  public void error(SAXParseException ex) { CapCom.debug("SAX Error", ex); }

  /** Report a fatal error at ERROR level. */
  public void fatalError(SAXParseException ex) { CapCom.error("SAX Fatal!", ex); }

  /** Report transform warning at DEBUG level. */
  public void warning(TransformerException ex) { CapCom.debug("Transformer Warning", ex); }

  /** Report a recoverable transform error at DEBUG level. */
  public void error(TransformerException ex) { CapCom.debug("Transformer Error", ex); }

  /** Report a fatal transform error at ERROR level. */
  public void fatalError(TransformerException ex) { CapCom.error("Transformer Fatal!", ex); }

  /** Does nothing but print a message at Debug level. */
  public void startDocument() { CapCom.debug("Starting document..."); }

  /** Prints a Debug-level message. */
  public void endDocument() { CapCom.debug("Document parsed."); }

}
