package cox.jmatt.java.MathTools.util;

/** <p>This is a generic interface for a MathTools accessory class. A <code>MathToolsAccessory</code> is an add-on or plugin class usable from within the MathTools 
    packages. <code>CapCom</code> has methods for creating accessories, reflected in <code>MathEngine</code>. Exactly what these classes are depends on what they 
    are. That is to say this interface, much like <code>GraphKeys</code>, provides a way to generically instantiate and make available classes completely outside 
    the MathTools packages.</p>
    <p>Accessories are created via {@link cox.jmatt.java.MathTools.CapCom#createAccessory CapCom.createAccessory} and require the fully-qualified class name. The 
    Math ClassLoader is used automatically and transparently so the accessory class MUST be visible to it. <code>MathEngine</code> access means that any script, 
    even one from <code>MTApplet</code>, so the accessory must be written accordingly. Accessories can be globally disabled (default) or enabled via the 
    <code>CapCom.allowAccessories</code> field but this is all or nothing. <code>ScriptPanel</code> automatically enables them but custom applications using 
    MathTools must do this manually.</p>
    <h3>So why not just use The Shelf?</h3>
    <p>Accessories provide a way to 'pull in' arbitrary classes without requiring an entire re-write of <code>ScriptPanel</code> or a totally new custom application 
    just to include one or two extra classes. That's not to say it <i>can't</i> be done, just that it doesn't <i>have</i> to be done. The Shelf is specifically 
    for use with custom MathTools-using Java applications. Accessories simply provide a simpler route.</p>
    <h2>Other Requirements</h2>
    <p>Other than implementing this interface an accessory only needs a zero-argument constructor. <code>CapCom</code> creates accessories using the 
    <code>Class.newInstance()</code> method so no parameters are passed in. There is also no intiialization performed other than that, so construction must handle 
    any such.</p>
*/
public interface MathToolsAccessory {

  /** <p>This is the only method mandated by this interface. Much like the <code>MathPrinter processMathObject()</code> method this one makes no requirement other 
      than existence. It is as broad and general as possible to allow for maximum flexibiity in scripting or Java applications where only the interface methods 
      are available.</p>
      <p>The String command key should be used to select what actions are performed and the results of those actions should be well-documented. The 
      <code>Object...</code> argument, if used, should also be well-explained as should the return values to expect.</p>
      @param pCommand The String command to process.
      @param pArgs The arguments required for processing.
      @return Whatever is appropriate or necessary.
  */
  public Object processCommand(String pCommand, Object... pArgs);

}

