package cox.jmatt.java.MathTools.util;

import java.awt.*;
import java.awt.event.*;
import cox.jmatt.java.MathTools.MathEngine;

/** This class presents a quick and easy GUI for the MathEngine parseRPN() method. It is available in the Demo, but it is also available to other classes that 
    have need of a quick calculator. QuickRPNCalculator provides a TextField to enter calculations, a TextArea to display the results, a Checkbox to enable 
    floating-point mode, a button to clear the TextArea and a [Mem] button to recall the last calculation entered. There should be no need to sublcass; 
    simply add QRPNCalc as-is!
*/
public class QuickRPNCalculator extends Panel implements ActionListener {

  private Checkbox cbxFloat;
  private TextArea txaDisplay;
  private TextField txfCalcBar;
  private Button btnGo;
  private Button btnClear;
  private Button btnHelp;
   private Button btnMem;
  private String sCalc;
  
  public QuickRPNCalculator() {
    super();
    cbxFloat = new Checkbox("Float");
    txaDisplay = new TextArea(10, 40);
    txfCalcBar = new TextField();
    btnGo = new Button("Calc");
    btnClear = new Button("Clear");
    btnMem = new Button("Mem");
    btnHelp = new Button(" ? ");
    setLayout(new BorderLayout());
    add(new Label("Quick RPN Calculator", Label.CENTER), BorderLayout.NORTH);
    add(txaDisplay, BorderLayout.CENTER);
    Panel pnlTemp = new Panel(new GridLayout(0,1));
    btnHelp.addActionListener(this);
    pnlTemp.add(btnHelp);
    btnClear.addActionListener(this);
    pnlTemp.add(btnClear);
    pnlTemp.add(cbxFloat);
    btnMem.addActionListener(this);
    pnlTemp.add(btnMem);
    btnGo.addActionListener(this);
    pnlTemp.add(btnGo);
    add(pnlTemp, BorderLayout.EAST);
    txfCalcBar.addActionListener(this);
    pnlTemp = new Panel(new BorderLayout());
    pnlTemp.add(new Label("> > >"), BorderLayout.WEST);
    pnlTemp.add(new Label("< < <"), BorderLayout.EAST);
    pnlTemp.add(txfCalcBar, BorderLayout.CENTER);
    add(pnlTemp, BorderLayout.SOUTH);
    sCalc = "";
  }
  /** Pull the String from the TextField calculation bar, feed it to MathEngine.parseRPN(), and dump the result in the TextArea. */
  public void calculate() {
    sCalc = ""+txfCalcBar.getText();
    if(sCalc.indexOf(".") > -1) cbxFloat.setState(true);
    txfCalcBar.setText("");
    txaDisplay.append(">> "+sCalc+"\n"+MathEngine.parseRPN((cbxFloat.getState()?". ":"")+sCalc)+"\n");
  }
  /** Display the help content on the TextArea. */
  public void showHelp() {
    txaDisplay.append("Calculator Help\nThe calculation entry field is below this display.\nEnter a calculation in RPN format:\n");
    txaDisplay.append("For example: '2+3*4' would be entered as '2 3 4 * +'.\n\n[Calc] performs the calculation.\n[Clear] clears the display\n");
    txaDisplay.append("[Mem] recalls the last calculation entered into the calc field.\n[]Float enables floating-point mode.\n");
  }
  /** Clear the calculator display. */
  public void clearDisplay() { txaDisplay.setText(""); }

  /** Does nothing more than respond to button clicks or [Enter] in the TextField. */
  public void actionPerformed(ActionEvent aev) {
    if(aev.getSource() == btnClear) {
      clearDisplay();
    } else if(aev.getSource() == btnHelp) {
      showHelp();
    } else if(aev.getSource() == btnMem) {
      txfCalcBar.setText(sCalc);
    } else {
      calculate();
    }
  }
}
